@@ -1,15 +1,15 @@
 /*
  * Sleuth Kit Data Model
- * 
- * Copyright 2011 Basis Technology Corp.
+ *
+ * Copyright 2011-2016 Basis Technology Corp.
  * Contact: carrier <at> sleuthkit <dot> org
- * 
+ *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *     http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -19,6 +19,7 @@
 package org.sleuthkit.datamodel;
 
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.List;
 import java.util.Set;
 
@@ -35,12 +36,14 @@ public interface Content extends SleuthkitVisitableItem {
 	 * Reads data that this content object is associated with (file contents,
 	 * volume contents, etc.).
 	 *
-	 * @param buf a character array of data (in bytes) to copy read data to
+	 * @param buf    a character array of data (in bytes) to copy read data to
 	 * @param offset byte offset in the content to start reading from
-	 * @param len number of bytes to read into buf.
+	 * @param len    number of bytes to read into buf.
+	 *
 	 * @return num of bytes read, or -1 on error
+	 *
 	 * @throws TskCoreException if critical error occurred during read in the
-	 * tsk core
+	 *                          tsk core
 	 */
 	public int read(byte[] buf, long offset, long len) throws TskCoreException;
 
@@ -56,7 +59,7 @@ public interface Content extends SleuthkitVisitableItem {
 	 * you should be able to read from it. In some cases, data corruption may
 	 * mean that you cannot read this much data.
 	 *
-	 * @return size of the content
+	 * @return size of the content in bytes
 	 */
 	public long getSize();
 
@@ -64,6 +67,7 @@ public interface Content extends SleuthkitVisitableItem {
 	 * Visitor pattern support
 	 *
 	 * @param v visitor supplying an algorithm to run on the content object
+	 *
 	 * @return visitor return value resulting from running the algorithm
 	 */
 	public <T> T accept(ContentVisitor<T> v);
@@ -77,8 +81,8 @@ public interface Content extends SleuthkitVisitableItem {
 
 	/**
 	 * @return returns the full path to this Content object starting with a "/"
-	 * followed by the Image name and similarly for all other segments in the
-	 * hierarchy.
+	 *         followed by the Image name and similarly for all other segments
+	 *         in the hierarchy.
 	 */
 	public String getUniquePath() throws TskCoreException;
 
@@ -91,9 +95,11 @@ public interface Content extends SleuthkitVisitableItem {
 	public long getId();
 
 	/**
-	 * Get the root data source of this content (image, virtual directory, etc.)
+	 * Gets the root data source (image, virtual directory, etc.) of this
+	 * content.
 	 *
 	 * @return Content associated with data source or null if one can't be found
+	 *
 	 * @throws TskCoreException if critical error occurred within tsk core
 	 */
 	public Content getDataSource() throws TskCoreException;
@@ -102,6 +108,7 @@ public interface Content extends SleuthkitVisitableItem {
 	 * Gets the child content objects of this content.
 	 *
 	 * @return List of children
+	 *
 	 * @throws TskCoreException if critical error occurred within tsk core
 	 */
 	public List<Content> getChildren() throws TskCoreException;
@@ -111,6 +118,7 @@ public interface Content extends SleuthkitVisitableItem {
 	 * should be more efficient than getting children and checking it empty.
 	 *
 	 * @return true if has children, false otherwise.
+	 *
 	 * @throws TskCoreException if critical error occurred within tsk core
 	 */
 	public boolean hasChildren() throws TskCoreException;
@@ -120,13 +128,15 @@ public interface Content extends SleuthkitVisitableItem {
 	 * than getting children and counting them.
 	 *
 	 * @return children count
+	 *
 	 * @throws TskCoreException if critical error occurred within tsk core
 	 */
 	public int getChildrenCount() throws TskCoreException;
 
 	/**
 	 * @return returns the parent of this Content object or null if there isn't
-	 * one as is the case for Image.
+	 *         one as is the case for Image.
+	 *
 	 * @throws TskCoreException
 	 */
 	public Content getParent() throws TskCoreException;
@@ -135,6 +145,7 @@ public interface Content extends SleuthkitVisitableItem {
 	 * Gets the child content ids of this content.
 	 *
 	 * @return List of children ids
+	 *
 	 * @throws TskCoreException if critical error occurred within tsk core
 	 */
 	public List<Long> getChildrenIds() throws TskCoreException;
@@ -143,38 +154,158 @@ public interface Content extends SleuthkitVisitableItem {
 	 * Create and add an artifact associated with this content to the blackboard
 	 *
 	 * @param artifactTypeID id of the artifact type (if the id doesn't already
-	 * exist an exception will be thrown)
+	 *                       exist an exception will be thrown)
+	 *
 	 * @return the blackboard artifact created (the artifact type id can be
-	 * looked up from this)
+	 *         looked up from this)
+	 *
 	 * @throws TskCoreException if critical error occurred within tsk core
+	 * @deprecated Please use newDataArtifact or newAnalysisResult.
 	 */
+	@Deprecated
 	public BlackboardArtifact newArtifact(int artifactTypeID) throws TskCoreException;
 
 	/**
 	 * Create and add an artifact associated with this content to the blackboard
 	 *
 	 * @param type artifact enum tyoe
+	 *
 	 * @return the blackboard artifact created (the artifact type id can be
-	 * looked up from this)
+	 *         looked up from this)
+	 *
 	 * @throws TskCoreException if critical error occurred within tsk core
+	 * @deprecated Please use newDataArtifact or newAnalysisResult.
 	 */
+	@Deprecated
 	public BlackboardArtifact newArtifact(BlackboardArtifact.ARTIFACT_TYPE type) throws TskCoreException;
 
+	/**
+	 * Create and add an analysis result associated with this content.
+	 *
+	 *
+	 * @param artifactType	  Type of analysis result artifact to create.
+	 * @param score          Score associated with this analysis.
+	 * @param conclusion     Conclusion from the analysis, may be empty.
+	 * @param configuration  Configuration element associated with this
+	 *                       analysis, may be empty.
+	 * @param justification	 Justification
+	 * @param attributesList Additional attributes to attach to this analysis
+	 *                       result artifact.
+	 *
+	 * @return AnalysisResultAdded The analysis return added and the current
+	 *         aggregate score of content.
+	 *
+	 * @throws TskCoreException if critical error occurred within tsk core.
+	 */
+	public AnalysisResultAdded newAnalysisResult(BlackboardArtifact.Type artifactType, Score score, String conclusion, String configuration, String justification, Collection<BlackboardAttribute> attributesList) throws TskCoreException;
+
+	/**
+	 * Create and add an analysis result associated with this content.
+	 *
+	 *
+	 * @param artifactType	  Type of analysis result artifact to create.
+	 * @param score          Score associated with this analysis.
+	 * @param conclusion     Conclusion from the analysis, may be empty.
+	 * @param configuration  Configuration element associated with this
+	 *                       analysis, may be empty.
+	 * @param justification	 Justification
+	 * @param attributesList Additional attributes to attach to this analysis
+	 *                       result artifact.
+	 * @param dataSourceId   The data source for the analysis result
+	 *
+	 * @return AnalysisResultAdded The analysis return added and the current
+	 *         aggregate score of content.
+	 *
+	 * @throws TskCoreException if critical error occurred within tsk core.
+	 */
+	public AnalysisResultAdded newAnalysisResult(BlackboardArtifact.Type artifactType, Score score, String conclusion, String configuration, String justification, Collection<BlackboardAttribute> attributesList, long dataSourceId) throws TskCoreException;
+
+	/**
+	 * Create and add a data artifact associated with this abstract file. This
+	 * method creates the data artifact with the os account id associated with
+	 * this abstract file if one exists.
+	 *
+	 * @param artifactType   Type of data artifact to create.
+	 * @param attributesList Additional attributes to attach to this data
+	 *                       artifact.
+	 *
+	 * @return DataArtifact New data artifact.
+	 *
+	 * @throws TskCoreException If a critical error occurred within tsk core.
+	 */
+	public DataArtifact newDataArtifact(BlackboardArtifact.Type artifactType, Collection<BlackboardAttribute> attributesList) throws TskCoreException;
+	
+	
+	/**
+	 * Create and add a data artifact associated with this content.
+	 *
+	 * @param artifactType   Type of analysis result artifact to create.
+	 * @param attributesList Additional attributes to attach to this data
+	 *                       artifact.
+	 * @param osAccountId    The OS account id associated with the artifact. May
+	 *                       be null.
+	 *
+	 * @return DataArtifact New data artifact.
+	 *
+	 * @throws TskCoreException If a critical error occurred within tsk core.
+	 */
+	public DataArtifact newDataArtifact(BlackboardArtifact.Type artifactType, Collection<BlackboardAttribute> attributesList, Long osAccountId) throws TskCoreException;
+
+	/**
+	 * Create and add a data artifact associated with this content.
+	 *
+	 * @param artifactType   Type of analysis result artifact to create.
+	 * @param attributesList Additional attributes to attach to this data
+	 *                       artifact.
+	 * @param osAccountId    The OS account id associated with the artifact. May
+	 *                       be null.
+	 * @param dataSourceId   The data source id of the artifact
+	 *
+	 * @return DataArtifact New data artifact.
+	 *
+	 * @throws TskCoreException If a critical error occurred within tsk core.
+	 */
+	public DataArtifact newDataArtifact(BlackboardArtifact.Type artifactType, Collection<BlackboardAttribute> attributesList, Long osAccountId, long dataSourceId) throws TskCoreException;
+
+	/**
+	 * Returns the final score for the content object.
+	 *
+	 * @return Score.
+	 *
+	 * @throws TskCoreException if critical error occurred within tsk core.
+	 */
+	public Score getAggregateScore() throws TskCoreException;
+
 	/**
 	 * Get all artifacts associated with this content that have the given type
 	 * name
 	 *
 	 * @param artifactTypeName name of the type to look up
+	 *
 	 * @return a list of blackboard artifacts matching the type
+	 *
 	 * @throws TskCoreException if critical error occurred within tsk core
 	 */
 	public ArrayList<BlackboardArtifact> getArtifacts(String artifactTypeName) throws TskCoreException;
 
+	/**
+	 * Get all analysis results associated with this content, that have the
+	 * given type.
+	 *
+	 * @param artifactType Type to look up.
+	 *
+	 * @return A list of analysis result artifacts matching the type.
+	 *
+	 * @throws TskCoreException If critical error occurred within tsk core.
+	 */
+	public List<AnalysisResult> getAnalysisResults(BlackboardArtifact.Type artifactType) throws TskCoreException;
+
 	/**
 	 * Return the TSK_GEN_INFO artifact for the file so that individual
 	 * attributes can be added to it. Creates one if it does not already exist.
 	 *
 	 * @return Instance of the TSK_GEN_INFO artifact
+	 *
 	 * @throws TskCoreException
 	 */
 	public BlackboardArtifact getGenInfoArtifact() throws TskCoreException;
@@ -185,9 +316,11 @@ public interface Content extends SleuthkitVisitableItem {
 	 * on the create argument.
 	 *
 	 * @param create If true, an artifact will be created if it does not already
-	 * exist.
+	 *               exist.
+	 *
 	 * @return Instance of the TSK_GEN_INFO artifact or null if artifact does
-	 * not already exist and create was set to false
+	 *         not already exist and create was set to false
+	 *
 	 * @throws TskCoreException
 	 */
 	public BlackboardArtifact getGenInfoArtifact(boolean create) throws TskCoreException;
@@ -196,7 +329,8 @@ public interface Content extends SleuthkitVisitableItem {
 	 * Return attributes of a given type from TSK_GEN_INFO.
 	 *
 	 * @param attr_type Attribute type to find inside of the TSK_GEN_INFO
-	 * artifact.
+	 *                  artifact.
+	 *
 	 * @return Attributes
 	 */
 	public ArrayList<BlackboardAttribute> getGenInfoAttributes(BlackboardAttribute.ATTRIBUTE_TYPE attr_type) throws TskCoreException;
@@ -206,7 +340,9 @@ public interface Content extends SleuthkitVisitableItem {
 	 * id
 	 *
 	 * @param artifactTypeID type id to look up
+	 *
 	 * @return a list of blackboard artifacts matching the type
+	 *
 	 * @throws TskCoreException if critical error occurred within tsk core
 	 */
 	public ArrayList<BlackboardArtifact> getArtifacts(int artifactTypeID) throws TskCoreException;
@@ -215,7 +351,9 @@ public interface Content extends SleuthkitVisitableItem {
 	 * Get all artifacts associated with this content that have the given type
 	 *
 	 * @param type type to look up
+	 *
 	 * @return a list of blackboard artifacts matching the type
+	 *
 	 * @throws TskCoreException if critical error occurred within tsk core
 	 */
 	public ArrayList<BlackboardArtifact> getArtifacts(BlackboardArtifact.ARTIFACT_TYPE type) throws TskCoreException;
@@ -224,15 +362,35 @@ public interface Content extends SleuthkitVisitableItem {
 	 * Get all artifacts associated with this content
 	 *
 	 * @return a list of blackboard artifacts
+	 *
 	 * @throws TskCoreException if critical error occurred within tsk core
 	 */
 	public ArrayList<BlackboardArtifact> getAllArtifacts() throws TskCoreException;
 
+	/**
+	 * Get all data artifacts associated with this content.
+	 *
+	 * @return A list of data artifacts.
+	 *
+	 * @throws TskCoreException If critical error occurred within tsk core.
+	 */
+	public List<DataArtifact> getAllDataArtifacts() throws TskCoreException;
+		
+	/**
+	 * Get all analysis results associated with this content.
+	 *
+	 * @return A list of analysis results.
+	 *
+	 * @throws TskCoreException If critical error occurred within tsk core.
+	 */
+	public List<AnalysisResult> getAllAnalysisResults() throws TskCoreException;
+
 	/**
 	 * Get the names of all the hashsets that this content is in.
 	 *
 	 * @return the names of the hashsets that this content is in
-	 * @throwsTskCoreException if critical error occurred within tsk core
+	 *
+	 * @throws TskCoreException if critical error occurred within tsk core
 	 */
 	public Set<String> getHashSetNames() throws TskCoreException;
 
@@ -241,7 +399,9 @@ public interface Content extends SleuthkitVisitableItem {
 	 * given type name
 	 *
 	 * @param artifactTypeName name of the type to look up
+	 *
 	 * @return count of blackboard artifacts matching the type
+	 *
 	 * @throws TskCoreException if critical error occurred within tsk core
 	 */
 	public long getArtifactsCount(String artifactTypeName) throws TskCoreException;
@@ -251,7 +411,9 @@ public interface Content extends SleuthkitVisitableItem {
 	 * given type id
 	 *
 	 * @param artifactTypeID type id to look up
+	 *
 	 * @return count of blackboard artifacts matching the type
+	 *
 	 * @throws TskCoreException if critical error occurred within tsk core
 	 */
 	public long getArtifactsCount(int artifactTypeID) throws TskCoreException;
@@ -261,7 +423,9 @@ public interface Content extends SleuthkitVisitableItem {
 	 * given type
 	 *
 	 * @param type type to look up
+	 *
 	 * @return count of blackboard artifacts matching the type
+	 *
 	 * @throws TskCoreException if critical error occurred within tsk core
 	 */
 	public long getArtifactsCount(BlackboardArtifact.ARTIFACT_TYPE type) throws TskCoreException;
@@ -270,6 +434,7 @@ public interface Content extends SleuthkitVisitableItem {
 	 * Get count of all artifacts associated with this content
 	 *
 	 * @return count of all blackboard artifacts for this content
+	 *
 	 * @throws TskCoreException if critical error occurred within tsk core
 	 */
 	public long getAllArtifactsCount() throws TskCoreException;
