@@ -1,7 +1,7 @@
 /*
- * Autopsy Forensic Browser
+ * Sleuth Kit Data Model
  *
- * Copyright 2011-2013 Basis Technology Corp.
+ * Copyright 2011-2018 Basis Technology Corp.
  * Contact: carrier <at> sleuthkit <dot> org
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -20,18 +20,15 @@ package org.sleuthkit.datamodel;
 
 import java.io.IOException;
 import java.io.InputStream;
-import java.util.logging.Level;
-import java.util.logging.Logger;
 
 /**
  * InputStream to read bytes from a Content object's data
  */
-public class ReadContentInputStream extends InputStream {
+public final class ReadContentInputStream extends InputStream {
 
 	private long currentOffset;
-	private long contentSize;
-	private Content content;
-	private static final Logger logger = Logger.getLogger(ReadContentInputStream.class.getName());
+	private final long contentSize;
+	private final Content content;
 
 	public ReadContentInputStream(Content content) {
 		this.content = content;
@@ -40,18 +37,18 @@ public class ReadContentInputStream extends InputStream {
 	}
 
 	@Override
-	public int read() throws IOException {
+	public int read() throws ReadContentInputStreamException {
 		byte[] buff = new byte[1];
 		return (read(buff) != -1) ? buff[0] : -1;
 	}
 
 	@Override
-	public int read(byte[] b) throws IOException {
+	public int read(byte[] b) throws ReadContentInputStreamException {
 		return read(b, 0, b.length);
 	}
 
 	@Override
-	public int read(byte[] b, int off, int len) throws IOException {
+	public int read(byte[] b, int off, int len) throws ReadContentInputStreamException {
 
 		final int buffLen = b.length;
 		//must return 0 for zero-length arrays
@@ -80,7 +77,7 @@ public class ReadContentInputStream extends InputStream {
 		// is the buffer big enough?
 		lenToRead = Math.min(lenToRead, buffLen - off);
 
-		byte[] retBuf = null;
+		byte[] retBuf;
 		if (off == 0) {
 			//write directly to user buffer
 			retBuf = b;
@@ -105,10 +102,7 @@ public class ReadContentInputStream extends InputStream {
 				return lenRead;
 			}
 		} catch (TskCoreException ex) {
-			logger.log(Level.WARNING, ("Error reading content into stream: " //NON-NLS
-					+ content.getId()) + ": " + content.getName()
-					+ ", at offset " + currentOffset + ", length to read: " + lenToRead, ex); //NON-NLS
-			throw new IOException(ex);
+			throw new ReadContentInputStreamException(String.format("Error reading file '%s' (id=%d) at offset %d.", content.getName(), content.getId(), currentOffset), ex);
 		}
 
 	}
@@ -166,8 +160,9 @@ public class ReadContentInputStream extends InputStream {
 	 * Set new current position in the stream, up to and including EOF
 	 *
 	 * @param newPosition new position in the stream to be set
+	 *
 	 * @return the actual position set, which can be less than position passed
-	 * in if EOF has been reached
+	 *         in if EOF has been reached
 	 */
 	public long seek(long newPosition) {
 		if (newPosition < 0) {
@@ -178,4 +173,21 @@ public class ReadContentInputStream extends InputStream {
 		return currentOffset;
 
 	}
+
+	/**
+	 * Exception thrown when there's an error reading from the
+	 * ReadContentInputStream.
+	 */
+	public final static class ReadContentInputStreamException extends IOException {
+
+		private static final long serialVersionUID = 1L;
+
+		public ReadContentInputStreamException(String message) {
+			super(message);
+		}
+
+		public ReadContentInputStreamException(String message, Throwable cause) {
+			super(message, cause);
+		}
+	}
 }
