@@ -1,15 +1,15 @@
 /*
- * Autopsy Forensic Browser
- * 
- * Copyright 2011 Basis Technology Corp.
+ * Sleuth Kit Data Model
+ *
+ * Copyright 2011-2021 Basis Technology Corp.
  * Contact: carrier <at> sleuthkit <dot> org
- * 
+ *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *     http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -38,6 +38,7 @@ public interface SleuthkitItemVisitor<T> {
 	 * Act on (visit) a Directory content object
 	 *
 	 * @param d the directory to visit / act on
+	 *
 	 * @return result of the visit
 	 */
 	T visit(Directory d);
@@ -46,6 +47,7 @@ public interface SleuthkitItemVisitor<T> {
 	 * Act on (visit) a File content object
 	 *
 	 * @param f the file to visit / act on
+	 *
 	 * @return result of the visit
 	 */
 	T visit(File f);
@@ -54,6 +56,7 @@ public interface SleuthkitItemVisitor<T> {
 	 * Act on (visit) a FileSystem content object
 	 *
 	 * @param fs the filesystem to visit / act on
+	 *
 	 * @return result of the visit
 	 */
 	T visit(FileSystem fs);
@@ -62,6 +65,7 @@ public interface SleuthkitItemVisitor<T> {
 	 * Act on (visit) an Image content object
 	 *
 	 * @param i the image to visit / act on
+	 *
 	 * @return result of the visit
 	 */
 	T visit(Image i);
@@ -70,6 +74,7 @@ public interface SleuthkitItemVisitor<T> {
 	 * Act on (visit) a Volume content object
 	 *
 	 * @param v the volume to visit / act on
+	 *
 	 * @return result of the visit
 	 */
 	T visit(Volume v);
@@ -78,14 +83,25 @@ public interface SleuthkitItemVisitor<T> {
 	 * Act on (visit) a VolumeSystem content object
 	 *
 	 * @param vs the volume system to visit / act on
+	 *
 	 * @return result of the visit
 	 */
 	T visit(VolumeSystem vs);
+	
+	/**
+	 * Act on (visit) a Pool content object
+	 *
+	 * @param pool the volume system to visit / act on
+	 *
+	 * @return result of the visit
+	 */
+	T visit(Pool pool);
 
 	/**
 	 * Act on (visit) a blackboard artifact object
 	 *
 	 * @param ba blackboard artifact object to visit / act on
+	 *
 	 * @return result of the visit
 	 */
 	T visit(BlackboardArtifact ba);
@@ -94,6 +110,7 @@ public interface SleuthkitItemVisitor<T> {
 	 * Act on (visit) a blackboard artifact type
 	 *
 	 * @param tw blackboard artifact type to visit / act on
+	 *
 	 * @return result of the visit
 	 */
 	T visit(BlackboardArtifact.ARTIFACT_TYPE tw);
@@ -102,22 +119,34 @@ public interface SleuthkitItemVisitor<T> {
 	 * Act on (visit) a layout file content object
 	 *
 	 * @param lf layout file to visit / act on
+	 *
 	 * @return result of the visit
 	 */
 	T visit(LayoutFile lf);
 
 	/**
-	 * Act on (visit) a LayoutDirectory content object
+	 * Act on (visit) a VirtualDirectory content object
 	 *
 	 * @param ld layout dir to visit / act on
+	 *
 	 * @return result of the visit
 	 */
 	T visit(VirtualDirectory ld);
 
+	/**
+	 * Act on (visit) a LocalDirectory content object
+	 *
+	 * @param ld layout dir to visit / act on
+	 *
+	 * @return result of the visit
+	 */
+	T visit(LocalDirectory ld);
+
 	/**
 	 * Act on (visit) a DerivedFile content object
 	 *
 	 * @param df derived file to visit / act on
+	 *
 	 * @return result of the visit
 	 */
 	T visit(DerivedFile df);
@@ -126,10 +155,56 @@ public interface SleuthkitItemVisitor<T> {
 	 * Act on (visit) a LocalFile content object
 	 *
 	 * @param lf local file to visit / act on
+	 *
 	 * @return result of the visit
 	 */
 	T visit(LocalFile lf);
 
+	/**
+	 * Act on (visit) a SlackFile content object
+	 *
+	 * @param sf slack file to visit / act on
+	 *
+	 * @return result of the visit
+	 */
+	T visit(SlackFile sf);
+
+	/**
+	 * Act on (visit) a Report content object
+	 *
+	 * @param report report to visit / act on
+	 *
+	 * @return result of the visit
+	 */
+	T visit(Report report);
+	
+	/**
+	 * Act on (visit) a OsAccount content object
+	 *
+	 * @param account report to visit / act on
+	 *
+	 * @return result of the visit
+	 */
+	T visit(OsAccount account);
+	
+	/**
+	 * Act on (visit) an UnsupportedContent object
+	 *
+	 * @param unsupportedContent content to visit / act on
+	 *
+	 * @return result of the visit
+	 */
+	T visit(UnsupportedContent unsupportedContent);
+	
+	/**
+	 * Act on (visit) a LocalFilesDataSource content object
+	 *
+	 * @param localFilesDataSource report to visit / act on
+	 *
+	 * @return result of the visit
+	 */
+	T visit(LocalFilesDataSource localFilesDataSource);
+
 	/**
 	 * The default visitor - quickest method for implementing a custom visitor.
 	 * Every visit method delegates to the defaultVisit method, the only
@@ -137,7 +212,7 @@ public interface SleuthkitItemVisitor<T> {
 	 * methods for the objects on which the algorithm needs to act differently.
 	 *
 	 * @param <T> generic type, signifies the object type to be returned from
-	 * visit()
+	 *            visit()
 	 */
 	static abstract public class Default<T> implements SleuthkitItemVisitor<T> {
 
@@ -172,6 +247,11 @@ public interface SleuthkitItemVisitor<T> {
 		public T visit(VolumeSystem vs) {
 			return defaultVisit(vs);
 		}
+		
+		@Override
+		public T visit(Pool p) {
+			return defaultVisit(p);
+		}
 
 		@Override
 		public T visit(BlackboardArtifact ba) {
@@ -193,6 +273,11 @@ public interface SleuthkitItemVisitor<T> {
 			return defaultVisit(vd);
 		}
 
+		@Override
+		public T visit(LocalDirectory ld) {
+			return defaultVisit(ld);
+		}
+
 		@Override
 		public T visit(DerivedFile df) {
 			return defaultVisit(df);
@@ -202,5 +287,30 @@ public interface SleuthkitItemVisitor<T> {
 		public T visit(LocalFile lf) {
 			return defaultVisit(lf);
 		}
+
+		@Override
+		public T visit(SlackFile sf) {
+			return defaultVisit(sf);
+		}
+
+		@Override
+		public T visit(Report report) {
+			return defaultVisit(report);
+		}
+		
+		@Override
+		public T visit(OsAccount account) {
+			return defaultVisit(account);
+		}
+		
+		@Override
+		public T visit(UnsupportedContent unsupportedContent) {
+			return defaultVisit(unsupportedContent);
+		}
+		
+		@Override
+		public T visit(LocalFilesDataSource localFilesDataSource) {
+			return defaultVisit(localFilesDataSource);
+		}
 	}
 }
