@@ -24,17 +24,17 @@ import java.util.EnumSet;
 import java.util.Set;
 
 /**
- * Contains enums for the integer values stored in the database and returned by the 
- * various data model objects. 
+ * Contains enums for the integer values stored in the database and returned by
+ * the various data model objects.
  */
 public class TskData {
 
-	private static ResourceBundle bundle = ResourceBundle.getBundle("org.sleuthkit.datamodel.Bundle");
+	private final static ResourceBundle bundle = ResourceBundle.getBundle("org.sleuthkit.datamodel.Bundle");
 
-	/** 
-     * The type of the file system file, as reported in the name structure of the file system.
-     * This is the dir_type column in the tsk_files table.
-     */
+	/**
+	 * The type of the file system file, as reported in the name structure of
+	 * the file system. This is the dir_type column in the tsk_files table.
+	 */
 	public enum TSK_FS_NAME_TYPE_ENUM {
 
 		UNDEF(0, "-"), ///< Unknown type
@@ -47,7 +47,8 @@ public class TskData {
 		SOCK(7, "s"), ///< Socket NON-NLS
 		SHAD(8, "h"), ///< Shadow inode (solaris) NON-NLS
 		WHT(9, "w"), ///< Whiteout (openbsd) NON-NLS
-		VIRT(10, "v");     ///< Special (TSK added "Virtual" files) NON-NLS
+		VIRT(10, "v"),     ///< Special (TSK added "Virtual" files) NON-NLS
+		VIRT_DIR(11, "V");     ///< Special (TSK added "Virtual" directories) NON-NLS
 
 		private short dirType;
 		String label;
@@ -79,6 +80,7 @@ public class TskData {
 		 * Convert to the enum type from the short value
 		 *
 		 * @param dir_type enum type value to convert
+		 *
 		 * @return converted long value
 		 */
 		static public TSK_FS_NAME_TYPE_ENUM valueOf(short dir_type) {
@@ -92,10 +94,10 @@ public class TskData {
 		}
 	}
 
-    /**
-     * The type of the file system file, as reported in the metadata structure of the file system.
-     * This is the meta_type column in the tsk_files table.
-     */
+	/**
+	 * The type of the file system file, as reported in the metadata structure
+	 * of the file system. This is the meta_type column in the tsk_files table.
+	 */
 	public enum TSK_FS_META_TYPE_ENUM {
 
 		TSK_FS_META_TYPE_UNDEF(0, "-"),
@@ -108,7 +110,8 @@ public class TskData {
 		TSK_FS_META_TYPE_SHAD(7, "s"), ///< SOLARIS ONLY NON-NLS
 		TSK_FS_META_TYPE_SOCK(8, "h"), ///< UNIX domain socket NON-NLS
 		TSK_FS_META_TYPE_WHT(9, "w"), ///< Whiteout NON-NLS
-		TSK_FS_META_TYPE_VIRT(10, "v");      ///< "Virtual File" created by TSK for file system areas NON-NLS
+		TSK_FS_META_TYPE_VIRT(10, "v"),      ///< "Virtual File" created by TSK for file system areas NON-NLS
+		TSK_FS_META_TYPE_VIRT_DIR(11, "v");      ///< "Virtual Directory" created by TSK for Orphan Files NON-NLS
 
 		private short metaType;
 		private String metaTypeStr;
@@ -143,10 +146,11 @@ public class TskData {
 		}
 	}
 
-    /**
-     * The allocated status of a file system file, as reported in the name structure of the file system.
-     * This is the dir_flags column in the tsk_files table.
-     */
+	/**
+	 * The allocated status of a file system file, as reported in the name
+	 * structure of the file system. This is the dir_flags column in the
+	 * tsk_files table.
+	 */
 	public enum TSK_FS_NAME_FLAG_ENUM {
 
 		ALLOC(1, bundle.getString("TskData.tskFsNameFlagEnum.allocated")), ///< Name is in an allocated state
@@ -178,6 +182,7 @@ public class TskData {
 		 * Convert dirFlag int value to the enum type
 		 *
 		 * @param dirFlag int value to convert
+		 *
 		 * @return the enum type corresponding to dirFlag
 		 */
 		public static TSK_FS_NAME_FLAG_ENUM valueOf(int dirFlag) {
@@ -191,10 +196,11 @@ public class TskData {
 		}
 	}
 
-    /**
-     * The allocated status of the file system file, as reported in the metadata structure of the file system.
-     * This is the meta_flags column in the tsk_files table.
-     */
+	/**
+	 * The allocated status of the file system file, as reported in the metadata
+	 * structure of the file system. This is the meta_flags column in the
+	 * tsk_files table.
+	 */
 	public enum TSK_FS_META_FLAG_ENUM {
 
 		ALLOC(1, bundle.getString("TskData.tskFsMetaFlagEnum.allocated")), ///< Metadata structure is currently in an allocated state
@@ -235,6 +241,7 @@ public class TskData {
 		 * Returns all the enum elements that match the flags in metaFlag
 		 *
 		 * @param metaFlags Flags to convert to Enums.
+		 *
 		 * @return matching TSK_FS_META_FLAG_ENUM elements
 		 */
 		public static Set<TSK_FS_META_FLAG_ENUM> valuesOf(short metaFlags) {
@@ -261,10 +268,10 @@ public class TskData {
 
 	}
 
-    /**
-     * Type of data that is stored in the attribute for a file system file.
-     * This is the attr_type column in the tsk_files table.
-     */
+	/**
+	 * Type of data that is stored in the attribute for a file system file. This
+	 * is the attr_type column in the tsk_files table.
+	 */
 	public enum TSK_FS_ATTR_TYPE_ENUM {
 
 		TSK_FS_ATTR_TYPE_NOT_FOUND(0x00), // 0
@@ -318,8 +325,8 @@ public class TskData {
 	};
 
 	/**
-	 * Flags for a partition in the disk image. 
-     * This is the flags column in the tsk_vs_parts table.
+	 * Flags for a partition in the disk image. This is the flags column in the
+	 * tsk_vs_parts table.
 	 */
 	public enum TSK_VS_PART_FLAG_ENUM {
 
@@ -345,12 +352,14 @@ public class TskData {
 
 	}
 
-    /**
-     * The permissions of a file system file.
-     * This is the mode column in the tsk_files table.
-     */
+	/**
+	 * The permissions of a file system file. This is the mode column in the
+	 * tsk_files table.
+	 */
 	public enum TSK_FS_META_MODE_ENUM {
-		/* The following describe the file permissions */
+		/*
+		 * The following describe the file permissions
+		 */
 
 		TSK_FS_META_MODE_ISUID(0004000), ///< set user id on execution
 		TSK_FS_META_MODE_ISGID(0002000), ///< set group id on execution
@@ -388,6 +397,7 @@ public class TskData {
 		 * given modes
 		 *
 		 * @param modes
+		 *
 		 * @return matching TSK_FS_META_MODE_ENUM elements
 		 */
 		public static Set<TSK_FS_META_MODE_ENUM> valuesOf(short modes) {
@@ -406,6 +416,7 @@ public class TskData {
 
 		/**
 		 * @param modes the set of modes to convert
+		 *
 		 * @return the short int representing the given set of modes
 		 */
 		public static short toInt(Set<TSK_FS_META_MODE_ENUM> modes) {
@@ -417,45 +428,48 @@ public class TskData {
 		}
 	};
 
-    
-    /**
-     * The type of the file system.
-     * This is the fs_type column in the tsk_fs_info table.
-     */
+	/**
+	 * The type of the file system. This is the fs_type column in the
+	 * tsk_fs_info table.
+	 */
 	public enum TSK_FS_TYPE_ENUM {
 
-		TSK_FS_TYPE_DETECT(0x00000000), ///< Use autodetection methods
-		TSK_FS_TYPE_NTFS(0x00000001), ///< NTFS file system
-		TSK_FS_TYPE_NTFS_DETECT(0x00000001), ///< NTFS auto detection
-		TSK_FS_TYPE_FAT12(0x00000002), ///< FAT12 file system
-		TSK_FS_TYPE_FAT16(0x00000004), ///< FAT16 file system
-		TSK_FS_TYPE_FAT32(0x00000008), ///< FAT32 file system
-		TSK_FS_TYPE_EXFAT(0x0000000A), ///< ExFAT file system
-		TSK_FS_TYPE_FAT_DETECT(0x0000000e), ///< FAT auto detection
-		TSK_FS_TYPE_FFS1(0x00000010), ///< UFS1 (FreeBSD, OpenBSD, BSDI ...)
-		TSK_FS_TYPE_FFS1B(0x00000020), ///< UFS1b (Solaris - has no type)
-		TSK_FS_TYPE_FFS2(0x00000040), ///< UFS2 - FreeBSD, NetBSD 
-		TSK_FS_TYPE_FFS_DETECT(0x00000070), ///< UFS auto detection
-		TSK_FS_TYPE_EXT2(0x00000080), ///< Ext2 file system
-		TSK_FS_TYPE_EXT3(0x00000100), ///< Ext3 file system
-		TSK_FS_TYPE_EXT_DETECT(0x00000180), ///< ExtX auto detection
-		TSK_FS_TYPE_SWAP(0x00000200), ///< SWAP file system
-		TSK_FS_TYPE_SWAP_DETECT(0x00000200), ///< SWAP auto detection
-		TSK_FS_TYPE_RAW(0x00000400), ///< RAW file system
-		TSK_FS_TYPE_RAW_DETECT(0x00000400), ///< RAW auto detection
-		TSK_FS_TYPE_ISO9660(0x00000800), ///< ISO9660 file system
-		TSK_FS_TYPE_ISO9660_DETECT(0x00000800), ///< ISO9660 auto detection
-		TSK_FS_TYPE_HFS(0x00001000), ///< HFS file system
-		TSK_FS_TYPE_HFS_DETECT(0x00001000), ///< HFS auto detection
-		TSK_FS_TYPE_EXT4(0x00002000), ///< Ext4 file system
-		TSK_FS_TYPE_YAFFS2(0x00004000), ///< YAFFS2 file system
-		TSK_FS_TYPE_YAFFS2_DETECT(0x00004000), ///< YAFFS2 auto detection
-		TSK_FS_TYPE_UNSUPP(0xffffffff);        ///< Unsupported file system
+		TSK_FS_TYPE_DETECT(0x00000000, bundle.getString("TskData.tskFsTypeEnum.autoDetect")), ///< Use autodetection methods
+		TSK_FS_TYPE_NTFS(0x00000001, "NTFS"), ///< NTFS file system
+		TSK_FS_TYPE_NTFS_DETECT(0x00000001, bundle.getString("TskData.tskFsTypeEnum.NTFSautoDetect")), ///< NTFS auto detection
+		TSK_FS_TYPE_FAT12(0x00000002, "FAT12"), ///< FAT12 file system
+		TSK_FS_TYPE_FAT16(0x00000004, "FAT16"), ///< FAT16 file system
+		TSK_FS_TYPE_FAT32(0x00000008, "FAT32"), ///< FAT32 file system
+		TSK_FS_TYPE_EXFAT(0x0000000A, "ExFAT"), ///< ExFAT file system
+		TSK_FS_TYPE_FAT_DETECT(0x0000000e, bundle.getString("TskData.tskFsTypeEnum.FATautoDetect")), ///< FAT auto detection
+		TSK_FS_TYPE_FFS1(0x00000010, "UFS1"), ///< UFS1 (FreeBSD, OpenBSD, BSDI ...)
+		TSK_FS_TYPE_FFS1B(0x00000020, "UFS1b"), ///< UFS1b (Solaris - has no type)
+		TSK_FS_TYPE_FFS2(0x00000040, "UFS2"), ///< UFS2 - FreeBSD, NetBSD 
+		TSK_FS_TYPE_FFS_DETECT(0x00000070, "UFS"), ///< UFS auto detection
+		TSK_FS_TYPE_EXT2(0x00000080, "Ext2"), ///< Ext2 file system
+		TSK_FS_TYPE_EXT3(0x00000100, "Ext3"), ///< Ext3 file system
+		TSK_FS_TYPE_EXT_DETECT(0x00000180, bundle.getString("TskData.tskFsTypeEnum.ExtXautoDetect")), ///< ExtX auto detection
+		TSK_FS_TYPE_SWAP(0x00000200, "SWAP"), ///< SWAP file system
+		TSK_FS_TYPE_SWAP_DETECT(0x00000200, bundle.getString("TskData.tskFsTypeEnum.SWAPautoDetect")), ///< SWAP auto detection
+		TSK_FS_TYPE_RAW(0x00000400, "RAW"), ///< RAW file system
+		TSK_FS_TYPE_RAW_DETECT(0x00000400, bundle.getString("TskData.tskFsTypeEnum.RAWautoDetect")), ///< RAW auto detection
+		TSK_FS_TYPE_ISO9660(0x00000800, "ISO9660"), ///< ISO9660 file system
+		TSK_FS_TYPE_ISO9660_DETECT(0x00000800, bundle.getString("TskData.tskFsTypeEnum.ISO9660autoDetect")), ///< ISO9660 auto detection
+		TSK_FS_TYPE_HFS(0x00001000, "HFS"), ///< HFS file system
+		TSK_FS_TYPE_HFS_DETECT(0x00001000, bundle.getString("TskData.tskFsTypeEnum.HFSautoDetect")), ///< HFS auto detection
+		TSK_FS_TYPE_EXT4(0x00002000, "Ext4"), ///< Ext4 file system
+		TSK_FS_TYPE_YAFFS2(0x00004000, "YAFFS2"), ///< YAFFS2 file system
+		TSK_FS_TYPE_YAFFS2_DETECT(0x00004000, bundle.getString("TskData.tskFsTypeEnum.YAFFS2autoDetect")), ///< YAFFS2 auto detection
+		TSK_FS_TYPE_APFS(0x00010000, "APFS"), ///< APFS file system
+		TSK_FS_TYPE_APFS_DETECT(0x00010000, bundle.getString("TskData.tskFsTypeEnum.APFSautoDetect")), ///< APFS auto detection
+		TSK_FS_TYPE_UNSUPP(0xffffffff, bundle.getString("TskData.tskFsTypeEnum.unsupported"));        ///< Unsupported file system
 
 		private int value;
+		private String displayName;
 
-		private TSK_FS_TYPE_ENUM(int value) {
+		private TSK_FS_TYPE_ENUM(int value, String displayName) {
 			this.value = value;
+			this.displayName = displayName;
 		}
 
 		/**
@@ -466,12 +480,22 @@ public class TskData {
 		public int getValue() {
 			return value;
 		}
+		
+		/**
+		 * Get display name of the enum
+		 * 
+		 * @return the displayName
+		 */
+		public String getDisplayName() {
+			return displayName;
+		}
 
 		/**
 		 * Convert fs type int value to the enum type - get the first matching
 		 * enum type
 		 *
 		 * @param fsTypeValue int value to convert
+		 *
 		 * @return the enum type - first enum type matching the fsTypeValue
 		 */
 		public static TSK_FS_TYPE_ENUM valueOf(int fsTypeValue) {
@@ -486,11 +510,11 @@ public class TskData {
 
 	};
 
-    /**
-     * The type of the disk image.
-     * This is the types column in the tsk_images_info table.
-     */
-    public enum TSK_IMG_TYPE_ENUM {
+	/**
+	 * The type of the disk image. This is the types column in the
+	 * tsk_images_info table.
+	 */
+	public enum TSK_IMG_TYPE_ENUM {
 
 		TSK_IMG_TYPE_DETECT(0, bundle.getString("TskData.tskImgTypeEnum.autoDetect")), // Auto Detection
 		TSK_IMG_TYPE_RAW_SING(1, bundle.getString("TskData.tskImgTypeEnum.rawSingle")), // Single raw file (dd)
@@ -500,6 +524,9 @@ public class TskData {
 		TSK_IMG_TYPE_AFF_AFM(16, "AFM"), // AFF with external metadata NON-NLS
 		TSK_IMG_TYPE_AFF_ANY(32, "AFF"), // All AFFLIB image formats (including beta ones) NON-NLS
 		TSK_IMG_TYPE_EWF_EWF(64, "E01"), // Expert Witness format (encase) NON-NLS
+		TSK_IMG_TYPE_VMDK_VMDK(128, "VMDK"), // VMware Virtual Disk (VMDK) NON-NLS
+		TSK_IMG_TYPE_VHD_VHD(256, "VHD"), // Virtual Hard Disk (VHD) image format NON-NLS
+		TSK_IMG_TYPE_POOL_POOL(16384, "POOL"), // Pool (internal use) NON-NLS
 		TSK_IMG_TYPE_UNSUPP(65535, bundle.getString("TskData.tskImgTypeEnum.unknown"));   // Unsupported Image Type
 
 		private long imgType;
@@ -539,11 +566,11 @@ public class TskData {
 		}
 	};
 
-    /**
-     * The type of the partition in the partition table.
-     * This is the flags column in the tsk_vs_parts table.
-     */
-    public enum TSK_VS_TYPE_ENUM {
+	/**
+	 * The type of the partition in the partition table. This is the flags
+	 * column in the tsk_vs_parts table.
+	 */
+	public enum TSK_VS_TYPE_ENUM {
 
 		TSK_VS_TYPE_DETECT(0x0000, bundle.getString("TskData.tskVSTypeEnum.autoDetect")), ///< Use autodetection methods
 		TSK_VS_TYPE_DOS(0x0001, "DOS"), ///< DOS Partition table NON-NLS
@@ -551,6 +578,7 @@ public class TskData {
 		TSK_VS_TYPE_SUN(0x0004, "SUN VTOC"), ///< Sun VTOC NON-NLS
 		TSK_VS_TYPE_MAC(0x0008, "Mac"), ///< Mac partition table NON-NLS
 		TSK_VS_TYPE_GPT(0x0010, "GPT"), ///< GPT partition table NON-NLS
+		TSK_VS_TYPE_APFS(0x0020, "APFS"), ///< APFS pool NON-NLS
 		TSK_VS_TYPE_DBFILLER(0x00F0, bundle.getString("TskData.tskVSTypeEnum.fake")), ///< fake partition table type for loaddb (for images that do not have a volume system)
 		TSK_VS_TYPE_UNSUPP(0xFFFF, bundle.getString("TskData.tskVSTypeEnum.unsupported"));    ///< Unsupported
 
@@ -591,23 +619,30 @@ public class TskData {
 		}
 	};
 
-
-    /**
-     * High-level type of an object from the database. 
-     * This is the type column in the tsk_objects table. 
-     */
+	/**
+	 * High-level type of an object from the database. This is the type column
+	 * in the tsk_objects table.
+	 */
 	public enum ObjectType {
 
-		IMG(0), ///< Disk Image - see tsk_image_info for more details
-		VS(1), ///< Volume System - see tsk_vs_info for more details
-		VOL(2), ///< Volume - see tsk_vs_parts for more details
-		FS(3), ///< File System - see tsk_fs_info for more details
-		ABSTRACTFILE(4); ///< File - see tsk_files for more details
+		IMG(0, bundle.getString("TskData.ObjectType.IMG.name")), ///< Disk Image - see tsk_image_info for more details
+		VS(1, bundle.getString("TskData.ObjectType.VS.name")), ///< Volume System - see tsk_vs_info for more details
+		VOL(2, bundle.getString("TskData.ObjectType.VOL.name")), ///< Volume - see tsk_vs_parts for more details
+		FS(3, bundle.getString("TskData.ObjectType.FS.name")), ///< File System - see tsk_fs_info for more details
+		ABSTRACTFILE(4, bundle.getString("TskData.ObjectType.AbstractFile.name")), ///< File - see tsk_files for more details
+		ARTIFACT(5, bundle.getString("TskData.ObjectType.Artifact.name")),	/// Artifact - see blackboard_artifacts for more details
+		REPORT(6, bundle.getString("TskData.ObjectType.Report.name")),	///< Report - see reports for more details
+		POOL(7, bundle.getString("TskData.ObjectType.Pool.name")),	///< Pool
+		OS_ACCOUNT(8, bundle.getString("TskData.ObjectType.OsAccount.name")), ///< OS Account - see tsk_os_accounts for more details
+		HOST_ADDRESS(9, bundle.getString("TskData.ObjectType.HostAddress.name")), ///< Host Address - see tsk_host_addresses for more details
+		UNSUPPORTED(-1, bundle.getString("TskData.ObjectType.Unsupported.name")) ///< Unsupported type
+		; 
+		private final short objectType;
+		private final String displayName;
 
-		private short objectType;
-
-		private ObjectType(int objectType) {
+		private ObjectType(int objectType, String displayName) {
 			this.objectType = (short) objectType;
+			this.displayName = displayName;
 		}
 
 		/**
@@ -618,11 +653,17 @@ public class TskData {
 		public short getObjectType() {
 			return objectType;
 		}
+		
+		@Override
+		public String toString() {
+			return displayName;
+		}
 
 		/**
 		 * Convert object type short value to the enum type
 		 *
 		 * @param objectType long value to convert
+		 *
 		 * @return the enum type
 		 */
 		public static ObjectType valueOf(short objectType) {
@@ -631,14 +672,14 @@ public class TskData {
 					return v;
 				}
 			}
-			throw new IllegalArgumentException(
-					MessageFormat.format(bundle.getString("TskData.objectTypeEnum.exception.msg1.text"), objectType));
+			return UNSUPPORTED;
 		}
 	}
 
-    /**
-     * The type of file in a database, such as file system versus local file. 
-     * This is the type field in the tsk_files table. */
+	/**
+	 * The type of file in a database, such as file system versus local file.
+	 * This is the type field in the tsk_files table.
+	 */
 	public enum TSK_DB_FILES_TYPE_ENUM {
 
 		FS(0, "File System"), ///< File that can be found in file system tree. 
@@ -648,6 +689,9 @@ public class TskData {
 		UNALLOC_BLOCKS(4, "Unallocated Blocks"), ///< Set of blocks not allocated by file system.  Parent should be image, volume, or file system.  Many columns in tsk_files will be NULL. Set layout in tsk_file_layout. 
 		UNUSED_BLOCKS(5, "Unused Blocks"), ///< Set of blocks that are unallocated AND not used by a carved or other file type.  Parent should be UNALLOC_BLOCKS, many columns in tsk_files will be NULL, set layout in tsk_file_layout. 
 		VIRTUAL_DIR(6, "Virtual Directory"), ///< Virtual directory (not on fs) with no meta-data entry that can be used to group files of types other than TSK_DB_FILES_TYPE_FS. Its parent is either another TSK_DB_FILES_TYPE_FS or a root directory or type TSK_DB_FILES_TYPE_FS.
+		SLACK(7, "Slack"), ///< Slack space for a single file
+		LOCAL_DIR(8, "Local Directory"), ///< Local directory that was added (not from a disk image)
+		LAYOUT_FILE(9, "Layout File"), ///< Set of blocks from an image that have been designated as a file
 		;
 
 		private final short fileType;
@@ -662,6 +706,7 @@ public class TskData {
 		 * Convert db files type short value to the enum type
 		 *
 		 * @param fileType long value to convert
+		 *
 		 * @return the enum type
 		 */
 		public static TSK_DB_FILES_TYPE_ENUM valueOf(short fileType) {
@@ -687,11 +732,60 @@ public class TskData {
 			return name;
 		}
 	}
+	
+	/**
+	 * The type of pool in a database.
+	 * This is the pool_type field in the tsk_pool_info table.
+	 */
+	public enum TSK_POOL_TYPE_ENUM {
+		TSK_POOL_TYPE_DETECT(0, "Auto detect"), ///< Use autodetection methods
+		TSK_POOL_TYPE_APFS(1, "APFS Pool"), ///< APFS Pooled Volumes
+		TSK_POOL_TYPE_UNSUPP(0xffff, "Unsupported") ///< Unsupported pool container type
+		;
 
-    /**
-     * Identifies if a file was in a hash database or not. 
-     * This is the known column in the tsk_files table. 
-     */
+		private final short poolType;
+		private final String name;
+
+		TSK_POOL_TYPE_ENUM(int poolType, String name) {
+			this.poolType = (short) poolType;
+			this.name = name;
+		}
+
+		/**
+		 * Convert db pool type short value to the enum type
+		 *
+		 * @param poolType long value to convert
+		 *
+		 * @return the enum type
+		 */
+		public static TSK_POOL_TYPE_ENUM valueOf(long poolType) {
+			for (TSK_POOL_TYPE_ENUM type : TSK_POOL_TYPE_ENUM.values()) {
+				if (type.poolType == poolType) {
+					return type;
+				}
+			}
+			throw new IllegalArgumentException(
+					MessageFormat.format(bundle.getString("TskData.tskDbFilesTypeEnum.exception.msg1.text"), poolType)); // TODO
+		}
+
+		/**
+		 * Get short value of the file type
+		 *
+		 * @return the long value of the file type
+		 */
+		public short getValue() {
+			return poolType;
+		}
+
+		public String getName() {
+			return name;
+		}
+	}	
+
+	/**
+	 * Identifies if a file was in a hash database or not. This is the known
+	 * column in the tsk_files table.
+	 */
 	public enum FileKnown {
 
 		UNKNOWN(0, bundle.getString("TskData.fileKnown.unknown")), ///< File marked as unknown by hash db
@@ -710,6 +804,7 @@ public class TskData {
 		 * Convert file known type byte value to the enum type
 		 *
 		 * @param known long value to convert
+		 *
 		 * @return the enum type
 		 */
 		public static FileKnown valueOf(byte known) {
@@ -735,4 +830,61 @@ public class TskData {
 			return this.known;
 		}
 	}
+
+	/**
+	 * DbType is the enum covering database type. It tells you what underlying
+	 * database you can use in Autopsy and TSK.
+	 */
+	public enum DbType {
+
+		// Add any additional remote database types here, and keep it in sync
+		// with the Sleuthkit version of this enum located at:
+		// sleuthkit/tsk/auto/db_connection_info.h
+		// Be sure to add to settingsValid() if you add a type here.
+		SQLITE(0),
+		POSTGRESQL(1);
+
+		private int value;
+
+		DbType(int val) {
+			this.value = val;
+		}
+
+		public int getValue() {
+			return this.value;
+		}
+	}
+	
+	/**
+	 * Encoding type records whether locally stored files have been encoded
+	 * or not, and the method used to do so. This is the encoding_type column
+	 * in the tsk_files_path table.
+	 * Files are encoded using EncodedFileOutputStream and are saved to the
+	 * database as derived files with the appropriate encoding type argument.
+	 */
+	public enum EncodingType{
+		// Update EncodedFileUtil.java to handle any new types
+		NONE(0),
+		XOR1(1);
+		
+		private final int type;
+		
+		private EncodingType(int type){
+			this.type = type;
+		}
+		
+		public int getType(){
+			return type;
+		}
+		
+		public static EncodingType valueOf(int type) {
+			for (EncodingType v : EncodingType.values()) {
+				if (v.type == type) {
+					return v;
+				}
+			}
+			throw new IllegalArgumentException(
+					MessageFormat.format(bundle.getString("TskData.encodingType.exception.msg1.text"), type));
+		}
+	}
 }
