@@ -1,7 +1,7 @@
 /*
- * Autopsy Forensic Browser
+ * Sleuth Kit Data Model
  * 
- * Copyright 2011 Basis Technology Corp.
+ * Copyright 2011-2020 Basis Technology Corp.
  * Contact: carrier <at> sleuthkit <dot> org
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -34,15 +34,16 @@ public class Volume extends AbstractContent {
 	private long flags;
 	private String desc;
 	private volatile long volumeHandle = 0;
+	private volatile String uniquePath;
 	private static ResourceBundle bundle = ResourceBundle.getBundle("org.sleuthkit.datamodel.Bundle");
 
 	/**
 	 * Constructor to create the data object mapped from tsk_vs_parts entry
 	 *
-	 * @param db database object
+	 * @param db              database object
 	 * @param obj_id
 	 * @param addr
-	 * @param startSector starting sector, relative to start of VS
+	 * @param startSector     starting sector, relative to start of VS
 	 * @param lengthInSectors
 	 * @param flags
 	 * @param desc
@@ -52,6 +53,7 @@ public class Volume extends AbstractContent {
 		this.addr = addr;
 		this.startSector = startSector;
 		this.lengthInSectors = lengthInSectors;
+		this.uniquePath = null;
 		this.flags = flags;
 		if (!desc.equals("")) {
 			this.desc = desc;
@@ -68,6 +70,12 @@ public class Volume extends AbstractContent {
 				throw new TskCoreException(bundle.getString("Volume.read.exception.msg1.text"));
 			}
 			VolumeSystem parentVs = (VolumeSystem) myParent;
+			
+			// Reading from APFS volumes/volume systems is not yet supported
+			if (parentVs.getType().equals(TskData.TSK_VS_TYPE_ENUM.TSK_VS_TYPE_APFS)) {
+				throw new TskCoreException("Reading APFS pool volumes not yet supported");
+			}
+			
 			// read from the volume
 			if (volumeHandle == 0) {
 				volumeHandle = SleuthkitJNI.openVsPart(parentVs.getVolumeSystemHandle(), addr);
@@ -79,14 +87,9 @@ public class Volume extends AbstractContent {
 
 	@Override
 	public void close() {
-		if (volumeHandle != 0) {
-			synchronized (this) {
-				if (volumeHandle != 0) {
-					SleuthkitJNI.closeVs(volumeHandle);
-					volumeHandle = 0;
-				}
-			}
-		}
+        // there is nothing to free. The VolumeSystem structure
+        // in C++ contains this structure and will free it. 
+        volumeHandle = 0;
 	}
 
 	@Override
@@ -104,16 +107,23 @@ public class Volume extends AbstractContent {
 	}
 
 	@Override
-	public synchronized String getUniquePath() throws TskCoreException {
-		String uniquePath = "";
-		String name = getName();
-		if (!name.isEmpty()) {
-			uniquePath = "/vol_" + name; //NON-NLS
-		}
+	public String getUniquePath() throws TskCoreException {
+		// It is possible that multiple threads could be doing this calculation
+		// simultaneously, but it's worth the potential extra processing to prevent deadlocks.
+		if(uniquePath == null) {
+			String tempUniquePath = "";
+			String name = getName();
+			if (!name.isEmpty()) {
+				tempUniquePath = "/vol_" + name; //NON-NLS
+			}
 
-		Content myParent = getParent();
-		if (myParent != null) {
-			uniquePath = myParent.getUniquePath() + uniquePath;
+			Content myParent = getParent();
+			if (myParent != null) {
+				tempUniquePath = myParent.getUniquePath() + tempUniquePath;
+			}
+			
+			// Don't update uniquePath until it is complete.
+			uniquePath = tempUniquePath;
 		}
 		return uniquePath;
 	}
@@ -182,6 +192,7 @@ public class Volume extends AbstractContent {
 	 * Convert volume type flag to string
 	 *
 	 * @param vsFlag long flag to convert
+	 *
 	 * @return string representation
 	 */
 	public static String vsFlagToValue(long vsFlag) {
@@ -200,6 +211,7 @@ public class Volume extends AbstractContent {
 	 * Convert volume flag string to long
 	 *
 	 * @param vsFlag string representation of the flag
+	 *
 	 * @return long representation of the flag
 	 */
 	public static long valueToVsFlag(String vsFlag) {
@@ -218,6 +230,7 @@ public class Volume extends AbstractContent {
 	 * Convert long representation of the flag to user readable format
 	 *
 	 * @param vsFlag long repr. of the flag
+	 *
 	 * @return user readable string representation
 	 */
 	public static String vsFlagToString(long vsFlag) {
@@ -264,6 +277,7 @@ public class Volume extends AbstractContent {
 
 	/**
 	 * @return a list of FileSystem that are direct descendents of this Image.
+	 *
 	 * @throws TskCoreException
 	 */
 	public List<FileSystem> getFileSystems() throws TskCoreException {
