@@ -1,7 +1,7 @@
 /*
- * Autopsy Forensic Browser
+ * Sleuth Kit Data Model
  * 
- * Copyright 2011 Basis Technology Corp.
+ * Copyright 2011-2017 Basis Technology Corp.
  * Contact: carrier <at> sleuthkit <dot> org
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -33,10 +33,10 @@ public class VolumeSystem extends AbstractContent {
 	/**
 	 * Constructor most inputs are from the database
 	 *
-	 * @param db case database handle
-	 * @param obj_id the unique content object id for the volume system
-	 * @param name name of the volume system
-	 * @param type type of the volume system
+	 * @param db        case database handle
+	 * @param obj_id    the unique content object id for the volume system
+	 * @param name      name of the volume system
+	 * @param type      type of the volume system
 	 * @param imgOffset offset of the volume system with respect to image
 	 * @param blockSize block size of this volume system
 	 */
@@ -94,6 +94,7 @@ public class VolumeSystem extends AbstractContent {
 	 * otherwise resuse the existing handle.
 	 *
 	 * @return volume system Handle pointer
+	 *
 	 * @throws TskException
 	 */
 	protected synchronized long getVolumeSystemHandle() throws TskCoreException {
@@ -115,7 +116,7 @@ public class VolumeSystem extends AbstractContent {
 		if (volumeSystemHandle != 0) {
 			synchronized (this) {
 				if (volumeSystemHandle != 0) {
-					SleuthkitJNI.closeVs(volumeSystemHandle);
+					// SleuthkitJNI.closeVs(volumeSystemHandle); // closeVs is currently a no-op
 					volumeSystemHandle = 0;
 				}
 			}
@@ -130,7 +131,7 @@ public class VolumeSystem extends AbstractContent {
 			super.finalize();
 		}
 	}
-
+	
 	@Override
 	public <T> T accept(SleuthkitItemVisitor<T> v) {
 		return v.visit(this);
@@ -153,6 +154,7 @@ public class VolumeSystem extends AbstractContent {
 
 	/**
 	 * @return a list of Volumes that are direct children of this VolumeSystem
+	 *
 	 * @throws TskCoreException
 	 */
 	public List<Volume> getVolumes() throws TskCoreException {
