@@ -54,7 +54,7 @@ usage()
  * Prints the layout information
  * */
 static TSK_WALK_RET_ENUM
-part_act(TSK_VS_INFO * vs, const TSK_VS_PART_INFO * part, void *ptr)
+part_act(TSK_VS_INFO * vs, const TSK_VS_PART_INFO * part, void * /*ptr*/)
 {
     if (part->flags & TSK_VS_PART_FLAG_META)
         tsk_printf("%.3" PRIuPNUM ":  Meta      ", part->addr);
@@ -104,7 +104,7 @@ part_act(TSK_VS_INFO * vs, const TSK_VS_PART_INFO * part, void *ptr)
 
         /* Print the layout */
         tsk_printf("%.10" PRIuDADDR "   %.10" PRIuDADDR "   %.10" PRIuDADDR
-            "   %.4" PRIuOFF "%c   %s\n", part->start,
+            "   %.4" PRIdOFF "%c   %s\n", part->start,
             (TSK_DADDR_T) (part->start + part->len - 1), part->len, size,
             unit, part->desc);
     }
@@ -118,8 +118,9 @@ part_act(TSK_VS_INFO * vs, const TSK_VS_PART_INFO * part, void *ptr)
 
     if ((recurse) && (vs->vstype == TSK_VS_TYPE_DOS)
         && (part->flags == TSK_VS_PART_FLAG_ALLOC)) {
-        if (recurse_cnt < 64)
+        if (recurse_cnt < 64) {
             recurse_list[recurse_cnt++] = part->start * part->vs->block_size;
+        }
     }
 
     return TSK_WALK_CONT;
