@@ -15,9 +15,10 @@
 
 #include "tsk_auto_i.h"
 #include "tsk/fs/tsk_fatxxfs.h"
+#include "tsk/img/img_writer.h"
 
 
-// @@@ Follow through some error paths for sanity check and update docs somewhere to relfect the new scheme
+// @@@ Follow through some error paths for sanity check and update docs somewhere to reflect the new scheme
 
 TskAuto::TskAuto()
 {
@@ -29,6 +30,8 @@ TskAuto::TskAuto()
     m_internalOpen = false;
     m_curVsPartValid = false;
     m_curVsPartDescr = "";
+    m_imageWriterEnabled = false;
+    m_imageWriterPath = NULL;
 }
 
 
@@ -109,8 +112,9 @@ uint8_t
 
     m_internalOpen = true;
     m_img_info = tsk_img_open_utf8(a_numImg, a_images, a_imgType, a_sSize);
-    if (m_img_info)
-        return 0;
+	if (m_img_info) {
+		return 0;
+	}
     else
         return 1;
 }
@@ -142,6 +146,11 @@ uint8_t TskAuto::openImageHandle(TSK_IMG_INFO * a_img_info)
 void
  TskAuto::closeImage()
 {
+    for (size_t i = 0; i < m_poolInfos.size(); i++) {
+        tsk_pool_close(m_poolInfos[i]);
+    }
+    m_poolInfos.clear();
+
     if ((m_img_info) && (m_internalOpen)) {
         tsk_img_close(m_img_info);
     }
@@ -187,21 +196,39 @@ TSK_OFF_T TskAuto::getImageSize() const
 }
 
 TSK_FILTER_ENUM 
-TskAuto::filterVs(const TSK_VS_INFO * vs_info) 
+TskAuto::filterVs(const TSK_VS_INFO * /*vs_info*/)
 {
     return TSK_FILTER_CONT;
 }
 
 TSK_FILTER_ENUM 
-TskAuto::filterVol(const TSK_VS_PART_INFO * vs_part) 
+TskAuto::filterVol(const TSK_VS_PART_INFO * /*vs_part*/)
 {
     return TSK_FILTER_CONT;
 }
 
+TSK_FILTER_ENUM
+TskAuto::filterPoolVol(const TSK_POOL_VOLUME_INFO * /*pool_vol*/)
+{
+    /* Most of our tools can't handle pool volumes yet */
+    if (tsk_verbose)
+        fprintf(stderr, "filterPoolVol: Pool handling is not yet implemented for this tool\n");
+    return TSK_FILTER_SKIP;
+}
+
+TSK_FILTER_ENUM
+TskAuto::filterPool(const TSK_POOL_INFO * /*pool_info*/) {
+    /* Most of our tools can't handle pool volumes yet */
+    if (tsk_verbose)
+        fprintf(stderr, "filterPool: Pool handling is not yet implemented for this tool\n");
+    return TSK_FILTER_SKIP;
+}
+
 TSK_FILTER_ENUM 
-TskAuto::filterFs(TSK_FS_INFO * fs_info) {
+TskAuto::filterFs(TSK_FS_INFO * /*fs_info*/)
+{
     return TSK_FILTER_CONT;
-};
+}
 
 
 
@@ -209,7 +236,7 @@ TskAuto::filterFs(TSK_FS_INFO * fs_info) {
  * Starts in sector 0 of the opened disk images and looks for a
  * volume or file system. Will call processFile() on each file
  * that is found.
- * @return 1 if an error occured (message will have been registered) and 0 on success
+ * @return 1 if an error occurred (message will have been registered) and 0 on success
  */
 uint8_t
 TskAuto::findFilesInImg()
@@ -233,7 +260,7 @@ TskAuto::findFilesInImg()
  * and returning an error would indicate to TSK that errno and such are set. 
  */
 TSK_WALK_RET_ENUM
-    TskAuto::vsWalkCb(TSK_VS_INFO * a_vs_info,
+    TskAuto::vsWalkCb(TSK_VS_INFO * /*a_vs_info*/,
     const TSK_VS_PART_INFO * a_vs_part, void *a_ptr)
 {
     TskAuto *tsk = (TskAuto *) a_ptr;
@@ -253,11 +280,18 @@ TSK_WALK_RET_ENUM
         return TSK_WALK_STOP;    
 
     // process it
-    TSK_RETVAL_ENUM retval2 =
-        tsk->findFilesInFsRet(a_vs_part->start *
-        a_vs_part->vs->block_size, TSK_FS_TYPE_DETECT);
-    if ((retval2 == TSK_STOP) || (tsk->getStopProcessing())) {
-        return TSK_WALK_STOP;
+    if (tsk->hasPool(a_vs_part->start * a_vs_part->vs->block_size)) {
+        if (TSK_STOP == tsk->findFilesInPool(a_vs_part->start * a_vs_part->vs->block_size)
+            || tsk->getStopProcessing()) {
+            return TSK_WALK_STOP;
+        }
+    }
+    else {
+        TSK_RETVAL_ENUM retval2 = tsk->findFilesInFsRet(
+            a_vs_part->start * a_vs_part->vs->block_size, TSK_FS_TYPE_DETECT);
+        if ((retval2 == TSK_STOP) || (tsk->getStopProcessing())) {
+            return TSK_WALK_STOP;
+        }
     }
 
     //all errors would have been registered
@@ -272,7 +306,7 @@ TSK_WALK_RET_ENUM
  * that is found.
  * @param a_start Byte offset to start analyzing from.
  * @param a_vtype Volume system type to analyze
- * @return 1 if an error occured (messages will have been registered) and 0 on success
+ * @return 1 if an error occurred (messages will have been registered) and 0 on success
  */
 uint8_t
 TskAuto::findFilesInVs(TSK_OFF_T a_start, TSK_VS_TYPE_ENUM a_vtype)
@@ -286,10 +320,14 @@ TskAuto::findFilesInVs(TSK_OFF_T a_start, TSK_VS_TYPE_ENUM a_vtype)
     }
 
     TSK_VS_INFO *vs_info;
-    // USE mm_walk to get the volumes
+    // Use mm_walk to get the volumes
     if ((vs_info = tsk_vs_open(m_img_info, a_start, a_vtype)) == NULL) {
-        /* we're going to ignore this error to avoid confusion if the
-         * fs_open passes. */
+
+        /* If the error code is for encryption, we will register it. Otherwise,
+         * ignore this error to avoid confusion if the fs_open passes. */
+        if (tsk_error_get_errno() == TSK_ERR_VS_ENCRYPTED) {
+            registerError();
+        }
         tsk_error_reset();
 
         if(tsk_verbose)
@@ -297,7 +335,12 @@ TskAuto::findFilesInVs(TSK_OFF_T a_start, TSK_VS_TYPE_ENUM a_vtype)
 
         /* There was no volume system, but there could be a file system 
          * Errors will have been registered */
-        findFilesInFs(a_start);
+        if (hasPool(a_start)) {
+            findFilesInPool(a_start);
+        }
+        else {
+            findFilesInFs(a_start);
+        }
     }
     // process the volume system
     else {
@@ -322,7 +365,7 @@ TskAuto::findFilesInVs(TSK_OFF_T a_start, TSK_VS_TYPE_ENUM a_vtype)
  * volume system or file system. Will call processFile() on each file
  * that is found.
  * @param a_start Byte offset to start analyzing from.
- * @return 1 if an error occured (message will have been registered), 0 on success
+ * @return 1 if an error occurred (message will have been registered), 0 on success
  */
 uint8_t
 TskAuto::findFilesInVs(TSK_OFF_T a_start)
@@ -330,6 +373,153 @@ TskAuto::findFilesInVs(TSK_OFF_T a_start)
     return findFilesInVs(a_start, TSK_VS_TYPE_DETECT);
 }
 
+/**
+ * Checks whether a volume contains a pool.
+ * @param a_start Byte offset to start analyzing from.
+ * @return true if a pool is found, false if not or on error 
+ */
+bool 
+TskAuto::hasPool(TSK_OFF_T a_start) 
+{
+    if (!m_img_info) {
+        tsk_error_reset();
+        tsk_error_set_errno(TSK_ERR_AUTO_NOTOPEN);
+        tsk_error_set_errstr("hasPool -- img_info");
+        registerError();
+        return false;
+    }
+
+    const auto pool = tsk_pool_open_img_sing(m_img_info, a_start, TSK_POOL_TYPE_DETECT);
+    if (pool == nullptr) {
+        return false;
+    }
+    tsk_pool_close(pool);
+    return true;
+}
+
+/**
+* Starts in a specified byte offset of the opened disk images and opens a pool
+* to search though any file systems in the pool. Will call processFile() on each file
+* that is found.
+* @param start Byte offset to start analyzing from.
+* @return 1 if an error occurred (message will have been registered), 0 on success
+*/
+uint8_t
+TskAuto::findFilesInPool(TSK_OFF_T start)
+{
+    return findFilesInPool(start, TSK_POOL_TYPE_DETECT);
+}
+
+
+/**
+* Starts in a specified byte offset of the opened disk images and opens a pool
+* to search though any file systems in the pool. Will call processFile() on each file
+* that is found.
+* @param start Byte offset to start analyzing from.
+* @param ptype The type of pool
+* @return 1 if an error occurred (message will have been registered), 0 on success
+*/
+uint8_t 
+TskAuto::findFilesInPool(TSK_OFF_T start, TSK_POOL_TYPE_ENUM ptype)
+{
+    if (!m_img_info) {
+        tsk_error_reset();
+        tsk_error_set_errno(TSK_ERR_AUTO_NOTOPEN);
+        tsk_error_set_errstr("findFilesInPool -- img_info");
+        registerError();
+        return TSK_ERR;
+    }
+
+    const auto pool = tsk_pool_open_img_sing(m_img_info, start, ptype);
+    if (pool == nullptr) {
+        tsk_error_set_errstr2(
+            "findFilesInPool: Error opening pool");
+        registerError();
+        return TSK_ERR;
+    }
+
+    // see if the super class wants to continue with this.
+    TSK_FILTER_ENUM retval1 = filterPool(pool);
+    if (retval1 == TSK_FILTER_SKIP)
+        return TSK_OK;
+    else if (retval1 == TSK_FILTER_STOP)
+        return TSK_STOP;
+
+    /* Only APFS pools are currently supported */
+    if (pool->ctype == TSK_POOL_TYPE_APFS) {
+
+        TSK_POOL_VOLUME_INFO *vol_info = pool->vol_list;
+        while (vol_info != NULL) {
+
+            TSK_FILTER_ENUM filterRetval = filterPoolVol(vol_info);
+            if ((filterRetval == TSK_FILTER_STOP) || (m_stopAllProcessing)) {
+                tsk_pool_close(pool);
+                return TSK_STOP;
+            }
+
+            if (filterRetval != TSK_FILTER_SKIP) {
+                TSK_IMG_INFO *pool_img = pool->get_img_info(pool, vol_info->block);
+                if (pool_img != NULL) {
+                    TSK_FS_INFO *fs_info = apfs_open(pool_img, 0, TSK_FS_TYPE_APFS, "");
+                    if (fs_info) {
+                        TSK_RETVAL_ENUM retval = findFilesInFsInt(fs_info, fs_info->root_inum);
+                        tsk_fs_close(fs_info);
+
+                        if (retval == TSK_STOP) {
+                            tsk_img_close(pool_img);
+                            tsk_pool_close(pool);
+                            return TSK_STOP;
+                        }
+                    }
+                    else {
+                        if (vol_info->flags & TSK_POOL_VOLUME_FLAG_ENCRYPTED) {
+                            tsk_error_reset();
+                            tsk_error_set_errno(TSK_ERR_FS_ENCRYPTED);
+                            tsk_error_set_errstr(
+                                "Encrypted APFS file system");
+                            tsk_error_set_errstr2("Block: %" PRIdOFF, vol_info->block);
+                            registerError();
+                        }
+                        else {
+                            tsk_error_set_errstr2(
+                                "findFilesInPool: Error opening APFS file system");
+                            registerError();
+                        }
+
+                        tsk_img_close(pool_img);
+                        tsk_pool_close(pool);
+                        return TSK_ERR;
+                    }
+
+                    tsk_img_close(pool_img);
+                }
+                else {
+                    tsk_pool_close(pool);
+                    tsk_error_set_errstr2(
+                        "findFilesInPool: Error opening APFS pool");
+                    registerError();
+                    return TSK_ERR;
+                }
+            }
+
+            vol_info = vol_info->next;
+        }
+    }
+    else {
+        tsk_pool_close(pool);
+        tsk_error_reset();
+        tsk_error_set_errno(TSK_ERR_POOL_UNSUPTYPE);
+        tsk_error_set_errstr("%d", pool->ctype);
+        registerError();
+        return TSK_ERR;
+    }
+
+    // Store the pool_info for later use. It will be closed at the end of the add image process.
+    m_poolInfos.push_back(pool);
+
+    return TSK_OK;
+}
+
 
 /**
  * Starts in a specified byte offset of the opened disk images and looks for a
@@ -353,13 +543,15 @@ TSK_RETVAL_ENUM
     TSK_FS_INFO *fs_info;
     if ((fs_info = tsk_fs_open_img(m_img_info, a_start, a_ftype)) == NULL) {
         if (isCurVsValid() == false) {
-            tsk_error_set_errstr2 ("Sector offset: %" PRIuOFF, a_start/512);
+            tsk_error_set_errstr2 ("Sector offset: %" PRIdOFF, a_start/512);
             registerError();
             return TSK_ERR;
         }
         else if (getCurVsPartFlag() & TSK_VS_PART_FLAG_ALLOC) {
-            tsk_error_set_errstr2 ("Sector offset: %" PRIuOFF ", Partition Type: %s",
-                a_start/512, getCurVsPartDescr().c_str() );
+            tsk_error_set_errstr2 (
+                "Sector offset: %" PRIdOFF ", Partition Type: %s",
+                a_start/512, getCurVsPartDescr().c_str()
+            );
             registerError();
             return TSK_ERR;
         }
@@ -385,7 +577,7 @@ TSK_RETVAL_ENUM
  *
  * @param a_start Byte offset of file system starting location.
  *
- * @returns 1 if an error occured (messages will have been registered) and 0 on success
+ * @returns 1 if an error occurred (messages will have been registered) and 0 on success
  */
 uint8_t
 TskAuto::findFilesInFs(TSK_OFF_T a_start)
@@ -402,7 +594,7 @@ TskAuto::findFilesInFs(TSK_OFF_T a_start)
  * @param a_start Byte offset of file system starting location.
  * @param a_ftype Type of file system that is located at the offset.
  *
- * @returns 1 if an error occured (messages will have been registered) and 0 on success
+ * @returns 1 if an error occurred (messages will have been registered) and 0 on success
  */
 uint8_t
 TskAuto::findFilesInFs(TSK_OFF_T a_start, TSK_FS_TYPE_ENUM a_ftype)
@@ -421,7 +613,7 @@ TskAuto::findFilesInFs(TSK_OFF_T a_start, TSK_FS_TYPE_ENUM a_ftype)
  * @param a_ftype Type of file system that will be analyzed.
  * @param a_inum inum to start walking files system at.
  *
- * @returns 1 if an error occured (messages will have been registered) and 0 on success
+ * @returns 1 if an error occurred (messages will have been registered) and 0 on success
  */
 uint8_t
     TskAuto::findFilesInFs(TSK_OFF_T a_start, TSK_FS_TYPE_ENUM a_ftype,
@@ -438,13 +630,13 @@ uint8_t
     TSK_FS_INFO *fs_info;
     if ((fs_info = tsk_fs_open_img(m_img_info, a_start, a_ftype)) == NULL) {
         if (isCurVsValid() == false) {
-            tsk_error_set_errstr2 ("Sector offset: %" PRIuOFF, a_start/512);
+            tsk_error_set_errstr2 ("Sector offset: %" PRIdOFF, a_start/512);
             registerError();
             return TSK_ERR;
         }
         else if (getCurVsPartFlag() & TSK_VS_PART_FLAG_ALLOC) {
             tsk_error_set_errstr2(
-                "Sector offset: %" PRIuOFF ", Partition Type: %s",
+                "Sector offset: %" PRIdOFF ", Partition Type: %s",
                 a_start / 512, getCurVsPartDescr().c_str());
             registerError();
             return 1;
@@ -470,7 +662,7 @@ uint8_t
  * @param a_start Byte offset of file system starting location.
  * @param a_inum inum to start walking files system at.
  *
- * @returns 1 if an error occured (messages will have been registered) and 0 on success
+ * @returns 1 if an error occurred (messages will have been registered) and 0 on success
  */
 uint8_t
 TskAuto::findFilesInFs(TSK_OFF_T a_start, TSK_INUM_T a_inum)
@@ -484,7 +676,7 @@ TskAuto::findFilesInFs(TSK_OFF_T a_start, TSK_INUM_T a_inum)
  *
  * @param a_fs_info Pointer to a previously opened file system.
  *
- * @returns 1 if an error occured (messages will have been registered) and 0 on success
+ * @returns 1 if an error occurred (messages will have been registered) and 0 on success
  */
 uint8_t
 TskAuto::findFilesInFs(TSK_FS_INFO * a_fs_info)
@@ -501,6 +693,30 @@ TskAuto::findFilesInFs(TSK_FS_INFO * a_fs_info)
     return m_errors.empty() ? 0 : 1;
 }
 
+/**
+* Processes the file system represented by the given TSK_FS_INFO
+* pointer. Will Call processFile() on each file that is found.
+*
+* @param a_fs_info Pointer to a previously opened file system.
+ * @param a_inum inum to start walking files system at.
+*
+* @returns 1 if an error occurred (messages will have been registered) and 0 on success
+*/
+uint8_t
+TskAuto::findFilesInFs(TSK_FS_INFO * a_fs_info, TSK_INUM_T inum)
+{
+    if (a_fs_info == NULL) {
+        tsk_error_reset();
+        tsk_error_set_errno(TSK_ERR_AUTO_NOTOPEN);
+        tsk_error_set_errstr("findFilesInFs - fs_info");
+        registerError();
+        return 1;
+    }
+
+    findFilesInFsInt(a_fs_info, inum);
+    return m_errors.empty() ? 0 : 1;
+}
+
 /** \internal
  * file name walk callback.  Walk the contents of each file
  * that is found.
@@ -517,7 +733,7 @@ TSK_WALK_RET_ENUM
         // we have no way to register an error...
         return TSK_WALK_STOP;
     }
-    
+
     TSK_RETVAL_ENUM retval = tsk->processFile(a_fs_file, a_path);
     if ((retval == TSK_STOP) || (tsk->getStopProcessing()))
         return TSK_WALK_STOP;
@@ -547,7 +763,7 @@ TSK_RETVAL_ENUM
                 m_fileFilterFlags), dirWalkCb, this)) {
 
         tsk_error_set_errstr2(
-            "Error walking directory in file system at offset %" PRIuOFF, a_fs_info->offset);
+            "Error walking directory in file system at offset %" PRIdOFF, a_fs_info->offset);
         registerError();
         return TSK_ERR;
     }
@@ -586,11 +802,12 @@ TSK_RETVAL_ENUM
 
 
 TSK_RETVAL_ENUM 
-TskAuto::processAttribute(TSK_FS_FILE * fs_file,
-                                         const TSK_FS_ATTR * fs_attr, const char *path) 
+TskAuto::processAttribute(TSK_FS_FILE * /*fs_file*/,
+                          const TSK_FS_ATTR * /*fs_attr*/,
+                          const char * /*path*/)
 {
     return TSK_OK;
-};
+}
 
 
 void TskAuto::setStopProcessing() {
@@ -601,6 +818,55 @@ bool TskAuto::getStopProcessing() const {
     return m_stopAllProcessing;
 }
 
+
+TSK_RETVAL_ENUM
+TskAuto::enableImageWriter(const char * imagePath) {
+#ifdef TSK_WIN32
+	m_imageWriterEnabled = false;
+
+	size_t ilen;
+	UTF16 *utf16;
+	UTF8 *utf8;
+	TSKConversionResult retval2;
+
+	// we allocate the buffer with the same number of chars as the UTF-8 length
+	ilen = strlen(imagePath);
+	if ((m_imageWriterPath =
+		(wchar_t *)tsk_malloc((ilen + 1) * sizeof(wchar_t))) == NULL) {
+		free(m_imageWriterPath);
+		return TSK_ERR;
+	}
+
+	utf8 = (UTF8 *)imagePath;
+	utf16 = (UTF16 *)m_imageWriterPath;
+
+	retval2 =
+		tsk_UTF8toUTF16((const UTF8 **)&utf8, &utf8[ilen],
+			&utf16, &utf16[ilen], TSKlenientConversion);
+	if (retval2 != TSKconversionOK) {
+		tsk_error_set_errno(TSK_ERR_IMG_CONVERT);
+		tsk_error_set_errstr
+		("enableImageWriter: Error converting output path %s %d",
+			imagePath, retval2);
+		free(m_imageWriterPath);
+		return TSK_ERR;
+	}
+	*utf16 = '\0';
+
+	// At this point the path conversion was successful, so image writer is ready to go
+	m_imageWriterEnabled = true;
+
+#else
+	m_imageWriterEnabled = false;
+#endif
+	return TSK_OK;
+}
+
+void
+TskAuto::disableImageWriter() {
+	m_imageWriterEnabled = false;
+}
+
 uint8_t TskAuto::registerError() {
     // add to our list of errors
     error_record er;
@@ -641,7 +907,7 @@ std::string TskAuto::errorRecordToString(error_record &rec) {
 uint8_t 
 TskAuto::handleError() {
     return 0;
-};
+}
 
 
 /**
@@ -650,7 +916,7 @@ TskAuto::handleError() {
  * @returns 1 if the file is an NTFS System file, 0 if not.
  */
 uint8_t
-    TskAuto::isNtfsSystemFiles(TSK_FS_FILE * a_fs_file, const char *a_path)
+    TskAuto::isNtfsSystemFiles(TSK_FS_FILE * a_fs_file, const char * /*a_path*/)
 {
     if ((a_fs_file) && (a_fs_file->fs_info)
         && (TSK_FS_TYPE_ISNTFS(a_fs_file->fs_info->ftype))
@@ -669,7 +935,8 @@ uint8_t
 uint8_t
 TskAuto::isFATSystemFiles(TSK_FS_FILE *a_fs_file)
 {
-    if (a_fs_file && a_fs_file->fs_info && a_fs_file->name) {
+    if (a_fs_file && a_fs_file->fs_info && a_fs_file->name
+        && TSK_FS_TYPE_ISFAT(a_fs_file->fs_info->ftype)) {
         FATFS_INFO *fatfs = (FATFS_INFO*)a_fs_file->fs_info;
         TSK_INUM_T addr = a_fs_file->name->meta_addr;
         if ((addr == fatfs->mbr_virt_inum) || 
@@ -716,12 +983,12 @@ uint8_t
 TskAuto::isDir(TSK_FS_FILE * a_fs_file)
 {
     if ((a_fs_file) && (a_fs_file->name)) {
-        if (a_fs_file->name->type == TSK_FS_NAME_TYPE_DIR) {
+        if (TSK_FS_IS_DIR_NAME(a_fs_file->name->type)){
             return 1;
         }
         else if (a_fs_file->name->type == TSK_FS_NAME_TYPE_UNDEF) {
             if ((a_fs_file->meta)
-                && (a_fs_file->meta->type == TSK_FS_META_TYPE_DIR)) {
+                && (TSK_FS_IS_DIR_META(a_fs_file->meta->type))){
                 return 1;
             }
         }
