@@ -17,7 +17,7 @@
 #include "tsk_case_db.h"
 #include "tsk_auto_i.h"
 
-TskCaseDb::TskCaseDb(TskDbSqlite * a_db)
+TskCaseDb::TskCaseDb(TskDb * a_db)
 {
     m_tag = TSK_CASE_DB_TAG;
     m_db = a_db;
@@ -45,28 +45,27 @@ TskCaseDb::~TskCaseDb()
 }
 
 /**
-* Creates a new case with a new database and initializes its tables.
-* Fails if there's already a file at the given path. Returns a pointer
-* to a new TskCaseDb if successful, else NULL.
+* Creates a new single-user case with a new database and initializes its tables.
+* Fails if there's already a file at the given path.
 *
 * @param path Full path to create new database at.
+* @returns Pointer to a new TskCaseDb object, NULL on error
 */
 TskCaseDb *
 TskCaseDb::newDb(const TSK_TCHAR * const path)
 {
+    TskDb *db = new TskDbSqlite(path, true);
 
     // Check if the database already exsists
-    struct STAT_STR stat_buf;
-    if (TSTAT(path, &stat_buf) == 0) {
+    if (db->dbExists()) {
         tsk_error_reset();
         tsk_error_set_errno(TSK_ERR_AUTO_DB);
         tsk_error_set_errstr("Database %" PRIttocTSK
             " already exists.  Must be deleted first.", path);
+        delete(db);
         return NULL;
     }
 
-    TskDbSqlite *db = new TskDbSqlite(path, true);
-
     // Open the database.
     if (db->open(true)) {
         delete(db);
@@ -77,26 +76,26 @@ TskCaseDb::newDb(const TSK_TCHAR * const path)
 }
 
 /**
-* Opens a case from an existing database.
+* Opens a single-user case from an existing database.
 *
 * @param path Full path to open database from.
+* @returns Pointer to a new TskCaseDb object, NULL on error
 */
 TskCaseDb *
 TskCaseDb::openDb(const TSK_TCHAR * path)
 {
+    TskDb *db = new TskDbSqlite(path, true);
 
     // Confirm that database already exsists
-    struct STAT_STR stat_buf;
-    if (TSTAT(path, &stat_buf) != 0) {
+    if (!db->dbExists()) {
         tsk_error_reset();
         tsk_error_set_errno(TSK_ERR_AUTO_DB);
         tsk_error_set_errstr("Database %" PRIttocTSK
             " does not exist.  Must be created first.", path);
+        delete(db);
         return NULL;
     }
 
-    TskDbSqlite *db = new TskDbSqlite(path, true);
-
     // Open the database.
     if (db->open(false)) {
         delete(db);
@@ -106,7 +105,6 @@ TskCaseDb::openDb(const TSK_TCHAR * path)
     return new TskCaseDb(db);
 }
 
-
 /**
  * Prepares the process to add an image to the database. This method
  * allows the caller to specify options to be used during the ingest.
@@ -135,7 +133,7 @@ uint8_t
     TSK_IMG_TYPE_ENUM imgType, unsigned int sSize)
 {
     TskAutoDb autoDb(m_db, m_NSRLDb, m_knownBadDb);
-    
+
     if (autoDb.startAddImage(numImg, imagePaths, imgType, sSize)) {
         autoDb.revertAddImage();
         return 1;
@@ -144,7 +142,7 @@ uint8_t
     if (autoDb.commitAddImage()) {
         return 1;
     }
-    
+
     return 0;
 }
 
