@@ -7,11 +7,11 @@
  */
 
 /** \file mymalloc.c
- * These functions allocate and realocate memory and set the error handling functions
+ * These functions allocate and reallocate memory and set the error handling functions
  * when an error occurs.
  */
 
-/*	The IBM Public Licence must be distributed with this software.
+/*	The IBM Public License must be distributed with this software.
 * AUTHOR(S)
 *	Wietse Venema
 *	IBM T.J. Watson Research
@@ -32,22 +32,28 @@ tsk_malloc(size_t len)
     if ((ptr = malloc(len)) == 0) {
         tsk_error_reset();
         tsk_error_set_errno(TSK_ERR_AUX_MALLOC);
-        tsk_error_set_errstr("tsk_malloc: %s (%"PRIuSIZE" requested)", strerror(errno), len);
+        tsk_error_set_errstr("tsk_malloc: %s (%" PRIuSIZE" requested)", strerror(errno), len);
     }
     else {
         memset(ptr, 0, len);
     }
-    return (ptr);
+    return ptr;
 }
 
 /* tsk_realloc - reallocate memory and set error values if needed */
 void *
 tsk_realloc(void *ptr, size_t len)
 {
-    if ((ptr = realloc(ptr, len)) == 0) {
+    // Use tmpPtr to prevent memory leak when realloc failed
+    void *tmpPtr = realloc(ptr, len);
+    if (tmpPtr == 0) {
         tsk_error_reset();
         tsk_error_set_errno(TSK_ERR_AUX_MALLOC);
-        tsk_error_set_errstr("tsk_realloc: %s (%"PRIuSIZE" requested)", strerror(errno), len);
+        tsk_error_set_errstr("tsk_realloc: %s (%" PRIuSIZE" requested)", strerror(errno), len);
+        return (void *)0;
     }
-    return (ptr);
+    else {
+        ptr = tmpPtr;
+    }
+    return ptr;
 }
