@@ -102,31 +102,21 @@ count_slack_file_act(TSK_FS_FILE * fs_file, TSK_OFF_T a_off,
 
     if (tsk_verbose)
         tsk_fprintf(stderr,
-            "count_slack_file_act: Remaining File:  %" PRIuOFF
+            "count_slack_file_act: Remaining File:  %" PRIdOFF
             "  Buffer: %" PRIuSIZE "\n", data->flen, size);
 
-    /* This is not the last data unit */
-    if (data->flen >= size) {
+    /* This is not the last data unit, or the file has no slack */
+    if (data->flen >= (TSK_OFF_T)size) {
         data->flen -= size;
     }
-    /* We have passed the end of the allocated space */
-    else if (data->flen == 0) {
+    else {
+        /* We have passed the end of the allocated space */
         if (data->count-- == 0) {
             tsk_printf("%" PRIuDADDR "\n", addr);
             data->found = 1;
             return TSK_WALK_STOP;
-
         }
-    }
-    /* This is the last data unit and there is unused space */
-    else if (data->flen < size) {
-        if (data->count-- == 0) {
-            tsk_printf("%" PRIuDADDR "\n", addr);
-            data->found = 1;
-            return TSK_WALK_STOP;
-
-        }
-        data->flen = 0;
+        data->flen = 0; // for scenarios where last buffer had some unalloc
     }
 
     return TSK_WALK_CONT;
@@ -196,6 +186,7 @@ tsk_fs_blkcalc(TSK_FS_INFO * fs, TSK_FS_BLKCALC_FLAG_ENUM a_lclflags,
     BLKCALC_DATA data;
 
     data.count = a_cnt;
+    data.uncnt = 0;
     data.found = 0;
 
     if (a_lclflags == TSK_FS_BLKCALC_BLKLS) {
