@@ -35,10 +35,7 @@ fatfs_open(TSK_IMG_INFO *a_img_info, TSK_OFF_T a_offset, TSK_FS_TYPE_ENUM a_ftyp
     const char *func_name = "fatfs_open";
     FATFS_INFO *fatfs = NULL;
     TSK_FS_INFO *fs = NULL;
-    TSK_OFF_T boot_sector_offset = 0;
 	int find_boot_sector_attempt = 0;
-    ssize_t bytes_read = 0;
-    FATFS_MASTER_BOOT_RECORD *bootSector;
 
     tsk_error_reset();
 
@@ -49,6 +46,13 @@ fatfs_open(TSK_IMG_INFO *a_img_info, TSK_OFF_T a_offset, TSK_FS_TYPE_ENUM a_ftyp
         return NULL;
     }
 
+    if (a_img_info->sector_size == 0) {
+        tsk_error_reset();
+        tsk_error_set_errno(TSK_ERR_FS_ARG);
+        tsk_error_set_errstr("fatfs_open: sector size is 0");
+        return NULL;
+    }
+
 	// Allocate an FATFS_INFO and initialize its generic TSK_FS_INFO members. 
     if ((fatfs = (FATFS_INFO*)tsk_fs_malloc(sizeof(FATFS_INFO))) == NULL) {
         return NULL;
@@ -63,13 +67,22 @@ fatfs_open(TSK_IMG_INFO *a_img_info, TSK_OFF_T a_offset, TSK_FS_TYPE_ENUM a_ftyp
 
 	// Look for a FAT boot sector. Try up to three times because FAT32 and exFAT file systems have backup boot sectors.
     for (find_boot_sector_attempt = 0; find_boot_sector_attempt < 3; ++find_boot_sector_attempt) {
-        if (find_boot_sector_attempt == 1) {
-			// The FATXX backup boot sector is located in sector 6, look there.
-            boot_sector_offset = 6 * fs->img_info->sector_size; 
-		}
-        else if (find_boot_sector_attempt == 2) {
-			// The exFAT backup boot sector is located in sector 12, look there.
-            boot_sector_offset = 12 * fs->img_info->sector_size;
+        TSK_OFF_T boot_sector_offset;
+        FATFS_MASTER_BOOT_RECORD *bootSector;
+        ssize_t bytes_read = 0;
+
+        switch (find_boot_sector_attempt) {
+            case 0:
+                boot_sector_offset = 0;
+                break;
+            case 1:
+			    // The FATXX backup boot sector is located in sector 6, look there.
+                boot_sector_offset = 6 * fs->img_info->sector_size; 
+                break;
+            case 2:
+                // The exFAT backup boot sector is located in sector 12, look there.
+                boot_sector_offset = 12 * fs->img_info->sector_size;
+                break;
 		}
 
         // Read in the prospective boot sector. 
@@ -210,7 +223,7 @@ getFATCacheIdx(FATFS_INFO * fatfs, TSK_DADDR_T sect)
  * Set *value to the entry in the File Allocation Table (FAT) 
  * for the given cluster
  *
- * *value is in clusters and may need to be coverted to
+ * *value is in clusters and may need to be converted to
  * sectors by the calling function
  *
  * Invalid values in the FAT (i.e. greater than the largest
@@ -225,7 +238,6 @@ fatfs_getFAT(FATFS_INFO * fatfs, TSK_DADDR_T clust, TSK_DADDR_T * value)
     uint16_t tmp16;
     TSK_FS_INFO *fs = (TSK_FS_INFO *) & fatfs->fs_info;
     TSK_DADDR_T sect, offs;
-    ssize_t cnt;
     int cidx;
 
     /* Sanity Check */
@@ -280,6 +292,7 @@ fatfs_getFAT(FATFS_INFO * fatfs, TSK_DADDR_T clust, TSK_DADDR_T * value)
          * size must therefore be at least 2 sectors large 
          */
         if (offs == (FATFS_FAT_CACHE_B - 1)) {
+            ssize_t cnt;
 
             // read the data -- TTLs will already have been updated
             cnt =
@@ -473,7 +486,7 @@ fatfs_block_walk(TSK_FS_INFO * fs, TSK_DADDR_T a_start_blk,
     }
 
     /* cycle through the sectors.  We do the sectors before the first
-     * cluster seperate from the data area */
+     * cluster separate from the data area */
     addr = a_start_blk;
 
     /* Before the data area beings (FAT, root directory etc.) */
@@ -635,7 +648,7 @@ fatfs_block_walk(TSK_FS_INFO * fs, TSK_DADDR_T a_start_blk,
         if ((a_flags & TSK_FS_BLOCK_WALK_FLAG_AONLY) == 0) {
             cnt = tsk_fs_read_block
                 (fs, addr, data_buf, fs->block_size * read_size);
-            if (cnt != fs->block_size * read_size) {
+            if (cnt != (ssize_t)(fs->block_size * read_size)) {
                 if (cnt >= 0) {
                     tsk_error_reset();
                     tsk_error_set_errno(TSK_ERR_FS_READ);
@@ -755,7 +768,7 @@ fatfs_fscheck(TSK_FS_INFO * fs, FILE * hFile)
     tsk_error_set_errstr("fscheck not implemented for FAT yet");
     return 1;
 
-    /* Check that allocated dentries point to start of allcated cluster chain */
+    /* Check that allocated dentries point to start of allocated cluster chain */
 
 
     /* Size of file is consistent with cluster chain length */
