@@ -421,10 +421,11 @@ fatfs_make_data_runs(TSK_FS_FILE * a_fs_file)
     else if (fs_meta->attr_state == TSK_FS_META_ATTR_ERROR) {
         return 1;
     }
-    else if (fs_meta->attr != NULL) {
+
+    if (fs_meta->attr != NULL) {
         tsk_fs_attrlist_markunused(fs_meta->attr);
     }
-    else if (fs_meta->attr == NULL) {
+    else  {
         fs_meta->attr = tsk_fs_attrlist_alloc();
     }
 
@@ -547,8 +548,8 @@ fatfs_make_data_runs(TSK_FS_FILE * a_fs_file)
         TSK_DADDR_T sbase;
         TSK_DADDR_T startclust = clust;
         TSK_OFF_T recoversize = fs_meta->size;
-        int retval;
         TSK_FS_ATTR_RUN *data_run = NULL;
+        TSK_FS_ATTR_RUN *data_run_tmp = NULL;
         TSK_FS_ATTR_RUN *data_run_head = NULL;
         TSK_OFF_T full_len_s = 0;
         uint8_t canRecover = 1; // set to 0 if recovery is not possible
@@ -603,6 +604,7 @@ fatfs_make_data_runs(TSK_FS_FILE * a_fs_file)
             return 1;
         }
         else {
+            int retval;
 
             /* If the starting cluster is already allocated then we can't
              * recover it */
@@ -683,6 +685,7 @@ fatfs_make_data_runs(TSK_FS_FILE * a_fs_file)
                 tsk_fs_attrlist_getnew(fs_meta->attr,
                     TSK_FS_ATTR_NONRES)) == NULL) {
             fs_meta->attr_state = TSK_FS_META_ATTR_ERROR;
+            tsk_fs_attr_run_free(data_run_head);
             return 1;
         }
 
@@ -702,7 +705,9 @@ fatfs_make_data_runs(TSK_FS_FILE * a_fs_file)
         }
         // create a one cluster run
         else {
-            TSK_FS_ATTR_RUN *data_run_tmp = tsk_fs_attr_run_alloc();
+            tsk_fs_attr_run_free(data_run_head);
+
+            data_run_tmp = tsk_fs_attr_run_alloc();
             if (data_run_tmp == NULL) {
                 fs_meta->attr_state = TSK_FS_META_ATTR_ERROR;
                 return 1;
@@ -754,6 +759,11 @@ fatfs_make_data_runs(TSK_FS_FILE * a_fs_file)
                 tsk_error_set_errstr
                     ("%s: Invalid sector address in FAT (too large): %"
                     PRIuDADDR " (plus %d sectors)", func_name, sbase, fatfs->csize);
+                tsk_fs_attr_run_free(data_run_head);
+                if (list_seen != NULL) {
+                    tsk_list_free(list_seen);
+                    list_seen = NULL;
+                }
                 return 1;
             }
 
@@ -765,6 +775,10 @@ fatfs_make_data_runs(TSK_FS_FILE * a_fs_file)
                 if (data_run_tmp == NULL) {
                     tsk_fs_attr_run_free(data_run_head);
                     fs_meta->attr_state = TSK_FS_META_ATTR_ERROR;
+                    if (list_seen != NULL) {
+                        tsk_list_free(list_seen);
+                        list_seen = NULL;
+                    }
                     return 1;
                 }
 
@@ -793,8 +807,10 @@ fatfs_make_data_runs(TSK_FS_FILE * a_fs_file)
                         "  cluster: %" PRIuDADDR, func_name, fs_meta->addr, clust);
                     fs_meta->attr_state = TSK_FS_META_ATTR_ERROR;
                     tsk_fs_attr_run_free(data_run_head);
-                    tsk_list_free(list_seen);
-                    list_seen = NULL;
+                    if (list_seen != NULL) {
+                        tsk_list_free(list_seen);
+                        list_seen = NULL;
+                    }
                     return 1;
                 }
                 clust = nxt;
@@ -804,13 +820,25 @@ fatfs_make_data_runs(TSK_FS_FILE * a_fs_file)
                     if (tsk_verbose)
                         tsk_fprintf(stderr,
                             "Loop found while processing file\n");
+                    if (data_run_head != NULL ) {
+                      tsk_fs_attr_run_free(data_run_head);
+                      // Make sure to set data_run_head to NULL to prevent a use-after-free
+                      data_run_head = NULL;
+                    }
+                    if (list_seen != NULL) {
+                        tsk_list_free(list_seen);
+                        list_seen = NULL;
+                    }
                     break;
                 }
 
                 if (tsk_list_add(&list_seen, clust)) {
                     fs_meta->attr_state = TSK_FS_META_ATTR_ERROR;
-                    tsk_list_free(list_seen);
-                    list_seen = NULL;
+                    tsk_fs_attr_run_free(data_run_head);
+                    if (list_seen != NULL) {
+                        tsk_list_free(list_seen);
+                        list_seen = NULL;
+                    }
                     return 1;
                 }
             }
@@ -821,6 +849,10 @@ fatfs_make_data_runs(TSK_FS_FILE * a_fs_file)
                 tsk_fs_attrlist_getnew(fs_meta->attr,
                     TSK_FS_ATTR_NONRES)) == NULL) {
             fs_meta->attr_state = TSK_FS_META_ATTR_ERROR;
+            if (list_seen != NULL) {
+                tsk_list_free(list_seen);
+                list_seen = NULL;
+            }
             return 1;
         }
 
@@ -830,6 +862,11 @@ fatfs_make_data_runs(TSK_FS_FILE * a_fs_file)
                 fs_meta->size, fs_meta->size, roundup(fs_meta->size,
                     fatfs->csize * fs->block_size), 0, 0)) {
             fs_meta->attr_state = TSK_FS_META_ATTR_ERROR;
+            tsk_fs_attr_run_free(data_run_head);
+            if (list_seen != NULL) {
+                tsk_list_free(list_seen);
+                list_seen = NULL;
+            }
             return 1;
         }
 
@@ -881,17 +918,17 @@ print_addr_act(TSK_FS_FILE * fs_file, TSK_OFF_T a_off, TSK_DADDR_T addr,
  * @returns 1 on error and 0 on success.
  */
 uint8_t
-fatfs_istat(TSK_FS_INFO *a_fs, FILE *a_hFile, TSK_INUM_T a_inum,
+fatfs_istat(TSK_FS_INFO *a_fs, TSK_FS_ISTAT_FLAG_ENUM istat_flags, FILE *a_hFile, TSK_INUM_T a_inum,
     TSK_DADDR_T a_numblock, int32_t a_sec_skew)
 {
     const char* func_name = "fatfs_istat";
     FATFS_INFO *fatfs = (FATFS_INFO*)a_fs;
-    TSK_FS_META *fs_meta = NULL; 
-    TSK_FS_FILE *fs_file =  NULL;
+    TSK_FS_META *fs_meta = NULL;
+    TSK_FS_FILE *fs_file = NULL;
     TSK_FS_META_NAME_LIST *fs_name_list = NULL;
     FATFS_PRINT_ADDR print;
     char timeBuf[128];
- 
+
     tsk_error_reset();
     if (fatfs_ptr_arg_is_null(a_fs, "a_fs", func_name) ||
         fatfs_ptr_arg_is_null(a_hFile, "a_hFile", func_name) ||
@@ -931,7 +968,7 @@ fatfs_istat(TSK_FS_INFO *a_fs, FILE *a_hFile, TSK_INUM_T a_inum,
     }
 
     /* Print the file size. */
-    tsk_fprintf(a_hFile, "Size: %" PRIuOFF "\n", fs_meta->size);
+    tsk_fprintf(a_hFile, "Size: %" PRIdOFF "\n", fs_meta->size);
 
     /* Print the name. */
     if (fs_meta->name2) {
@@ -971,7 +1008,7 @@ fatfs_istat(TSK_FS_INFO *a_fs, FILE *a_hFile, TSK_INUM_T a_inum,
     }
 
     tsk_fprintf(a_hFile, "Written:\t%s\n", tsk_fs_time_to_str(fs_meta->mtime,
-            timeBuf));
+        timeBuf));
     tsk_fprintf(a_hFile, "Accessed:\t%s\n",
         tsk_fs_time_to_str(fs_meta->atime, timeBuf));
     tsk_fprintf(a_hFile, "Created:\t%s\n",
@@ -979,22 +1016,37 @@ fatfs_istat(TSK_FS_INFO *a_fs, FILE *a_hFile, TSK_INUM_T a_inum,
 
     /* Print the specified number of sector addresses. */
     tsk_fprintf(a_hFile, "\nSectors:\n");
-    if (a_numblock > 0) {
-        /* A bad hack to force a specified number of blocks */
-        fs_meta->size = a_numblock * a_fs->block_size;
+    if (istat_flags & TSK_FS_ISTAT_RUNLIST) {
+        const TSK_FS_ATTR *fs_attr_default =
+            tsk_fs_file_attr_get_type(fs_file,
+                TSK_FS_ATTR_TYPE_DEFAULT, 0, 0);
+        if (fs_attr_default && (fs_attr_default->flags & TSK_FS_ATTR_NONRES)) {
+            if (tsk_fs_attr_print(fs_attr_default, a_hFile)) {
+                tsk_fprintf(a_hFile, "\nError creating run lists\n");
+                tsk_error_print(a_hFile);
+                tsk_error_reset();
+            }
+        }
     }
-    print.istat_seen = 0;
-    print.idx = 0;
-    print.hFile = a_hFile;
-    if (tsk_fs_file_walk(fs_file,
+    else {
+
+        if (a_numblock > 0) {
+            /* A bad hack to force a specified number of blocks */
+            fs_meta->size = a_numblock * a_fs->block_size;
+        }
+        print.istat_seen = 0;
+        print.idx = 0;
+        print.hFile = a_hFile;
+        if (tsk_fs_file_walk(fs_file,
             (TSK_FS_FILE_WALK_FLAG_ENUM)(TSK_FS_FILE_WALK_FLAG_AONLY | TSK_FS_FILE_WALK_FLAG_SLACK),
-            print_addr_act, (void *) &print)) {
-        tsk_fprintf(a_hFile, "\nError reading file\n");
-        tsk_error_print(a_hFile);
-        tsk_error_reset();
-    }
-    else if (print.idx != 0) {
-        tsk_fprintf(a_hFile, "\n");
+            print_addr_act, (void *)&print)) {
+            tsk_fprintf(a_hFile, "\nError reading file\n");
+            tsk_error_print(a_hFile);
+            tsk_error_reset();
+        }
+        else if (print.idx != 0) {
+            tsk_fprintf(a_hFile, "\n");
+        }
     }
 
     tsk_fs_file_close(fs_file);
@@ -1018,7 +1070,7 @@ inode_walk_dent_act(TSK_FS_FILE * fs_file, const char *a_path, void *a_ptr)
     unsigned int flags = TSK_FS_FILE_WALK_FLAG_SLACK | TSK_FS_FILE_WALK_FLAG_AONLY;
 
     if ((fs_file->meta == NULL)
-        || (fs_file->meta->type != TSK_FS_META_TYPE_DIR))
+        || ( ! TSK_FS_IS_DIR_META(fs_file->meta->type)))
         return TSK_WALK_CONT;
 
     /* Get the sector addresses & ignore any errors */
@@ -1069,7 +1121,7 @@ fatfs_inode_walk(TSK_FS_INFO *a_fs, TSK_INUM_T a_start_inum,
 
     tsk_error_reset();
     if (fatfs_ptr_arg_is_null(a_fs, "a_fs", func_name) ||
-        fatfs_ptr_arg_is_null(a_action, "a_action", func_name)) {
+        fatfs_ptr_arg_is_null(*(void **) &a_action, "a_action", func_name)) {
         return 1;
     }
 
@@ -1353,7 +1405,7 @@ fatfs_inode_walk(TSK_FS_INFO *a_fs, TSK_INUM_T a_start_inum,
             /* Read in a cluster. */
             cnt = tsk_fs_read_block
                 (a_fs, sect, dino_buf, num_sectors_to_process << fatfs->ssize_sh);
-            if (cnt != (num_sectors_to_process << fatfs->ssize_sh)) {
+            if (cnt != (ssize_t)(num_sectors_to_process << fatfs->ssize_sh)) {
                 if (cnt >= 0) {
                     tsk_error_reset();
                     tsk_error_set_errno(TSK_ERR_FS_READ);
