@@ -48,15 +48,16 @@ typedef struct {
  * be added.
  * @param buf Buffer that contains the directory contents.
  * @param len Length of buffer in bytes (must be a multiple of sector
-*  size).
+ *  size).
  * @param addrs Array where each element is the original address of
  * the corresponding sector in a_buf (size of array is number of sectors in
  * the directory).
+ * @param recursion_depth Recursion depth to limit the number of self-calls
  * @return TSK_RETVAL_ENUM
 */
 TSK_RETVAL_ENUM
 fatxxfs_dent_parse_buf(FATFS_INFO *fatfs, TSK_FS_DIR *a_fs_dir, char *buf,
-    TSK_OFF_T len, TSK_DADDR_T *addrs)
+    TSK_OFF_T len, TSK_DADDR_T *addrs, int recursion_depth)
 {
     char *func_name = "fatxxfs_dent_parse_buf";
     unsigned int idx = 0; 
@@ -370,12 +371,12 @@ fatxxfs_dent_parse_buf(FATFS_INFO *fatfs, TSK_FS_DIR *a_fs_dir, char *buf,
                         /* The parent directory is not in the list.  We are going to walk
                         * the directory until we hit this directory. This process will
                         * populate the buffer table and we will then rescan it */
-                        if (tsk_fs_dir_walk(fs, fs->root_inum,
+                        if (tsk_fs_dir_walk_internal(fs, fs->root_inum,
                             (TSK_FS_DIR_WALK_FLAG_ENUM)(TSK_FS_DIR_WALK_FLAG_ALLOC |
                             TSK_FS_DIR_WALK_FLAG_UNALLOC |
                             TSK_FS_DIR_WALK_FLAG_RECURSE),
                             fatfs_find_parent_act,
-                            (void *) &a_fs_dir->fs_file->meta->addr)) {
+                            (void *) &a_fs_dir->fs_file->meta->addr, recursion_depth)) {
                                 return TSK_OK;
                         }
 
