@@ -61,9 +61,8 @@ tsk_fs_attr_run_alloc()
 void
 tsk_fs_attr_run_free(TSK_FS_ATTR_RUN * fs_attr_run)
 {
-    TSK_FS_ATTR_RUN *fs_attr_run_prev;
     while (fs_attr_run) {
-        fs_attr_run_prev = fs_attr_run;
+        TSK_FS_ATTR_RUN *fs_attr_run_prev = fs_attr_run;
         fs_attr_run = fs_attr_run->next;
         fs_attr_run_prev->next = NULL;
         free(fs_attr_run_prev);
@@ -134,12 +133,10 @@ tsk_fs_attr_free(TSK_FS_ATTR * a_fs_attr)
         tsk_fs_attr_run_free(a_fs_attr->nrd.run);
     a_fs_attr->nrd.run = NULL;
 
-    if (a_fs_attr->rd.buf)
-        free(a_fs_attr->rd.buf);
+    free(a_fs_attr->rd.buf);
     a_fs_attr->rd.buf = NULL;
 
-    if (a_fs_attr->name)
-        free(a_fs_attr->name);
+    free(a_fs_attr->name);
     a_fs_attr->name = NULL;
 
     free(a_fs_attr);
@@ -295,8 +292,8 @@ tsk_fs_attr_set_run(TSK_FS_FILE * a_fs_file, TSK_FS_ATTR * a_fs_attr,
     if (alloc_size < size) {
         tsk_error_reset();
         tsk_error_set_errno(TSK_ERR_FS_ARG);
-        tsk_error_set_errstr("tsk_fs_attr_set_run: alloc_size (%" PRIuOFF
-            ") is less than size (%" PRIuOFF ")", alloc_size, size);
+        tsk_error_set_errstr("tsk_fs_attr_set_run: alloc_size (%" PRIdOFF
+            ") is less than size (%" PRIdOFF ")", alloc_size, size);
         return 1;
     }
 
@@ -325,7 +322,7 @@ tsk_fs_attr_set_run(TSK_FS_FILE * a_fs_file, TSK_FS_ATTR * a_fs_attr,
     }
 
     /*
-     * If this is not in the begining, then we need to make a filler 
+     * If this is not in the beginning, then we need to make a filler 
      * to account for the cluster numbers we haven't seen yet
      *
      * This commonly happens when we process an MFT entry that
@@ -369,6 +366,101 @@ dump_attr(TSK_FS_ATTR * a_fs_attr)
     }
 }
 
+/*
+ * Prints the data runs for a non-resident attribute
+ */
+uint8_t
+tsk_fs_attr_print(const TSK_FS_ATTR * a_fs_attr, FILE* hFile) {
+    TSK_FS_ATTR_RUN *fs_attr_run;
+    uint32_t skip_remain;
+    TSK_OFF_T tot_size;
+    TSK_FS_INFO *fs = a_fs_attr->fs_file->fs_info;
+    TSK_OFF_T off = 0;
+    uint8_t stop_loop = 0;
+
+    if ( ! (a_fs_attr->flags & TSK_FS_ATTR_NONRES)) {
+        tsk_error_set_errstr("tsk_fs_attr_print called on non-resident attribute");
+        return TSK_ERR;
+    }
+
+    tot_size = a_fs_attr->size;
+    skip_remain = a_fs_attr->nrd.skiplen;
+
+    for (fs_attr_run = a_fs_attr->nrd.run; fs_attr_run;
+        fs_attr_run = fs_attr_run->next) {
+        TSK_DADDR_T addr, len_idx, run_len, run_start_addr;
+
+        addr = fs_attr_run->addr;
+        run_len = 0;
+        run_start_addr = addr;
+
+        /* cycle through each block in the run */
+        for (len_idx = 0; len_idx < fs_attr_run->len; len_idx++) {
+
+
+            /* If the address is too large then give an error */
+            if (addr + len_idx > fs->last_block) {
+                if (a_fs_attr->fs_file->
+                    meta->flags & TSK_FS_META_FLAG_UNALLOC)
+                    tsk_error_set_errno(TSK_ERR_FS_RECOVER);
+                else
+                    tsk_error_set_errno(TSK_ERR_FS_BLK_NUM);
+                tsk_error_set_errstr
+                    ("Invalid address in run (too large): %" PRIuDADDR "",
+                    addr + len_idx);
+                return TSK_ERR;
+            }
+
+
+            /* Need to account for the skip length, which is the number of bytes
+            * in the start of the attribute that are skipped and that are not
+            * included in the overall length.  We will seek past those and not
+            * return those in the action.  We just read a block size so check
+            * if there is data to be returned in this buffer. */
+
+            if (skip_remain >= fs->block_size) {
+                skip_remain -= fs->block_size;
+                run_start_addr++;
+            }
+            else {
+                size_t ret_len;
+
+                /* Do we want to return a full block, or just the end? */
+                if ((TSK_OFF_T)fs->block_size - skip_remain <
+                    tot_size - off)
+                    ret_len = fs->block_size - skip_remain;
+                else
+                    ret_len = (size_t)(tot_size - off);
+
+                off += ret_len;
+                run_len++;
+                skip_remain = 0;
+
+                if (off >= tot_size) {
+                    stop_loop = 1;
+                    break;
+                }
+            }
+        }    
+
+        if (fs_attr_run->flags & TSK_FS_ATTR_RUN_FLAG_SPARSE) {
+            tsk_fprintf(hFile, "  Starting address: X, length: %lld  Sparse", run_len);
+        }
+        else if (fs_attr_run->flags & TSK_FS_ATTR_RUN_FLAG_FILLER) {
+            tsk_fprintf(hFile, "  Starting address: X, length: %lld  Filler", run_len);
+        }
+        else {
+            tsk_fprintf(hFile, "  Starting address: %lld, length: %lld  %s", run_start_addr, run_len, 
+                (fs_attr_run->flags & TSK_FS_ATTR_RUN_FLAG_ENCRYPTED) ? "Encrypted": "");
+        }
+        tsk_fprintf(hFile, "\n");
+        if (stop_loop) {
+            break;
+        }
+    }
+    return TSK_OK;
+}
+
 /**
  * \internal
  * Add a set of consecutive runs to an attribute. This will add and remove FILLER entries
@@ -435,7 +527,7 @@ tsk_fs_attr_add_run(TSK_FS_INFO * a_fs, TSK_FS_ATTR * a_fs_attr,
 
         if (tsk_verbose)
             tsk_fprintf(stderr,
-                "tsk_fs_attr_add: %" PRIuOFF "@%" PRIuOFF
+                "tsk_fs_attr_add: %" PRIuDADDR "@%" PRIuDADDR
                 " (Filler: %s)\n", data_run_cur->offset, data_run_cur->len,
                 (data_run_cur->flags & TSK_FS_ATTR_RUN_FLAG_FILLER) ? "Yes"
                 : "No");
@@ -450,10 +542,11 @@ tsk_fs_attr_add_run(TSK_FS_INFO * a_fs, TSK_FS_ATTR * a_fs_attr,
                 tsk_error_set_errno(TSK_ERR_FS_GENFS);
                 tsk_error_set_errstr
                     ("tsk_fs_attr_add_run: could not add data_run b.c. offset (%"
-                    PRIuOFF ") is larger than FILLER (%" PRIuOFF ") (%"
+						PRIuDADDR ") is larger than FILLER (%" PRIuDADDR ") (%"
                     PRIuINUM ")", a_data_run_new->offset,
                     data_run_cur->offset, a_fs_attr->fs_file->meta->addr);
-                dump_attr(a_fs_attr);
+                if (tsk_verbose)
+                    dump_attr(a_fs_attr);
                 return 1;
             }
 
@@ -471,7 +564,7 @@ tsk_fs_attr_add_run(TSK_FS_INFO * a_fs, TSK_FS_ATTR * a_fs_attr,
                         a_fs_attr->nrd.run = a_data_run_new;
                 }
 
-                /* The new run does not start at the begining of
+                /* The new run does not start at the beginning of
                  * the filler, so make a new start filler
                  */
                 else {
@@ -561,7 +654,8 @@ tsk_fs_attr_add_run(TSK_FS_INFO * a_fs, TSK_FS_ATTR * a_fs_attr,
             "): No filler entry for %" PRIuDADDR ". Final: %" PRIuDADDR,
             a_fs_attr->fs_file->meta->addr, a_data_run_new->offset,
             data_run_prev->offset + data_run_prev->len);
-        dump_attr(a_fs_attr);
+        if (tsk_verbose)
+            dump_attr(a_fs_attr);
         return 1;
     }
 
@@ -702,7 +796,7 @@ tsk_fs_attr_walk_res(const TSK_FS_ATTR * fs_attr,
     retval = TSK_WALK_CONT;
     for (off = 0; off < fs_attr->size; off += read_len) {
 
-        if (fs_attr->size - off > buf_len)
+        if ((uint64_t) (fs_attr->size - off) > buf_len)
             read_len = buf_len;
         else
             read_len = (size_t) (fs_attr->size - off);
@@ -722,8 +816,7 @@ tsk_fs_attr_walk_res(const TSK_FS_ATTR * fs_attr,
             break;
     }
 
-    if (buf)
-        free(buf);
+    free(buf);
 
     if (retval == TSK_WALK_ERROR)
         return 1;
@@ -777,7 +870,6 @@ tsk_fs_attr_walk_nonres(const TSK_FS_ATTR * fs_attr,
         }
     }
 
-
     /* cycle through the number of runs we have */
     retval = TSK_WALK_CONT;
     for (fs_attr_run = fs_attr->nrd.run; fs_attr_run;
@@ -801,6 +893,7 @@ tsk_fs_attr_walk_nonres(const TSK_FS_ATTR * fs_attr,
                 tsk_error_set_errstr
                     ("Invalid address in run (too large): %" PRIuDADDR "",
                     addr + len_idx);
+                free(buf);
                 return 1;
             }
 
@@ -831,8 +924,8 @@ tsk_fs_attr_walk_nonres(const TSK_FS_ATTR * fs_attr,
                 else {
                     ssize_t cnt;
 
-                    cnt = tsk_fs_read_block
-                        (fs, addr + len_idx, buf, fs->block_size);
+                    cnt = tsk_fs_read_block_decrypt
+                        (fs, addr + len_idx, buf, fs->block_size, fs_attr_run->crypto_id + len_idx);
                     if (cnt != fs->block_size) {
                         if (cnt >= 0) {
                             tsk_error_reset();
@@ -841,6 +934,7 @@ tsk_fs_attr_walk_nonres(const TSK_FS_ATTR * fs_attr,
                         tsk_error_set_errstr2
                             ("tsk_fs_file_walk: Error reading block at %"
                             PRIuDADDR, addr + len_idx);
+                        free(buf);
                         return 1;
                     }
                     if ((off + fs->block_size > fs_attr->nrd.initsize)
@@ -908,8 +1002,7 @@ tsk_fs_attr_walk_nonres(const TSK_FS_ATTR * fs_attr,
             break;
     }
 
-    if (buf)
-        free(buf);
+    free(buf);
 
     if (retval == TSK_WALK_ERROR)
         return 1;
@@ -1034,12 +1127,12 @@ tsk_fs_attr_read(const TSK_FS_ATTR * a_fs_attr, TSK_OFF_T a_offset,
         if (a_offset >= a_fs_attr->size) {
             tsk_error_reset();
             tsk_error_set_errno(TSK_ERR_FS_READ_OFF);
-            tsk_error_set_errstr("tsk_fs_attr_read - %" PRIuOFF, a_offset);
+            tsk_error_set_errstr("tsk_fs_attr_read - %" PRIdOFF, a_offset);
             return -1;
         }
 
         len_toread = a_len;
-        if (a_offset + a_len > a_fs_attr->size) {
+        if (a_offset + (TSK_OFF_T)a_len > a_fs_attr->size) {
             len_toread = (size_t) (a_fs_attr->size - a_offset);
             memset(&a_buf[len_toread], 0, a_len - len_toread);
         }
@@ -1054,7 +1147,7 @@ tsk_fs_attr_read(const TSK_FS_ATTR * a_fs_attr, TSK_OFF_T a_offset,
         TSK_FS_ATTR_RUN *data_run_cur;
         TSK_DADDR_T blkoffset_toread;   // block offset of where we want to start reading from
         size_t byteoffset_toread;       // byte offset in blkoffset_toread of where we want to start reading from
-        size_t len_remain;      // length remaining to copy
+        ssize_t len_remain;      // length remaining to copy
         size_t len_toread;      // length total to copy
 
         if (((a_flags & TSK_FS_FILE_READ_FLAG_SLACK)
@@ -1063,7 +1156,7 @@ tsk_fs_attr_read(const TSK_FS_ATTR * a_fs_attr, TSK_OFF_T a_offset,
                 && (a_offset >= a_fs_attr->size))) {
             tsk_error_reset();
             tsk_error_set_errno(TSK_ERR_FS_READ_OFF);
-            tsk_error_set_errstr("tsk_fs_attr_read - %" PRIuOFF, a_offset);
+            tsk_error_set_errstr("tsk_fs_attr_read - %" PRIdOFF, a_offset);
             return -1;
         }
 
@@ -1073,36 +1166,36 @@ tsk_fs_attr_read(const TSK_FS_ATTR * a_fs_attr, TSK_OFF_T a_offset,
         // determine how many bytes we can copy
         len_toread = a_len;
         if (a_flags & TSK_FS_FILE_READ_FLAG_SLACK) {
-            if (a_offset + a_len > a_fs_attr->nrd.allocsize)
+            if (a_offset + (TSK_OFF_T)a_len > a_fs_attr->nrd.allocsize)
                 len_toread =
                     (size_t) (a_fs_attr->nrd.allocsize - a_offset);
         }
         else {
-            if (a_offset + a_len > a_fs_attr->size)
+            if (a_offset + (TSK_OFF_T)a_len > a_fs_attr->size)
                 len_toread = (size_t) (a_fs_attr->size - a_offset);
         }
+
+
         // wipe the buffer we won't read into
         if (len_toread < a_len)
             memset(&a_buf[len_toread], 0, a_len - len_toread);
 
         len_remain = len_toread;
 
-        // cycle through the run until we find where we can start to process the clusters
-        for (data_run_cur = a_fs_attr->nrd.run; data_run_cur;
+        // cycle through the runs until we find the one where our offset starts
+        for (data_run_cur = a_fs_attr->nrd.run; data_run_cur && len_remain > 0;
             data_run_cur = data_run_cur->next) {
+
             TSK_DADDR_T blkoffset_inrun;
             size_t len_inrun;
 
-            // we are done
-            if (len_remain <= 0)
-                break;
-
             // See if this run contains the starting offset they requested
             if (data_run_cur->offset + data_run_cur->len <=
                 blkoffset_toread)
                 continue;
 
-            // block offset into this run
+            // We want this run, so find out the offset that we want
+            // we'll start at 0 if we already read data in the last run. 
             if (data_run_cur->offset < blkoffset_toread)
                 blkoffset_inrun = blkoffset_toread - data_run_cur->offset;
             else
@@ -1111,20 +1204,28 @@ tsk_fs_attr_read(const TSK_FS_ATTR * a_fs_attr, TSK_OFF_T a_offset,
             // see if we need to read the rest of this run and into the next or if it is all here
             len_inrun = len_remain;
             if ((data_run_cur->len - blkoffset_inrun) * fs->block_size -
-                byteoffset_toread < len_remain)
+                byteoffset_toread < (size_t)len_remain) {
                 len_inrun =
                     (size_t) ((data_run_cur->len -
                         blkoffset_inrun) * fs->block_size -
                     byteoffset_toread);
+            }
 
             /* sparse files/runs just get 0s */
             if (data_run_cur->flags & TSK_FS_ATTR_RUN_FLAG_SPARSE) {
                 memset(&a_buf[len_toread - len_remain], 0, len_inrun);
             }
+
             /* FILLER entries exist when the source file system can store run
              * info out of order and we did not get all of the run info.  We
              * return 0s if data is read from this type of run. */
             else if (data_run_cur->flags & TSK_FS_ATTR_RUN_FLAG_FILLER) {
+                if (a_buf == NULL) {
+                    tsk_error_reset();
+                    tsk_error_set_errno(TSK_ERR_FS_READ_OFF);
+                    tsk_error_set_errstr("tsk_fs_attr_read - missing a_buf");
+                    return -1;
+                }
                 memset(&a_buf[len_toread - len_remain], 0, len_inrun);
                 if (tsk_verbose)
                     fprintf(stderr,
@@ -1133,6 +1234,7 @@ tsk_fs_attr_read(const TSK_FS_ATTR * a_fs_attr, TSK_OFF_T a_offset,
                         (a_fs_attr->fs_file->meta) ? a_fs_attr->
                         fs_file->meta->addr : 0);
             }
+
             // we return 0s for reads past the initsize (unless they want slack space)
             else if (((TSK_OFF_T) ((data_run_cur->offset +
                             blkoffset_inrun) * fs->block_size +
@@ -1147,11 +1249,13 @@ tsk_fs_attr_read(const TSK_FS_ATTR * a_fs_attr, TSK_OFF_T a_offset,
                                 meta)) ? a_fs_attr->fs_file->meta->
                         addr : 0);
             }
+
+            // we are going to read some data
             else {
                 TSK_OFF_T fs_offset_b;
                 ssize_t cnt;
 
-                // calcuate the byte offset in the file system
+                // calculate the byte offset in the file system that we want to read from
                 fs_offset_b =
                     (data_run_cur->addr +
                     blkoffset_inrun) * fs->block_size;
@@ -1159,19 +1263,17 @@ tsk_fs_attr_read(const TSK_FS_ATTR * a_fs_attr, TSK_OFF_T a_offset,
                 // add the byte offset in the block
                 fs_offset_b += byteoffset_toread;
 
-                // reset this in case we need to also read from the next run 
-                byteoffset_toread = 0;
-
                 cnt =
-                    tsk_fs_read(fs, fs_offset_b,
-                    &a_buf[len_toread - len_remain], len_inrun);
-                if (cnt != len_inrun) {
+                    tsk_fs_read_decrypt(fs, fs_offset_b,
+                    &a_buf[len_toread - len_remain], len_inrun, 
+                    data_run_cur->crypto_id + blkoffset_inrun);
+                if (cnt != (ssize_t)len_inrun) {
                     if (cnt >= 0) {
                         tsk_error_reset();
                         tsk_error_set_errno(TSK_ERR_FS_READ);
                     }
                     tsk_error_set_errstr2
-                        ("tsk_fs_attr_read_type: offset: %" PRIuOFF
+                        ("tsk_fs_attr_read_type: offset: %" PRIdOFF
                         "  Len: %" PRIuSIZE "", fs_offset_b, len_inrun);
                     return cnt;
                 }
@@ -1193,7 +1295,11 @@ tsk_fs_attr_read(const TSK_FS_ATTR * a_fs_attr, TSK_OFF_T a_offset,
                 }
 
             }
+
             len_remain -= len_inrun;
+
+            // reset this in case we need to also read from the next run 
+            byteoffset_toread = 0;
         }
         return (ssize_t) (len_toread - len_remain);
     }
