@@ -193,27 +193,27 @@ hfsmode2tsknametype(uint16_t a_mode)
 typedef struct {
     TSK_FS_DIR *fs_dir;
     TSK_FS_NAME *fs_name;
+    uint32_t cnid;
 } HFS_DIR_OPEN_META_INFO;
 
 static uint8_t
 hfs_dir_open_meta_cb(HFS_INFO * hfs, int8_t level_type,
-    const void *targ_data, const hfs_btree_key_cat * cur_key,
+    const hfs_btree_key_cat * cur_key, int cur_keylen, size_t nodesize,
     TSK_OFF_T key_off, void *ptr)
 {
-    uint32_t *cnid_p = (uint32_t *) targ_data;
     HFS_DIR_OPEN_META_INFO *info = (HFS_DIR_OPEN_META_INFO *) ptr;
     TSK_FS_INFO *fs = &hfs->fs_info;
 
     if (tsk_verbose)
         fprintf(stderr,
             "hfs_dir_open_meta_cb: want %" PRIu32 " vs got %" PRIu32
-            " (%s node)\n", *cnid_p, tsk_getu32(hfs->fs_info.endian,
+            " (%s node)\n", info->cnid, tsk_getu32(hfs->fs_info.endian,
                 cur_key->parent_cnid),
             (level_type == HFS_BT_NODE_TYPE_IDX) ? "Index" : "Leaf");
 
     if (level_type == HFS_BT_NODE_TYPE_IDX) {
         if (tsk_getu32(hfs->fs_info.endian,
-                cur_key->parent_cnid) < *cnid_p) {
+                cur_key->parent_cnid) < info->cnid) {
             return HFS_BTREE_CB_IDX_LT;
         }
         else {
@@ -226,26 +226,26 @@ hfs_dir_open_meta_cb(HFS_INFO * hfs, int8_t level_type,
         size_t rec_off2;
 
         if (tsk_getu32(hfs->fs_info.endian,
-                cur_key->parent_cnid) < *cnid_p) {
+                cur_key->parent_cnid) < info->cnid) {
             return HFS_BTREE_CB_LEAF_GO;
         }
         else if (tsk_getu32(hfs->fs_info.endian,
-                cur_key->parent_cnid) > *cnid_p) {
+                cur_key->parent_cnid) > info->cnid) {
             return HFS_BTREE_CB_LEAF_STOP;
         }
+	// Need at least 2 bytes for key_len
+        if (cur_keylen < 2) {
+            tsk_error_set_errno(TSK_ERR_FS_GENFS);
+            tsk_error_set_errstr("hfs_dir_open_meta: cur_keylen value out of bounds");
+            return HFS_BTREE_CB_ERR;
+        }
         rec_off2 = 2 + tsk_getu16(hfs->fs_info.endian, cur_key->key_len);
-        // @@@ NEED TO REPLACE THIS SOMEHOW, but need to figure out the max length
-        /*
-           if (rec_off2 > nodesize) {
-           tsk_error_set_errno(TSK_ERR_FS_GENFS);
-           tsk_error_set_errstr(
-           "hfs_dir_open_meta: offset of record+keylen %d in leaf node %d too large (%"PRIuSIZE" vs %"
-           PRIu16 ")", rec, cur_node, rec_off2, nodesize);
-           tsk_fs_name_free(fs_name);
-           free(node);
-           return TSK_COR;
-           }
-         */
+
+        if ((nodesize < 2) || (rec_off2 >= nodesize - 2)) {
+            tsk_error_set_errno(TSK_ERR_FS_GENFS);
+            tsk_error_set_errstr("hfs_dir_open_meta: nodesize value out of bounds");
+            return HFS_BTREE_CB_ERR;
+        }
         rec_type = tsk_getu16(hfs->fs_info.endian, &rec_buf[rec_off2]);
 
         // Catalog entry is for a file
@@ -268,6 +268,11 @@ hfs_dir_open_meta_cb(HFS_INFO * hfs, int8_t level_type,
 
         /* This is a folder in the folder */
         else if (rec_type == HFS_FOLDER_RECORD) {
+            if ((nodesize < sizeof(hfs_folder)) || (rec_off2 > nodesize - sizeof(hfs_folder))) {
+                tsk_error_set_errno(TSK_ERR_FS_GENFS);
+                tsk_error_set_errstr("hfs_dir_open_meta: nodesize value out of bounds");
+                return HFS_BTREE_CB_ERR;
+            }
             hfs_folder *folder = (hfs_folder *) & rec_buf[rec_off2];
 
             info->fs_name->meta_addr =
@@ -275,10 +280,19 @@ hfs_dir_open_meta_cb(HFS_INFO * hfs, int8_t level_type,
             info->fs_name->type = TSK_FS_NAME_TYPE_DIR;
             info->fs_name->flags = TSK_FS_NAME_FLAG_ALLOC;
 
+            // Make sure there is enough space in cur_key for the name 
+            // (name is unicode so each characters is two bytes; 6 bytes
+            // of non-name characters)
+            const int32_t nameLength =
+                tsk_getu16(hfs->fs_info.endian, cur_key->name.length);
 
+            if (2*nameLength > tsk_getu16(hfs->fs_info.endian, cur_key->key_len) - 6) {
+                error_returned
+                ("hfs_dir_open_meta_cb: name length is too long");
+                return HFS_BTREE_CB_ERR;
+            }
             if (hfs_UTF16toUTF8(fs, (uint8_t *) cur_key->name.unicode,
-                    tsk_getu16(hfs->fs_info.endian, cur_key->name.length),
-                    info->fs_name->name, HFS_MAXNAMLEN + 1,
+                    nameLength, info->fs_name->name, HFS_MAXNAMLEN + 1,
                     HFS_U16U8_FLAG_REPLACE_SLASH)) {
                 return HFS_BTREE_CB_ERR;
             }
@@ -286,6 +300,11 @@ hfs_dir_open_meta_cb(HFS_INFO * hfs, int8_t level_type,
 
         /* This is a normal file in the folder */
         else if (rec_type == HFS_FILE_RECORD) {
+            if ((nodesize < sizeof(hfs_file)) || (rec_off2 > nodesize - sizeof(hfs_file))) {
+                tsk_error_set_errno(TSK_ERR_FS_GENFS);
+                tsk_error_set_errstr("hfs_dir_open_meta: nodesize value out of bounds");
+                return HFS_BTREE_CB_ERR;
+            }
             hfs_file *file = (hfs_file *) & rec_buf[rec_off2];
             // This could be a hard link.  We need to test this CNID, and follow it if necessary.
             unsigned char is_err;
@@ -326,9 +345,19 @@ hfs_dir_open_meta_cb(HFS_INFO * hfs, int8_t level_type,
                         file->std.perm.mode));
             }
             info->fs_name->flags = TSK_FS_NAME_FLAG_ALLOC;
+
+            // Make sure there is enough space in cur_key for the name 
+            // (name is unicode so each characters is two bytes; 6 bytes
+            // of non-name characters)
+            const int32_t nameLength =
+                tsk_getu16(hfs->fs_info.endian, cur_key->name.length);
+            if (2*nameLength > tsk_getu16(hfs->fs_info.endian, cur_key->key_len) - 6) {
+                error_returned
+                ("hfs_dir_open_meta_cb: name length is too long");
+                return HFS_BTREE_CB_ERR;
+            }
             if (hfs_UTF16toUTF8(fs, (uint8_t *) cur_key->name.unicode,
-                    tsk_getu16(hfs->fs_info.endian, cur_key->name.length),
-                    info->fs_name->name, HFS_MAXNAMLEN + 1,
+                    nameLength, info->fs_name->name, HFS_MAXNAMLEN + 1,
                     HFS_U16U8_FLAG_REPLACE_SLASH)) {
                 return HFS_BTREE_CB_ERR;
             }
@@ -351,20 +380,21 @@ hfs_dir_open_meta_cb(HFS_INFO * hfs, int8_t level_type,
 
 /** \internal
 * Process a directory and load up FS_DIR with the entries. If a pointer to
-* an already allocated FS_DIR struture is given, it will be cleared.  If no existing
+* an already allocated FS_DIR structure is given, it will be cleared.  If no existing
 * FS_DIR structure is passed (i.e. NULL), then a new one will be created. If the return
 * value is error or corruption, then the FS_DIR structure could
-* have entries (depending on when the error occured).
+* have entries (depending on when the error occurred).
 *
 * @param a_fs File system to analyze
 * @param a_fs_dir Pointer to FS_DIR pointer. Can contain an already allocated
 * structure or a new structure.
 * @param a_addr Address of directory to process.
+* @param recursion_depth Recursion depth to limit the number of self-calls
 * @returns error, corruption, ok etc.
 */
 TSK_RETVAL_ENUM
 hfs_dir_open_meta(TSK_FS_INFO * fs, TSK_FS_DIR ** a_fs_dir,
-    TSK_INUM_T a_addr)
+    TSK_INUM_T a_addr, int recursion_depth)
 {
     HFS_INFO *hfs = (HFS_INFO *) fs;
     uint32_t cnid;              /* catalog node ID of the entry (= inum) */
@@ -411,7 +441,7 @@ hfs_dir_open_meta(TSK_FS_INFO * fs, TSK_FS_DIR ** a_fs_dir,
         return TSK_ERR;
     }
 
-    if ((fs_name = tsk_fs_name_alloc(HFS_MAXNAMLEN, 0)) == NULL) {
+    if ((fs_name = tsk_fs_name_alloc(HFS_MAXNAMLEN + 1, 0)) == NULL) {
         return TSK_ERR;
     }
     info.fs_dir = fs_dir;
@@ -488,7 +518,8 @@ hfs_dir_open_meta(TSK_FS_INFO * fs, TSK_FS_DIR ** a_fs_dir,
         }
     }
 
-    if (hfs_cat_traverse(hfs, &cnid, hfs_dir_open_meta_cb, &info)) {
+    info.cnid = cnid;
+    if (hfs_cat_traverse(hfs, hfs_dir_open_meta_cb, &info)) {
         tsk_fs_name_free(fs_name);
         return TSK_ERR;
     }
