@@ -72,10 +72,9 @@ static void
 iso9660_inode_list_free(TSK_FS_INFO * fs)
 {
     ISO_INFO *iso = (ISO_INFO *) fs;
-    iso9660_inode_node *tmp;
 
     while (iso->in_list) {
-        tmp = iso->in_list;
+        iso9660_inode_node *tmp = iso->in_list;
         iso->in_list = iso->in_list->next;
         free(tmp);
     }
@@ -91,10 +90,11 @@ iso9660_inode_list_free(TSK_FS_INFO * fs)
  * @param buf Buffer of data to process
  * @param count Length of buffer in bytes.
  * @param hFile File handle to print details to  (or NULL for no printing)
+ * @param recursion_depth Recursion depth to limit the number of self-calls
  * @returns NULL on error
  */
 static rockridge_ext *
-parse_susp(TSK_FS_INFO * fs, char *buf, int count, FILE * hFile)
+parse_susp(TSK_FS_INFO * fs, char *buf, int count, FILE * hFile, int recursion_depth)
 {
     rockridge_ext *rr;
     ISO_INFO *iso = (ISO_INFO *) fs;
@@ -104,6 +104,11 @@ parse_susp(TSK_FS_INFO * fs, char *buf, int count, FILE * hFile)
     if (tsk_verbose)
         tsk_fprintf(stderr, "parse_susp: count is: %d\n", count);
 
+    // 32 is an arbitrary chosen value.
+    if (recursion_depth > 32) {
+        return NULL;
+    }
+
     // allocate the output data structure
     rr = (rockridge_ext *) tsk_malloc(sizeof(rockridge_ext));
     if (rr == NULL) {
@@ -142,7 +147,6 @@ parse_susp(TSK_FS_INFO * fs, char *buf, int count, FILE * hFile)
             // read the continued buffer and parse it
             if ((tsk_getu32(fs->endian, ce->blk_m) < fs->last_block) &&
                 (tsk_getu32(fs->endian, ce->offset_m) < fs->block_size)) {
-                ssize_t cnt;
                 TSK_OFF_T off;
                 char *buf2;
 
@@ -155,11 +159,20 @@ parse_susp(TSK_FS_INFO * fs, char *buf, int count, FILE * hFile)
                         ce->celen_m));
 
                 if (buf2 != NULL) {
-                    cnt =
+                    ssize_t cnt =
                         tsk_fs_read(fs, off, buf2,
                         tsk_getu32(fs->endian, ce->celen_m));
+
                     if (cnt == tsk_getu32(fs->endian, ce->celen_m)) {
-                        parse_susp(fs, buf2, (int) cnt, hFile);
+                        rockridge_ext *rr_sub_entry = parse_susp(fs, buf2, (int) cnt, hFile, recursion_depth + 1);
+
+                        // Prevent an infinite loop
+                        if (rr_sub_entry == NULL) {
+                          free(buf2);
+                          free(rr);
+                          return NULL;
+			}
+                        free(rr_sub_entry);
                     }
                     else if (tsk_verbose) {
                         fprintf(stderr,
@@ -207,7 +220,7 @@ parse_susp(TSK_FS_INFO * fs, char *buf, int count, FILE * hFile)
             }
             buf += head->len;
         }
-        // SUSP Extention Registration -- not used
+        // SUSP Extension Registration -- not used
         else if ((head->sig[0] == 'E') && (head->sig[1] == 'R')) {
             iso9660_susp_er *er = (iso9660_susp_er *) buf;
             if (hFile) {
@@ -229,7 +242,7 @@ parse_susp(TSK_FS_INFO * fs, char *buf, int count, FILE * hFile)
             }
             buf += head->len;
         }
-        // SUSP Extention Sigs  -- not used
+        // SUSP Extension Sigs  -- not used
         else if ((head->sig[0] == 'E') && (head->sig[1] == 'S')) {
             if (hFile) {
                 fprintf(hFile, "ES Entry\n");
@@ -290,6 +303,7 @@ parse_susp(TSK_FS_INFO * fs, char *buf, int count, FILE * hFile)
         // RR -- alternative name
         else if ((head->sig[0] == 'N') && (head->sig[1] == 'M')) {
             iso9660_rr_nm_entry *rr_nm;
+            int len;
 
             if ((uintptr_t)buf + sizeof(iso9660_rr_nm_entry) - 1> (uintptr_t)end) {
                 if (tsk_verbose) 
@@ -299,14 +313,20 @@ parse_susp(TSK_FS_INFO * fs, char *buf, int count, FILE * hFile)
 
             rr_nm = (iso9660_rr_nm_entry *) buf;
 
-            if ((uintptr_t)&rr_nm->name[0] + (int) rr_nm->len - 5 - 1> (uintptr_t)end) {
+            if ((rr_nm->len < 6) || ((uintptr_t)&rr_nm->name[0] + (int) rr_nm->len - 5 - 1> (uintptr_t)end)) {
                 if (tsk_verbose) 
                     tsk_fprintf(stderr, "parse_susp: not enough room for RR alternative name\n");
                 break;
             }
 
-            strncpy(rr->fn, &rr_nm->name[0], (int) rr_nm->len - 5);
-            rr->fn[(int) rr_nm->len - 5] = '\0';
+            // Make sure the name will fit in the buffer
+            len = rr_nm->len - 5;
+            if (len >= ISO9660_MAXNAMLEN_RR) {
+                len = ISO9660_MAXNAMLEN_RR - 1;
+            }
+
+            strncpy(rr->fn, &rr_nm->name[0], len);
+            rr->fn[len] = '\0';
             if (hFile) {
                 fprintf(hFile, "NM Entry\n");
                 fprintf(hFile, "* %s\n", rr->fn);
@@ -414,7 +434,7 @@ iso9660_load_inodes_dir(TSK_FS_INFO * fs, TSK_OFF_T a_offs, int count,
 
     if (tsk_verbose)
         tsk_fprintf(stderr,
-            "iso9660_load_inodes_dir: offs: %" PRIuOFF
+            "iso9660_load_inodes_dir: offs: %" PRIdOFF
             " count: %d ctype: %d fn: %s\n", a_offs, count, ctype, a_fn);
 
     // cycle through each sector -- entries will not cross them
@@ -502,6 +522,7 @@ iso9660_load_inodes_dir(TSK_FS_INFO * fs, TSK_OFF_T a_offs, int count,
                     tsk_error_set_errno(TSK_ERR_FS_ARG);
                     tsk_error_set_errstr
                         ("iso9660_load_inodes_dir: Name argument specified is too long");
+                    free(in_node);
                     return -1;
                 }
                 strncpy(in_node->inode.fn, a_fn, ISO9660_MAXNAMLEN_STD + 1);
@@ -531,6 +552,8 @@ iso9660_load_inodes_dir(TSK_FS_INFO * fs, TSK_OFF_T a_offs, int count,
                         if (tsk_verbose)
                             tsk_fprintf(stderr,
                                         "iso9660_load_inodes_dir: UTF-16 name length is too large, bailing\n");
+                        free(in_node);
+                        in_node = NULL;
                         break;
                     }
 
@@ -576,6 +599,8 @@ iso9660_load_inodes_dir(TSK_FS_INFO * fs, TSK_OFF_T a_offs, int count,
                         if (tsk_verbose)
                             tsk_fprintf(stderr,
                                         "iso9660_load_inodes_dir: ASCII name length is too large, bailing\n");
+                        free(in_node);
+                        in_node = NULL;
                         break;
                     }
 
@@ -589,6 +614,8 @@ iso9660_load_inodes_dir(TSK_FS_INFO * fs, TSK_OFF_T a_offs, int count,
                     tsk_error_set_errno(TSK_ERR_FS_ARG);
                     tsk_error_set_errstr
                         ("Invalid ctype in iso9660_load_inodes_dir");
+                    free(in_node);
+                    in_node = NULL;
                     return -1;
                 }
 
@@ -611,6 +638,8 @@ iso9660_load_inodes_dir(TSK_FS_INFO * fs, TSK_OFF_T a_offs, int count,
                     if (tsk_verbose)
                         tsk_fprintf(stderr,
                                     "iso9660_load_inodes_dir: length of name after processing is 0. bailing\n");
+                    free(in_node);
+                    in_node = NULL;
                     break;
                     
                 }
@@ -629,16 +658,20 @@ iso9660_load_inodes_dir(TSK_FS_INFO * fs, TSK_OFF_T a_offs, int count,
                     tsk_fprintf(stderr,
                                 "iso9660_load_inodes_dir: file starts past end of image (%"PRIu32"). bailing\n",
                                 tsk_getu32(fs->endian, dentry->ext_loc_m));
+                free(in_node);
+                in_node = NULL;
                 break;
             }
             in_node->offset =
                 tsk_getu32(fs->endian, dentry->ext_loc_m) * fs->block_size;
             
-            if (tsk_getu32(fs->endian, in_node->inode.dr.data_len_m) + in_node->offset > fs->block_count * fs->block_size) {
+            if (tsk_getu32(fs->endian, in_node->inode.dr.data_len_m) + in_node->offset > (TSK_OFF_T)(fs->block_count * fs->block_size)) {
                 if (tsk_verbose)
                     tsk_fprintf(stderr,
                                 "iso9660_load_inodes_dir: file ends past end of image (%"PRIu32" bytes). bailing\n",
                                 tsk_getu32(fs->endian, in_node->inode.dr.data_len_m) + in_node->offset);
+                free(in_node);
+                in_node = NULL;
                 break;
             }
             /* record size to make sure fifos show up as unique files */
@@ -666,11 +699,13 @@ iso9660_load_inodes_dir(TSK_FS_INFO * fs, TSK_OFF_T a_offs, int count,
                 in_node->inode.rr =
                     parse_susp(fs,
                     &buf[b_offs + sizeof(iso9660_dentry) + dentry->fi_len],
-                    extra_bytes, NULL);
+                    extra_bytes, NULL, 0);
                 if (in_node->inode.rr == NULL) {
                     if (tsk_verbose)
                         tsk_fprintf(stderr,
                                     "iso9660_load_inodes_dir: parse_susp returned error (%s). bailing\n", tsk_error_get());
+                    free(in_node);
+                    in_node = NULL;
                     break;
                 }
                 
@@ -753,7 +788,7 @@ iso9660_load_inodes_dir(TSK_FS_INFO * fs, TSK_OFF_T a_offs, int count,
  * Process the path table for a joliet secondary volume descriptor
  * and load all of the files pointed to it.
  * The path table contains an entry for each directory.  This code
- * then locates each of the diretories and proceses the contents.
+ * then locates each of the directories and processes the contents.
  *
  * @param fs File system to process
  * @param svd Pointer to the secondary volume descriptor
@@ -774,7 +809,9 @@ iso9660_load_inodes_pt_joliet(TSK_FS_INFO * fs, iso9660_svd * svd,
     pt_len = tsk_getu32(fs->endian, svd->pt_size_m);
 
     while (pt_len > 0) {
-        char utf16_buf[ISO9660_MAXNAMLEN_JOL + 1];      // UTF-16 name from img
+        // Since further on cnt + 1 is used and cnt can be ISO9660_MAXNAMLEN_JOL
+        // + 2 ensures utf16_buf is sufficiently large.
+        char utf16_buf[ISO9660_MAXNAMLEN_JOL + 2];      // UTF-16 name from img
         char utf8buf[2 * ISO9660_MAXNAMLEN_JOL + 1];    // UTF-8 version of name
         int readlen;
         TSK_OFF_T extent;       /* offset of extent for current directory */
@@ -802,7 +839,7 @@ iso9660_load_inodes_pt_joliet(TSK_FS_INFO * fs, iso9660_svd * svd,
         if (dir.len_di > ISO9660_MAXNAMLEN_JOL)
             readlen = ISO9660_MAXNAMLEN_JOL;
 
-        memset(utf16_buf, 0, ISO9660_MAXNAMLEN_JOL);
+        memset(utf16_buf, 0, ISO9660_MAXNAMLEN_JOL + 2);
         /* get UCS-2 filename for the entry */
         cnt = tsk_fs_read(fs, pt_offs, (char *) utf16_buf, readlen);
         if (cnt != dir.len_di) {
@@ -881,7 +918,6 @@ iso9660_load_inodes_pt(ISO_INFO * iso)
     char fn[ISO9660_MAXNAMLEN_STD + 1]; /* store current directory name */
     path_table_rec dir;
     TSK_OFF_T pt_offs;          /* offset of where we are in path table */
-    size_t pt_len;              /* bytes left in path table */
     TSK_OFF_T extent;           /* offset of extent for current directory */
     ssize_t cnt;
     uint8_t is_first = 1;
@@ -915,6 +951,7 @@ iso9660_load_inodes_pt(ISO_INFO * iso)
 
     /* Now look for unique files in the primary descriptors */
     for (p = iso->pvd; p != NULL; p = p->next) {
+        size_t pt_len;              /* bytes left in path table */
 
         pt_offs =
             (TSK_OFF_T) (tsk_getu32(fs->endian,
@@ -1121,22 +1158,30 @@ static void
 iso9660_close(TSK_FS_INFO * fs)
 {
     ISO_INFO *iso = (ISO_INFO *) fs;
-    iso9660_pvd_node *p;
-    iso9660_svd_node *s;
 
     fs->tag = 0;
+
     while (iso->pvd != NULL) {
-        p = iso->pvd;
+        iso9660_pvd_node *p = iso->pvd;
         iso->pvd = iso->pvd->next;
         free(p);
     }
 
     while (iso->svd != NULL) {
-        s = iso->svd;
+        iso9660_svd_node *s = iso->svd;
         iso->svd = iso->svd->next;
         free(s);
     }
 
+    while (iso->in_list != NULL) {
+        iso9660_inode_node *in = iso->in_list;
+        iso->in_list = iso->in_list->next;
+        if (in->inode.rr != NULL) {
+            free(in->inode.rr);
+        }
+        free(in);
+    }
+
     tsk_fs_free(fs);
 }
 
@@ -1211,7 +1256,7 @@ iso9660_inode_walk(TSK_FS_INFO * fs, TSK_INUM_T start, TSK_INUM_T last,
     ISO_INFO *iso = (ISO_INFO *) fs;
     TSK_INUM_T inum, end_inum_tmp;
     TSK_FS_FILE *fs_file;
-    int myflags;
+    unsigned int myflags;
     iso9660_inode *dinode;
 
     // clean up any error messages that are lying around
@@ -1367,12 +1412,11 @@ iso9660_inode_walk(TSK_FS_INFO * fs, TSK_INUM_T start, TSK_INUM_T last,
      * Cleanup.
      */
     tsk_fs_file_close(fs_file);
-    if (dinode != NULL)
-        free((char *) dinode);
+    free(dinode);
     return 0;
 }
 
-// @@@ Doesn' thit seem to ignore interleave?
+// @@@ Doesn' this seem to ignore interleave?
 /* return 1 if block is allocated in a file's extent, return 0 otherwise */
 static int
 iso9660_is_block_alloc(TSK_FS_INFO * fs, TSK_DADDR_T blk_num)
@@ -1401,7 +1445,7 @@ iso9660_is_block_alloc(TSK_FS_INFO * fs, TSK_DADDR_T blk_num)
 }
 
 
-TSK_FS_BLOCK_FLAG_ENUM static
+static TSK_FS_BLOCK_FLAG_ENUM
 iso9660_block_getflags(TSK_FS_INFO * a_fs, TSK_DADDR_T a_addr)
 {
     return (iso9660_is_block_alloc(a_fs, a_addr)) ?
@@ -1540,11 +1584,12 @@ iso9660_make_data_run(TSK_FS_FILE * a_fs_file)
     else if (a_fs_file->meta->attr_state == TSK_FS_META_ATTR_ERROR) {
         return 1;
     }
+
     // not sure why this would ever happen, but...
-    else if (a_fs_file->meta->attr != NULL) {
+    if (a_fs_file->meta->attr != NULL) {
         tsk_fs_attrlist_markunused(a_fs_file->meta->attr);
     }
-    else if (a_fs_file->meta->attr == NULL) {
+    else  {
         a_fs_file->meta->attr = tsk_fs_attrlist_alloc();
     }
 
@@ -2011,7 +2056,7 @@ iso9660_print_rockridge(FILE * hFile, rockridge_ext * rr)
  * @returns 1 on error and 0 on success
  */
 static uint8_t
-iso9660_istat(TSK_FS_INFO * fs, FILE * hFile, TSK_INUM_T inum,
+iso9660_istat(TSK_FS_INFO * fs, TSK_FS_ISTAT_FLAG_ENUM istat_flags, FILE * hFile, TSK_INUM_T inum,
     TSK_DADDR_T numblock, int32_t sec_skew)
 {
     ISO_INFO *iso = (ISO_INFO *) fs;
@@ -2105,10 +2150,12 @@ iso9660_istat(TSK_FS_INFO * fs, FILE * hFile, TSK_INUM_T inum,
             cnt =
                 tsk_fs_read(fs, dinode->susp_off, buf2,
                 (size_t) dinode->susp_len);
+
+            rockridge_ext *rr_entry = NULL;
             if (cnt == dinode->susp_len) {
-                parse_susp(fs, buf2, (int) cnt, hFile);
+                rr_entry = parse_susp(fs, buf2, (int) cnt, hFile, 0);
             }
-            else {
+            if (rr_entry == NULL) {
                 fprintf(hFile, "Error reading Rock Ridge Location\n");
                 if (tsk_verbose) {
                     fprintf(stderr,
@@ -2175,8 +2222,21 @@ iso9660_istat(TSK_FS_INFO * fs, FILE * hFile, TSK_INUM_T inum,
         tsk_fs_time_to_str(fs_file->meta->atime, timeBuf));
 
     tsk_fprintf(hFile, "\nSectors:\n");
-    /* since blocks are all contiguous, print them here to simplify file_walk */
-    {
+    if (istat_flags & TSK_FS_ISTAT_RUNLIST) {
+        const TSK_FS_ATTR *fs_attr_default =
+            tsk_fs_file_attr_get_type(fs_file,
+                TSK_FS_ATTR_TYPE_DEFAULT, 0, 0);
+        if (fs_attr_default && (fs_attr_default->flags & TSK_FS_ATTR_NONRES)) {
+            if (tsk_fs_attr_print(fs_attr_default, hFile)) {
+                tsk_fprintf(hFile, "\nError creating run lists\n");
+                tsk_error_print(hFile);
+                tsk_error_reset();
+            }
+        }
+    }
+    else {
+        /* since blocks are all contiguous, print them here to simplify file_walk */
+   
         int block = tsk_getu32(fs->endian, dinode->dr.ext_loc_m);
         TSK_OFF_T size = fs_file->meta->size;
         int rowcount = 0;
@@ -2194,8 +2254,7 @@ iso9660_istat(TSK_FS_INFO * fs, FILE * hFile, TSK_INUM_T inum,
     }
 
     tsk_fs_file_close(fs_file);
-    if (dinode != NULL)
-        free((char *) dinode);
+    free(dinode);
     return 0;
 }
 
@@ -2336,8 +2395,10 @@ load_vol_desc(TSK_FS_INFO * fs)
         magic_seen = 1;
 
         // see if we are done
-        if (vd->type == ISO9660_VOL_DESC_SET_TERM)
+        if (vd->type == ISO9660_VOL_DESC_SET_TERM) {
+            free(vd);
             break;
+        }
 
         switch (vd->type) {
 
@@ -2409,6 +2470,7 @@ load_vol_desc(TSK_FS_INFO * fs)
 
             /* boot records are just read and discarded for now... */
         case ISO9660_BOOT_RECORD:
+            free(vd);
 #if 0
             cnt = tsk_fs_read(fs, offs, (char *) b, sizeof(iso_bootrec));
             if (cnt != sizeof(iso_bootrec)) {
@@ -2422,6 +2484,10 @@ load_vol_desc(TSK_FS_INFO * fs)
             offs += sizeof(iso_bootrec);
 #endif
             break;
+
+        default:
+            free(vd);
+            break;
         }
     }
 
@@ -2487,6 +2553,13 @@ iso9660_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
         return NULL;
     }
 
+    if (img_info->sector_size == 0) {
+        tsk_error_reset();
+        tsk_error_set_errno(TSK_ERR_FS_ARG);
+        tsk_error_set_errstr("iso9660_open: sector size is 0");
+        return NULL;
+    }
+
     if (tsk_verbose) {
         tsk_fprintf(stderr, "iso9660_open img_info: %" PRIu64
             " ftype: %" PRIu8 " test: %" PRIu8 "\n", (uint64_t) img_info,
@@ -2584,7 +2657,7 @@ iso9660_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
 
     fs->first_block = 0;
     fs->last_block = fs->last_block_act = fs->block_count - 1;
-
+    
     // determine the last block we have in this image
     if ((TSK_DADDR_T) ((img_info->size - offset) / fs->block_size) <
         fs->block_count)
