@@ -79,7 +79,7 @@
  * file data into a TSK_FS_DIR structure.
  *
  * @param a_fs File system
- * @param a_fs_dir Structore to store file names into
+ * @param a_fs_dir Structure to store file names into
  * @param buf Buffer that contains the directory content
  * @param a_length Number of bytes in buffer
  * @param a_addr The metadata address for the directory being processed
@@ -87,7 +87,7 @@
  * @returns TSK_ERR on error and TSK_OK otherwise
  */
 static uint8_t
-iso9660_proc_dir(TSK_FS_INFO * a_fs, TSK_FS_DIR * a_fs_dir, char *buf,
+iso9660_proc_dir(TSK_FS_INFO * a_fs, TSK_FS_DIR * a_fs_dir, const char *buf,
     size_t a_length, TSK_INUM_T a_addr, TSK_OFF_T a_dir_addr)
 {
     ISO_INFO *iso = (ISO_INFO *) a_fs;
@@ -98,6 +98,12 @@ iso9660_proc_dir(TSK_FS_INFO * a_fs, TSK_FS_DIR * a_fs_dir, char *buf,
     iso9660_inode_node *in;
     TSK_OFF_T dir_offs = a_dir_addr * a_fs->block_size;
 
+    // had an issue once where dir was too small
+    // many later calculations assume we can fit at least one entry
+    if (a_length < sizeof(iso9660_dentry)) {      
+        return TSK_OK;
+    }
+
     if ((fs_name = tsk_fs_name_alloc(ISO9660_MAXNAMLEN + 1, 0)) == NULL)
         return TSK_ERR;
 
@@ -114,7 +120,6 @@ iso9660_proc_dir(TSK_FS_INFO * a_fs, TSK_FS_DIR * a_fs_dir, char *buf,
 
     buf_idx += dd->entry_len;
     if (buf_idx > a_length - sizeof(iso9660_dentry)) {
-        free(buf);
         tsk_fs_name_free(fs_name);
         return TSK_OK;
     }
@@ -142,7 +147,7 @@ iso9660_proc_dir(TSK_FS_INFO * a_fs, TSK_FS_DIR * a_fs_dir, char *buf,
         dd = (iso9660_dentry *) & buf[buf_idx];
 
         // process the entry (if it has a defined and valid length)
-        if ((dd->entry_len) && (buf_idx + dd->entry_len < a_length)) {
+        if ((dd->entry_len) && (buf_idx + dd->entry_len <= a_length)) {
 
             /* We need to find the data in the pre-processed list because that
              * contains the meta data address that TSK assigned to this file.
@@ -152,7 +157,7 @@ iso9660_proc_dir(TSK_FS_INFO * a_fs, TSK_FS_DIR * a_fs_dir, char *buf,
              * that had a file with 0 bytes with the same starting block as another
              * file. */
             for (in = iso->in_list; in; in = in->next) {
-                if (in->dentry_offset == dir_offs + buf_idx)
+                if (in->dentry_offset == dir_offs + (TSK_OFF_T)buf_idx)
                     break;
             }
 
@@ -195,7 +200,6 @@ iso9660_proc_dir(TSK_FS_INFO * a_fs, TSK_FS_DIR * a_fs_dir, char *buf,
         }
     }
 
-    free(buf);
     tsk_fs_name_free(fs_name);
 
     return TSK_OK;
@@ -205,20 +209,21 @@ iso9660_proc_dir(TSK_FS_INFO * a_fs, TSK_FS_DIR * a_fs_dir, char *buf,
 
 /** \internal
  * Process a directory and load up FS_DIR with the entries. If a pointer to
- * an already allocated FS_DIR struture is given, it will be cleared.  If no existing
+ * an already allocated FS_DIR structure is given, it will be cleared.  If no existing
  * FS_DIR structure is passed (i.e. NULL), then a new one will be created. If the return
  * value is error or corruption, then the FS_DIR structure could
- * have entries (depending on when the error occured).
+ * have entries (depending on when the error occurred).
  *
  * @param a_fs File system to analyze
  * @param a_fs_dir Pointer to FS_DIR pointer. Can contain an already allocated
  * structure or a new structure.
  * @param a_addr Address of directory to process.
+ * @param recursion_depth Recursion depth to limit the number of self-calls
  * @returns error, corruption, ok etc.
  */
 TSK_RETVAL_ENUM
 iso9660_dir_open_meta(TSK_FS_INFO * a_fs, TSK_FS_DIR ** a_fs_dir,
-    TSK_INUM_T a_addr)
+    TSK_INUM_T a_addr, int recursion_depth)
 {
     TSK_RETVAL_ENUM retval;
     TSK_FS_DIR *fs_dir;
@@ -279,12 +284,13 @@ iso9660_dir_open_meta(TSK_FS_INFO * a_fs, TSK_FS_DIR ** a_fs_dir,
         return TSK_ERR;
 
     cnt = tsk_fs_file_read(fs_dir->fs_file, 0, buf, length, 0);
-    if (cnt != length) {
+    if (cnt != (ssize_t)length) {
         if (cnt >= 0) {
             tsk_error_reset();
             tsk_error_set_errno(TSK_ERR_FS_READ);
         }
         tsk_error_set_errstr2("iso9660_dir_open_meta");
+        free(buf);
         return TSK_ERR;
     }
 
@@ -295,21 +301,26 @@ iso9660_dir_open_meta(TSK_FS_INFO * a_fs, TSK_FS_DIR ** a_fs_dir,
     // if we are listing the root directory, add the Orphan directory entry
     if (a_addr == a_fs->root_inum) {
         TSK_FS_NAME *fs_name = tsk_fs_name_alloc(256, 0);
-        if (fs_name == NULL)
+        if (fs_name == NULL) {
+            free(buf);
             return TSK_ERR;
+        }
 
         if (tsk_fs_dir_make_orphan_dir_name(a_fs, fs_name)) {
             tsk_fs_name_free(fs_name);
+            free(buf);
             return TSK_ERR;
         }
 
         if (tsk_fs_dir_add(fs_dir, fs_name)) {
             tsk_fs_name_free(fs_name);
+            free(buf);
             return TSK_ERR;
         }
         tsk_fs_name_free(fs_name);
     }
 
+    free(buf);
     return retval;
 }
 
