@@ -28,7 +28,6 @@
 #include <map>
 #include <vector>
 
-
 /** 
  * Class to hold the pair of MFT entry and sequence. 
  */
@@ -36,11 +35,13 @@ class NTFS_META_ADDR {
 private:
     uint64_t addr; ///< MFT entry
     uint32_t seq; ///< Sequence 
+    uint32_t hash; ///< Hash of the path
 
 public:
-    NTFS_META_ADDR(uint64_t a_addr, uint32_t a_seq) {
+    NTFS_META_ADDR(uint64_t a_addr, uint32_t a_seq, uint32_t a_hash) {
         addr = a_addr;
         seq = a_seq;
+        hash = a_hash;
     }
 
     uint64_t getAddr() {
@@ -50,6 +51,10 @@ public:
     uint32_t getSeq() {
         return seq;
     }
+
+    uint32_t getHash(){
+        return hash;
+    }
 };
 
 
@@ -71,8 +76,8 @@ public:
          * @param inum Address of child in the folder.
          * @param seq Sequence of child in the folder
          */
-        void add (uint32_t parSeq, TSK_INUM_T inum, uint32_t seq) {
-            NTFS_META_ADDR addr(inum, seq);
+        void add (uint32_t parSeq, TSK_INUM_T inum, uint32_t seq, uint32_t hash) {
+            NTFS_META_ADDR addr(inum, seq, hash);
             seq2addrs[parSeq].push_back(addr);
         }
 
@@ -119,7 +124,7 @@ static std::map<TSK_INUM_T, NTFS_PAR_MAP> * getParentMap(NTFS_INFO *ntfs) {
 /** \internal
  * Add a parent and child pair to the map stored in NTFS_INFO
  *
- * Note: This routine assumes &ntfs->orhpan_map_lock is locked by the caller.
+ * Note: This routine assumes &ntfs->orphan_map_lock is locked by the caller.
  *
  * @param ntfs structure to add the pair to
  * @param par Parent address
@@ -131,14 +136,14 @@ ntfs_parent_map_add(NTFS_INFO * ntfs, TSK_FS_META_NAME_LIST *name_list, TSK_FS_M
 {
     std::map<TSK_INUM_T, NTFS_PAR_MAP> *tmpParentMap = getParentMap(ntfs);
     NTFS_PAR_MAP &tmpParMap = (*tmpParentMap)[name_list->par_inode];
-    tmpParMap.add(name_list->par_seq, child_meta->addr, child_meta->seq);
+    tmpParMap.add(name_list->par_seq, child_meta->addr, child_meta->seq, tsk_fs_dir_hash(name_list->name));
     return 0;
 }
 
 /** \internal
  * Returns if a parent has children or not.
  *
- * Note: This routine assumes &ntfs->orhpan_map_lock is locked by the caller.
+ * Note: This routine assumes &ntfs->orphan_map_lock is locked by the caller.
  *
  * @param ntfs File system that has already been analyzed
  * @param par Parent inode to find child files for
@@ -161,7 +166,7 @@ ntfs_parent_map_exists(NTFS_INFO *ntfs, TSK_INUM_T par, uint32_t seq)
  * Look up a map entry by the parent address. You should call ntfs_parent_map_exists() before this, otherwise
  * an empty entry could be created. 
  *
- * Note: This routine assumes &ntfs->orhpan_map_lock is locked by the caller.
+ * Note: This routine assumes &ntfs->orphan_map_lock is locked by the caller.
  *
  * @param ntfs File system that has already been analyzed
  * @param par Parent inode to find child files for
@@ -206,17 +211,23 @@ ntfs_orphan_map_free(NTFS_INFO * a_ntfs)
 /* inode_walk callback that is used to populate the orphan_map
  * structure in NTFS_INFO */
 static TSK_WALK_RET_ENUM
-ntfs_parent_act(TSK_FS_FILE * fs_file, void *ptr)
+ntfs_parent_act(TSK_FS_FILE * fs_file, void * /*ptr*/)
 {
     NTFS_INFO *ntfs = (NTFS_INFO *) fs_file->fs_info;
     TSK_FS_META_NAME_LIST *fs_name_list;
 
+    if ((fs_file->meta->flags & TSK_FS_META_FLAG_ALLOC) &&
+        fs_file->meta->type == TSK_FS_META_TYPE_REG) {
+        ++ntfs->alloc_file_count;
+    }
+
     /* go through each file name structure */
     fs_name_list = fs_file->meta->name2;
     while (fs_name_list) {
         if (ntfs_parent_map_add(ntfs, fs_name_list,
-                fs_file->meta))
+                fs_file->meta)) {
             return TSK_WALK_ERROR;
+        }
         fs_name_list = fs_name_list->next;
     }
     return TSK_WALK_CONT;
@@ -236,6 +247,8 @@ ntfs_dent_copy(NTFS_INFO * ntfs, ntfs_idxentry * idxe,
     UTF8 *name8;
     int retVal;
 
+    tsk_fs_name_reset(fs_name);
+
     fs_name->meta_addr = tsk_getu48(fs->endian, idxe->file_ref);
     fs_name->meta_seq = tsk_getu16(fs->endian, idxe->seq_num);
 
@@ -273,6 +286,45 @@ ntfs_dent_copy(NTFS_INFO * ntfs, ntfs_idxentry * idxe,
 }
 
 
+/* Copy the short file name pointed to by idxe into fs_name.
+ * No other fields are copied.  Just the name into shrt_name. */
+static uint8_t
+ntfs_dent_copy_short_only(NTFS_INFO * ntfs, ntfs_idxentry * idxe,
+    TSK_FS_NAME * fs_name)
+{
+    ntfs_attr_fname *fname = (ntfs_attr_fname *) & idxe->stream;
+    TSK_FS_INFO *fs = (TSK_FS_INFO *) & ntfs->fs_info;
+    UTF16 *name16;
+    UTF8 *name8;
+    int retVal;
+
+    name16 = (UTF16 *) & fname->name;
+    name8 = (UTF8 *) fs_name->shrt_name;
+
+    retVal = tsk_UTF16toUTF8(fs->endian, (const UTF16 **) &name16,
+        (UTF16 *) ((uintptr_t) name16 +
+            fname->nlen * 2), &name8,
+        (UTF8 *) ((uintptr_t) name8 +
+            fs_name->shrt_name_size), TSKlenientConversion);
+
+    if (retVal != TSKconversionOK) {
+        *name8 = '\0';
+        if (tsk_verbose)
+            tsk_fprintf(stderr,
+                "Error converting NTFS 8.3 name to UTF8: %d %" PRIuINUM,
+                retVal, fs_name->meta_addr);
+    }
+
+    /* Make sure it is NULL Terminated */
+    if ((uintptr_t) name8 > (uintptr_t) fs_name->shrt_name + fs_name->shrt_name_size)
+        fs_name->shrt_name[fs_name->shrt_name_size] = '\0';
+    else
+        *name8 = '\0';
+
+    return 0;
+}
+
+
 
 
 /* This is a sanity check to see if the time is valid
@@ -321,9 +373,10 @@ ntfs_proc_idxentry(NTFS_INFO * a_ntfs, TSK_FS_DIR * a_fs_dir,
 {
     uintptr_t endaddr, endaddr_alloc;
     TSK_FS_NAME *fs_name;
+    TSK_FS_NAME *fs_name_preventry = NULL;
     TSK_FS_INFO *fs = (TSK_FS_INFO *) & a_ntfs->fs_info;
 
-    if ((fs_name = tsk_fs_name_alloc(NTFS_MAXNAMLEN_UTF8, 0)) == NULL) {
+    if ((fs_name = tsk_fs_name_alloc(NTFS_MAXNAMLEN_UTF8, 16)) == NULL) {
         return TSK_ERR;
     }
 
@@ -447,33 +500,53 @@ ntfs_proc_idxentry(NTFS_INFO * a_ntfs, TSK_FS_DIR * a_fs_dir,
                         "ntfs_proc_idxentry: Skipping because of invalid times\n");
                 continue;
             }
-        }
-
+        } 
         
 
         /* For all fname entries, there will exist a DOS style 8.3
-         * entry.  We don't process those because we already processed
-         * them before in their full version.  If the type is
-         * full POSIX or WIN32 that does not satisfy DOS, then a
-         * type NTFS_FNAME_DOS will exist.  If the name is WIN32,
-         * but already satisfies DOS, then a type NTFS_FNAME_WINDOS
-         * will exist
+         * entry.  
+         * If the original name is 8.3 compliant, it will be in
+         * a WINDOS type.  If it is not compliant, then it will 
+         * exist in a POSIX or WIN32 type and the 8.3 compliant
+         * one will be in DOS. The DOS entry typically follows
+         * the WIN32 or POSIX. 
          *
-         * Note that we could be missing some info from deleted files
+         * Our approach is to stash away the non-compliant names
+         * for one more entry to see if the next try is its 
+         * corresponding 8.3 entry. 
+         *
+         * If the 8.3 entry is not for the previous entry, we 
+         * skip it on the theory that it corresponds to a previous
+         * WIN32 or POSIX entry. Note that we could be missing some info from deleted files
          * if the windows version was deleted and the DOS wasn't...
-         *
-         * @@@ This should be added to the shrt_name entry of TSK_FS_NAME.  The short
-         * name entry typically comes after the long name
          */
 
         if (fname->nspace == NTFS_FNAME_DOS) {
-            if (tsk_verbose)
-                tsk_fprintf(stderr,
-                    "ntfs_proc_idxentry: Skipping because of name space: %d\n",
-                    fname->nspace);
+            // Was the previous entry not 8.3 compliant?
+            if (fs_name_preventry) {
+                // check its the same entry and if so, add short name
+                if (fs_name_preventry->meta_addr == tsk_getu48(fs->endian, a_idxe->file_ref)) {
+                    ntfs_dent_copy_short_only(a_ntfs, a_idxe, fs_name_preventry);
+                }
+
+                // regardless, add preventry to dir and move on to next entry.
+                if (tsk_fs_dir_add(a_fs_dir, fs_name_preventry)) {
+                    tsk_fs_name_free(fs_name);
+                    return TSK_ERR;
+                }
+                fs_name_preventry = NULL;
+            }
 
             goto incr_entry;
         }
+        // if we stashed the previous entry and the next wasn't a DOS entry, add it to the list
+        else if (fs_name_preventry) {
+            if (tsk_fs_dir_add(a_fs_dir, fs_name_preventry)) {
+                tsk_fs_name_free(fs_name);
+                return TSK_ERR;
+            }
+            fs_name_preventry = NULL;
+        }
 
         /* Copy it into the generic form */
         if (ntfs_dent_copy(a_ntfs, a_idxe, fs_name)) {
@@ -510,9 +583,17 @@ ntfs_proc_idxentry(NTFS_INFO * a_ntfs, TSK_FS_DIR * a_fs_dir,
                     tsk_getu16(fs->endian, a_idxe->idxlen)),
                 fs_name->flags);
 
-        if (tsk_fs_dir_add(a_fs_dir, fs_name)) {
-            tsk_fs_name_free(fs_name);
-            return TSK_ERR;
+        // WINDOS entries will not have a short 8.3 version, so add them now.
+        // otherwise, we stash the name to see if we get the 8.3 next. 
+        if (fname->nspace == NTFS_FNAME_WINDOS) {
+            if (tsk_fs_dir_add(a_fs_dir, fs_name)) {
+                tsk_fs_name_free(fs_name);
+                return TSK_ERR;
+            }
+            fs_name_preventry = NULL;
+        }
+        else {
+            fs_name_preventry = fs_name;
         }
 
       incr_entry:
@@ -542,6 +623,15 @@ ntfs_proc_idxentry(NTFS_INFO * a_ntfs, TSK_FS_DIR * a_fs_dir,
 
     }                           /* end of loop of index entries */
 
+    // final check in case we were looking for the short name, we never saw
+    if (fs_name_preventry) {
+        if (tsk_fs_dir_add(a_fs_dir, fs_name_preventry)) {
+            tsk_fs_name_free(fs_name);
+            return TSK_ERR;
+        }
+        fs_name_preventry = NULL;
+    }
+
     tsk_fs_name_free(fs_name);
     return TSK_OK;
 }
@@ -570,11 +660,20 @@ ntfs_fix_idxrec(NTFS_INFO * ntfs, ntfs_idxrec * idxrec, uint32_t len)
 
     /* sanity check so we don't run over in the next loop */
     if ((unsigned int) ((tsk_getu16(fs->endian, idxrec->upd_cnt) - 1) *
-            ntfs->ssize_b) > len) {
+            NTFS_UPDATE_SEQ_STRIDE) > len) {
         tsk_error_reset();
         tsk_error_set_errno(TSK_ERR_FS_INODE_COR);
         tsk_error_set_errstr
-            ("fix_idxrec: More Update Sequence Entries than idx record size");
+            ("ntfs_fix_idxrec: More Update Sequence Entries than idx record size");
+        return 1;
+    }
+
+    uint16_t upd_off = tsk_getu16(fs->endian, idxrec->upd_off);
+    if (upd_off > len || sizeof(ntfs_upd) > (len - upd_off)) {
+        tsk_error_reset();
+        tsk_error_set_errno(TSK_ERR_FS_INODE_COR);
+        tsk_error_set_errstr
+            ("ntfs_fix_idxrec: Corrupt idx record");
         return 1;
     }
 
@@ -590,7 +689,7 @@ ntfs_fix_idxrec(NTFS_INFO * ntfs, ntfs_idxrec * idxrec, uint32_t len)
     for (i = 1; i < tsk_getu16(fs->endian, idxrec->upd_cnt); i++) {
 
         /* The offset into the buffer of the value to analyze */
-        int offset = i * ntfs->ssize_b - 2;
+        int offset = i * NTFS_UPDATE_SEQ_STRIDE - 2;
         uint8_t *new_val, *old_val;
 
         /* get the current sequence value */
@@ -635,20 +734,21 @@ ntfs_fix_idxrec(NTFS_INFO * ntfs, ntfs_idxrec * idxrec, uint32_t len)
 
 /** \internal
 * Process a directory and load up FS_DIR with the entries. If a pointer to
-* an already allocated FS_DIR struture is given, it will be cleared.  If no existing
+* an already allocated FS_DIR structure is given, it will be cleared.  If no existing
 * FS_DIR structure is passed (i.e. NULL), then a new one will be created. If the return
 * value is error or corruption, then the FS_DIR structure could
-* have entries (depending on when the error occured).
+* have entries (depending on when the error occurred).
 *
 * @param a_fs File system to analyze
 * @param a_fs_dir Pointer to FS_DIR pointer. Can contain an already allocated
 * structure or a new structure.
 * @param a_addr Address of directory to process.
+* @param recursion_depth Recursion depth to limit the number of self-calls
 * @returns error, corruption, ok etc.
 */
 TSK_RETVAL_ENUM
 ntfs_dir_open_meta(TSK_FS_INFO * a_fs, TSK_FS_DIR ** a_fs_dir,
-    TSK_INUM_T a_addr)
+    TSK_INUM_T a_addr, int recursion_depth)
 {
     NTFS_INFO *ntfs = (NTFS_INFO *) a_fs;
     TSK_FS_DIR *fs_dir;
@@ -659,7 +759,6 @@ ntfs_dir_open_meta(TSK_FS_INFO * a_fs, TSK_FS_DIR ** a_fs_dir,
     ntfs_idxroot *idxroot;
     ntfs_idxelist *idxelist;
     ntfs_idxrec *idxrec_p, *idxrec;
-    int off;
     TSK_OFF_T idxalloc_len;
     TSK_FS_LOAD_FILE load_file;
 
@@ -765,12 +864,7 @@ ntfs_dir_open_meta(TSK_FS_INFO * a_fs, TSK_FS_DIR ** a_fs_dir,
         return TSK_COR;
     }
 
-    /* Get the header of the index entry list */
-    idxelist = &idxroot->list;
-
-    /* Get the offset to the start of the index entry list */
-    idxe = (ntfs_idxentry *) ((uintptr_t) idxelist +
-        tsk_getu32(a_fs->endian, idxelist->begin_off));
+    
 
     /*
      * NTFS does not have "." and ".." entries in the index trees
@@ -845,12 +939,15 @@ ntfs_dir_open_meta(TSK_FS_INFO * a_fs, TSK_FS_DIR ** a_fs_dir,
             "ntfs_dir_open_meta: Processing $IDX_ROOT of inum %" PRIuINUM
             "\n", a_addr);
 
+    /* Get the header of the index entry list */
+    idxelist = &idxroot->list;
+
     /* Verify the offset pointers */
     if ((tsk_getu32(a_fs->endian, idxelist->seqend_off) <
             tsk_getu32(a_fs->endian, idxelist->begin_off)) ||
         (tsk_getu32(a_fs->endian, idxelist->bufend_off) <
             tsk_getu32(a_fs->endian, idxelist->seqend_off)) ||
-        (((uintptr_t) idxe + tsk_getu32(a_fs->endian,
+        (((uintptr_t) idxelist + tsk_getu32(a_fs->endian,
                     idxelist->bufend_off)) >
             ((uintptr_t) fs_attr_root->rd.buf +
                 fs_attr_root->rd.buf_size))) {
@@ -862,6 +959,10 @@ ntfs_dir_open_meta(TSK_FS_INFO * a_fs, TSK_FS_DIR ** a_fs_dir,
         return TSK_COR;
     }
 
+    /* Get the offset to the start of the index entry list */
+    idxe = (ntfs_idxentry *) ((uintptr_t) idxelist +
+        tsk_getu32(a_fs->endian, idxelist->begin_off));
+
     retval_tmp = ntfs_proc_idxentry(ntfs, fs_dir,
         (fs_dir->fs_file->meta->flags & TSK_FS_META_FLAG_UNALLOC) ? 1 : 0,
         idxe,
@@ -901,6 +1002,7 @@ ntfs_dir_open_meta(TSK_FS_INFO * a_fs, TSK_FS_DIR ** a_fs_dir,
         }
     }
     else {
+        int off;
 
         if (fs_attr_idx->flags & TSK_FS_ATTR_RES) {
             tsk_error_reset();
@@ -965,6 +1067,16 @@ ntfs_dir_open_meta(TSK_FS_INFO * a_fs, TSK_FS_DIR ** a_fs_dir,
         for (off = 0; off < idxalloc_len; off += ntfs->csize_b) {
             uint32_t list_len, rec_len;
 
+            // Ensure that there is enough data for an idxrec
+            if (sizeof(ntfs_idxrec) > idxalloc_len - off) {
+                tsk_error_reset();
+                tsk_error_set_errno(TSK_ERR_FS_INODE_COR);
+                tsk_error_set_errstr
+                    ("ntfs_dir_open_meta: Not enough data in idxalloc buffer for an idxrec.");
+                free(idxalloc);
+                return TSK_COR;
+            }
+
             idxrec = (ntfs_idxrec *) & idxalloc[off];
 
             if (tsk_verbose)
@@ -973,7 +1085,7 @@ ntfs_dir_open_meta(TSK_FS_INFO * a_fs, TSK_FS_DIR ** a_fs_dir,
                     PRIx32 "\n", off, tsk_getu32(a_fs->endian,
                         idxrec->magic));
 
-            /* Is this the begining of an index record? */
+            /* Is this the beginning of an index record? */
             if (tsk_getu32(a_fs->endian,
                     idxrec->magic) != NTFS_IDXREC_MAGIC)
                 continue;
@@ -1060,8 +1172,8 @@ ntfs_dir_open_meta(TSK_FS_INFO * a_fs, TSK_FS_DIR ** a_fs_dir,
 
             /* Length from end of attribute to start of this */
             rec_len =
-                (uint32_t) (idxalloc_len - (uintptr_t) idxrec_p -
-                (uintptr_t) idxalloc);
+                (uint32_t) (idxalloc_len - ((uintptr_t) idxrec_p -
+                (uintptr_t) idxalloc));
 
             if (tsk_verbose)
                 tsk_fprintf(stderr,
@@ -1075,6 +1187,16 @@ ntfs_dir_open_meta(TSK_FS_INFO * a_fs, TSK_FS_DIR ** a_fs_dir,
             }
 
             idxelist = &idxrec_p->list;
+            if (tsk_getu32(a_fs->endian, idxelist->begin_off) > rec_len) {
+                tsk_error_reset();
+                tsk_error_set_errno(TSK_ERR_FS_INODE_COR);
+                tsk_error_set_errstr
+                    ("Error: Index list offsets are invalid on entry: %"
+                    PRIuINUM, fs_dir->fs_file->meta->addr);
+                free(idxalloc);
+                return TSK_COR;
+            }
+
             idxe = (ntfs_idxentry *) ((uintptr_t) idxelist +
                 tsk_getu32(a_fs->endian, idxelist->begin_off));
 
@@ -1166,6 +1288,16 @@ ntfs_dir_open_meta(TSK_FS_INFO * a_fs, TSK_FS_DIR ** a_fs_dir,
         for (size_t a = 0; a < childFiles.size(); a++) {
             TSK_FS_FILE *fs_file_orp = NULL;
 
+            /* Check if fs_dir already has an allocated entry for this
+             * file.  If so, ignore it. We used to rely on fs_dir_add
+             * to get rid of this, but it wasted a lot of lookups. If 
+             * We have only unalloc for this same entry (from idx entries),
+             * then try to add it.   If we got an allocated entry from
+             * the idx entries, then assume we have everything. */
+            if (tsk_fs_dir_contains(fs_dir, childFiles[a].getAddr(), childFiles[a].getHash()) == TSK_FS_NAME_FLAG_ALLOC) {
+                continue;
+            }
+
             /* Fill in the basics of the fs_name entry
              * so we can print in the fls formats */
             fs_name->meta_addr = childFiles[a].getAddr();
@@ -1275,7 +1407,6 @@ ntfs_find_file_rec(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
     uint8_t decrem = 0;
     size_t len = 0, i;
     char *begin = NULL;
-    int retval;
 
 
     if (fs_name_list->par_inode < fs->first_inum ||
@@ -1299,9 +1430,10 @@ ntfs_find_file_rec(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
      * - The parent is no longer a directory
      * - The sequence number of the parent is no longer correct
      */
-    if ((fs_file_par->meta->type != TSK_FS_META_TYPE_DIR)
+    if (( ! TSK_FS_IS_DIR_META(fs_file_par->meta->type))
         || (fs_file_par->meta->seq != fs_name_list->par_seq)) {
         const char *str = TSK_FS_ORPHAN_STR;
+        int retval;
         len = strlen(str);
 
         /* @@@ There should be a sanity check here to verify that the
@@ -1407,7 +1539,7 @@ ntfs_find_file_rec(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
  * @param dir_walk_flags Flags to use during search
  * @param action Callback that will be called for each name that uses the specified addresses.
  * @param ptr Pointer that will be passed into action when it is called (so that you can pass in other data)
- * @returns 1 on error, 0 on sucess
+ * @returns 1 on error, 0 on success
  */
 
 uint8_t
@@ -1567,7 +1699,7 @@ ntfs_find_file(TSK_FS_INFO * fs, TSK_INUM_T inode_toid, uint32_t type_toid,
 
 
 int
-ntfs_name_cmp(TSK_FS_INFO * a_fs_info, const char *s1, const char *s2)
+ntfs_name_cmp(TSK_FS_INFO * /*a_fs_info*/, const char *s1, const char *s2)
 {
     return strcasecmp(s1, s2);
 }
