@@ -23,7 +23,11 @@
 // the hash, so there are 2 ^ 12 or 4096 possible entries.
 static const size_t IDX_IDX_ENTRY_COUNT = 4096;
 static const size_t IDX_IDX_SIZE = IDX_IDX_ENTRY_COUNT * sizeof(uint64_t);
-static const uint64_t IDX_IDX_ENTRY_NOT_SET = 0xFFFFFFFFFFFFFFFF;
+#if defined( __BORLANDC__ ) && ( __BORLANDC__ < 0x0560 )
+static const uint64_t IDX_IDX_ENTRY_NOT_SET = 0xFFFFFFFFFFFFFFFFUL;
+#else
+static const uint64_t IDX_IDX_ENTRY_NOT_SET = 0xFFFFFFFFFFFFFFFFULL;
+#endif
 
 
 /**
@@ -205,7 +209,7 @@ static uint8_t
                 tsk_error_reset();
                 tsk_error_set_errno(TSK_ERR_HDB_OPEN);
                 tsk_error_set_errstr(
-                    "%s: error opening index of index: %"PRIttocTSK" - %d",
+                    "%s: error opening index of index: %" PRIttocTSK" - %d",
                     func_name, hdb_binsrch_info->idx_idx_fname, (int)GetLastError());
                 return 1;
         }
@@ -215,7 +219,7 @@ static uint8_t
             tsk_error_reset();
             tsk_error_set_errno(TSK_ERR_HDB_OPEN);
             tsk_error_set_errstr(
-                "%s: error converting file handle from Windows to C for: %"PRIttocTSK, 
+                "%s: error converting file handle from Windows to C for: %" PRIttocTSK, 
                 func_name, hdb_binsrch_info->idx_idx_fname);
             return 1;
         }
@@ -226,7 +230,7 @@ static uint8_t
             tsk_error_reset();
             tsk_error_set_errno(TSK_ERR_HDB_OPEN);
             tsk_error_set_errstr(
-                "%s: error getting size of index of index file: %"PRIttocTSK" - %d",
+                "%s: error getting size of index of index file: %" PRIttocTSK" - %d",
                 func_name, hdb_binsrch_info->idx_idx_fname, (int)GetLastError());
             return 1;
         }
@@ -245,7 +249,7 @@ static uint8_t
             tsk_error_reset();
             tsk_error_set_errno(TSK_ERR_HDB_OPEN);
             tsk_error_set_errstr(
-                "%s: error opening index of index: %"PRIttocTSK,
+                "%s: error opening index of index: %" PRIttocTSK,
                 func_name, hdb_binsrch_info->idx_idx_fname);
             return 1;
         }
@@ -286,6 +290,9 @@ static uint8_t
 * @param hash The hash type that was used to make the index.
 *
 * @return 1 on error and 0 on success
+* @@@ NOTE: When an error is returned, hdb_info_base could 
+*  be in an inconsistent state.  Some variables could be set and others
+*  are not, depending on what kind of error was detected.
 */
 static uint8_t 
     hdb_binsrch_open_idx_file(TSK_HDB_INFO *hdb_info_base, TSK_HDB_HTYPE_ENUM htype)
@@ -321,7 +328,7 @@ static uint8_t
             tsk_error_reset();
             tsk_error_set_errno(TSK_ERR_HDB_MISSING);
             tsk_error_set_errstr(
-                "hdb_binsrch_open_idx_file: Error finding index file: %"PRIttocTSK,
+                "hdb_binsrch_open_idx_file: Error finding index file: %" PRIttocTSK,
                 hdb_binsrch_info->idx_fname);
             return 1;
         }
@@ -333,7 +340,7 @@ static uint8_t
                 tsk_error_reset();
                 tsk_error_set_errno(TSK_ERR_HDB_OPEN);
                 tsk_error_set_errstr(
-                    "hdb_binsrch_open_idx: Error opening index file: %"PRIttocTSK,
+                    "hdb_binsrch_open_idx: Error opening index file: %" PRIttocTSK,
                     hdb_binsrch_info->idx_fname);
                 return 1;
         }
@@ -350,11 +357,14 @@ static uint8_t
 
         szLow = GetFileSize(hWin, &szHi);
         if (szLow == 0xffffffff) {
+            fclose(hdb_binsrch_info->hIdx);
+            hdb_binsrch_info->hIdx = NULL;
+
             tsk_release_lock(&hdb_binsrch_info->base.lock);
             tsk_error_reset();
             tsk_error_set_errno(TSK_ERR_HDB_OPEN);
             tsk_error_set_errstr(
-                "hdb_binsrch_open_idx_file: Error getting size of index file: %"PRIttocTSK" - %d",
+                "hdb_binsrch_open_idx_file: Error getting size of index file: %" PRIttocTSK" - %d",
                 hdb_binsrch_info->idx_fname, (int)GetLastError());
             return 1;
         }
@@ -389,6 +399,9 @@ static uint8_t
 
     /* Do some testing on the first line */
     if (NULL == fgets(head, TSK_HDB_MAXLEN, hdb_binsrch_info->hIdx)) {
+        fclose(hdb_binsrch_info->hIdx);
+        hdb_binsrch_info->hIdx = NULL;
+
         tsk_release_lock(&hdb_binsrch_info->base.lock);
         tsk_error_reset();
         tsk_error_set_errno(TSK_ERR_HDB_READIDX);
@@ -399,6 +412,9 @@ static uint8_t
 
     if (strncmp(head, TSK_HDB_IDX_HEAD_TYPE_STR, strlen(TSK_HDB_IDX_HEAD_TYPE_STR))
         != 0) {
+            fclose(hdb_binsrch_info->hIdx);
+            hdb_binsrch_info->hIdx = NULL;
+
             tsk_release_lock(&hdb_binsrch_info->base.lock);
             tsk_error_reset();
             tsk_error_set_errno(TSK_ERR_HDB_UNKTYPE);
@@ -409,6 +425,9 @@ static uint8_t
 
     /* Do some testing on the second line */
     if (NULL == fgets(head2, TSK_HDB_MAXLEN, hdb_binsrch_info->hIdx)) {
+        fclose(hdb_binsrch_info->hIdx);
+        hdb_binsrch_info->hIdx = NULL;
+
         tsk_release_lock(&hdb_binsrch_info->base.lock);
         tsk_error_reset();
         tsk_error_set_errno(TSK_ERR_HDB_READIDX);
@@ -441,6 +460,9 @@ static uint8_t
     if (strcmp(ptr, TSK_HDB_DBTYPE_NSRL_STR) == 0) {
         if ((hdb_binsrch_info->base.db_type != TSK_HDB_DBTYPE_NSRL_ID) &&
             (hdb_binsrch_info->base.db_type != TSK_HDB_DBTYPE_IDXONLY_ID)) {
+                fclose(hdb_binsrch_info->hIdx);
+                hdb_binsrch_info->hIdx = NULL;
+
                 tsk_release_lock(&hdb_binsrch_info->base.lock);
                 tsk_error_reset();
                 tsk_error_set_errno(TSK_ERR_HDB_UNKTYPE);
@@ -453,6 +475,9 @@ static uint8_t
     else if (strcmp(ptr, TSK_HDB_DBTYPE_MD5SUM_STR) == 0) {
         if ((hdb_binsrch_info->base.db_type != TSK_HDB_DBTYPE_MD5SUM_ID) &&
             (hdb_binsrch_info->base.db_type != TSK_HDB_DBTYPE_IDXONLY_ID)) {
+                fclose(hdb_binsrch_info->hIdx);
+                hdb_binsrch_info->hIdx = NULL;
+
                 tsk_release_lock(&hdb_binsrch_info->base.lock);
                 tsk_error_reset();
                 tsk_error_set_errno(TSK_ERR_HDB_UNKTYPE);
@@ -465,6 +490,9 @@ static uint8_t
     else if (strcmp(ptr, TSK_HDB_DBTYPE_HK_STR) == 0) {
         if ((hdb_binsrch_info->base.db_type != TSK_HDB_DBTYPE_HK_ID) &&
             (hdb_binsrch_info->base.db_type != TSK_HDB_DBTYPE_IDXONLY_ID)) {
+                fclose(hdb_binsrch_info->hIdx);
+                hdb_binsrch_info->hIdx = NULL;
+
                 tsk_release_lock(&hdb_binsrch_info->base.lock);
                 tsk_error_reset();
                 tsk_error_set_errno(TSK_ERR_HDB_UNKTYPE);
@@ -477,6 +505,9 @@ static uint8_t
     else if (strcmp(ptr, TSK_HDB_DBTYPE_ENCASE_STR) == 0) {
         if ((hdb_binsrch_info->base.db_type != TSK_HDB_DBTYPE_ENCASE_ID) &&
             (hdb_binsrch_info->base.db_type != TSK_HDB_DBTYPE_IDXONLY_ID)) {
+                fclose(hdb_binsrch_info->hIdx);
+                hdb_binsrch_info->hIdx = NULL;
+
                 tsk_release_lock(&hdb_binsrch_info->base.lock);
                 tsk_error_reset();
                 tsk_error_set_errno(TSK_ERR_HDB_UNKTYPE);
@@ -487,6 +518,9 @@ static uint8_t
         }
     }
     else if (hdb_binsrch_info->base.db_type != TSK_HDB_DBTYPE_IDXONLY_ID) {
+        fclose(hdb_binsrch_info->hIdx);
+        hdb_binsrch_info->hIdx = NULL;
+
         tsk_release_lock(&hdb_binsrch_info->base.lock);
         tsk_error_reset();
         tsk_error_set_errno(TSK_ERR_HDB_UNKTYPE);
@@ -499,6 +533,9 @@ static uint8_t
     /* Do some sanity checking */
     if (((hdb_binsrch_info->idx_size - hdb_binsrch_info->idx_off) % hdb_binsrch_info->idx_llen) !=
         0) {
+            fclose(hdb_binsrch_info->hIdx);
+            hdb_binsrch_info->hIdx = NULL;
+
             tsk_release_lock(&hdb_binsrch_info->base.lock);
             tsk_error_reset();
             tsk_error_set_errno(TSK_ERR_HDB_CORRUPT);
@@ -509,6 +546,9 @@ static uint8_t
 
     /* allocate a buffer for a row */
     if ((hdb_binsrch_info->idx_lbuf = (char*)tsk_malloc(hdb_binsrch_info->idx_llen + 1)) == NULL) {
+        fclose(hdb_binsrch_info->hIdx);
+        hdb_binsrch_info->hIdx = NULL;
+
         tsk_release_lock(&hdb_binsrch_info->base.lock);
         return 1;
     }
@@ -541,9 +581,6 @@ uint8_t
     if (hdb_binsrch_open_idx_file(hdb_info_base, htype))
     {
         tsk_release_lock(&hdb_binsrch_info->base.lock);
-        tsk_error_reset();
-        tsk_error_set_errno(TSK_ERR_HDB_ARG);
-        tsk_error_set_errstr("hdb_binsrch_open_idx: unable to open index file");
         return 1;
     }
 
@@ -680,7 +717,7 @@ uint8_t
                 tsk_error_reset();
                 tsk_error_set_errno(TSK_ERR_HDB_CREATE);
                 tsk_error_set_errstr(
-                    "%s: %"PRIttocTSK" GetFileSize: %d",
+                    "%s: %" PRIttocTSK" GetFileSize: %d",
                     func_name, hdb_binsrch_info->uns_fname, (int)GetLastError());
                 return 1;
         }
@@ -740,6 +777,8 @@ uint8_t
 
 /**
 * Add a string entry to the intermediate index file.
+* Will not add an all-zero hash since this creates errors in the final
+* index file, but does not return an error in this case.
 *
 * @param hdb_binsrch_info Hash database state info
 * @param hvalue String of hash value to add
@@ -750,8 +789,22 @@ uint8_t
     hdb_binsrch_idx_add_entry_str(TSK_HDB_BINSRCH_INFO *hdb_binsrch_info, char *hvalue, TSK_OFF_T offset)
 {
     int i;
+    int found_non_zero_char = 0;
 
-    // make the hashes all upper case
+    /* Check if the hash is all-zero, and skip it if it is. This is extremely unlikely to be a real hash, and
+     * causes problems with sorting the index file because we use an all zero entry as a special header
+     * value */
+    for (i = 0; hvalue[i] != '\0'; i++) {
+        if (hvalue[i] != '0') {
+            found_non_zero_char = 1;
+            break;
+        }
+    }
+    if ( ! found_non_zero_char) {
+        return 0;
+    }
+
+    /* make the hashes all upper case. */
     for (i = 0; hvalue[i] != '\0'; i++) {
         if (islower((int) hvalue[i]))
             fprintf(hdb_binsrch_info->hIdxTmp, "%c", toupper((int) hvalue[i]));
@@ -827,7 +880,7 @@ static uint8_t
                 tsk_error_reset();
                 tsk_error_set_errno(TSK_ERR_HDB_CREATE);
                 tsk_error_set_errstr(
-                    "%s: error creating index of index file %"PRIttocTSK" - %d)",
+                    "%s: error creating index of index file %" PRIttocTSK" - %d)",
                     func_name, hdb_binsrch_info->idx_idx_fname, winErrNo);
                 return 1;
         }
@@ -838,7 +891,7 @@ static uint8_t
             tsk_error_reset();
             tsk_error_set_errno(TSK_ERR_HDB_OPEN);
             tsk_error_set_errstr(
-                "%s: error converting file handle from Windows to C for: %"PRIttocTSK, 
+                "%s: error converting file handle from Windows to C for: %" PRIttocTSK, 
                 func_name, hdb_binsrch_info->idx_idx_fname);
             return 1;
         }
@@ -848,14 +901,14 @@ static uint8_t
         tsk_error_reset();
         tsk_error_set_errno(TSK_ERR_HDB_CREATE);
         tsk_error_set_errstr(
-            "%s: error creating index of index file %"PRIttocTSK,
+            "%s: error creating index of index file %" PRIttocTSK,
             func_name, hdb_binsrch_info->idx_idx_fname);
         return 1;
     }
 #endif
 
     // Allocate an array to hold the starting offsets in the index file for each 
-    // set of hashes with identical intitial (3) nibbles.
+    // set of hashes with identical initial (3) nibbles.
     hdb_binsrch_info->idx_offsets = (uint64_t*)tsk_malloc(IDX_IDX_SIZE);
     if (NULL == hdb_binsrch_info->idx_offsets) {
         return 1;
@@ -867,6 +920,7 @@ static uint8_t
     // corresponding set.
     TSK_OFF_T idx_off = hdb_binsrch_info->idx_off;
     char digits[4];
+    digits[3] = '\0';
     long int offsets_idx = 0;
     while (fgets(hdb_binsrch_info->idx_lbuf, (int)hdb_binsrch_info->idx_llen + 1, hdb_binsrch_info->hIdx)) {
         strncpy(digits, hdb_binsrch_info->idx_lbuf, 3);		
@@ -907,11 +961,8 @@ uint8_t
     hdb_binsrch_info->idx_size = 0;
     hdb_binsrch_info->idx_off = 0;
     hdb_binsrch_info->idx_llen = 0;
-    if (hdb_binsrch_info->idx_lbuf != NULL)
-    {
-        free(hdb_binsrch_info->idx_lbuf);
-        hdb_binsrch_info->idx_lbuf = NULL;
-    }
+    free(hdb_binsrch_info->idx_lbuf);
+    hdb_binsrch_info->idx_lbuf = NULL;
 
     if (tsk_verbose)
         tsk_fprintf(stderr, "hdb_idxfinalize: Sorting index\n");
@@ -1086,12 +1137,20 @@ int8_t
     if (hdb_binsrch_open_idx(hdb_info_base, htype))
         return -1;
 
-    /* Sanity check */
+    /* Sanity checks */
     if (hdb_binsrch_info->hash_len != strlen(hash)) {
         tsk_error_reset();
         tsk_error_set_errno(TSK_ERR_HDB_ARG);
         tsk_error_set_errstr(
-            "%s: Hash passed is different size than expected (%d vs %Zd)",
+            "%s: Hash passed is different size than expected (%d vs %zd)",
+            func_name, hdb_binsrch_info->hash_len, strlen(hash));
+        return -1;
+    }
+    else if (hdb_binsrch_info->idx_llen == 0) {
+        tsk_error_reset();
+        tsk_error_set_errno(TSK_ERR_HDB_CORRUPT);
+        tsk_error_set_errstr(
+            "%s: Error: Index line length is zero",
             func_name, hdb_binsrch_info->hash_len, strlen(hash));
         return -1;
     }
@@ -1202,7 +1261,7 @@ int8_t
             tsk_error_reset();
             tsk_error_set_errno(TSK_ERR_HDB_READIDX);
             tsk_error_set_errstr(
-                "hdb_lookup: Error seeking in search: %" PRIuOFF,
+                "hdb_lookup: Error seeking in search: %" PRIdOFF,
                 offset);
             return -1;
         }
@@ -1236,7 +1295,7 @@ int8_t
                 return -1;
         }
 
-        /* Set the delimter to NULL so we can treat the hash as a string */
+        /* Set the delimiter to NULL so we can treat the hash as a string */
         hdb_binsrch_info->idx_lbuf[hdb_binsrch_info->hash_len] = '\0';
         cmp = strcasecmp(hdb_binsrch_info->idx_lbuf, ucHash);
 
@@ -1299,7 +1358,7 @@ int8_t
                         tsk_error_set_errno(TSK_ERR_HDB_READIDX);
                         tsk_error_set_errstr(
                             "hdb_lookup: Error seeking for prev entries: %"
-                            PRIuOFF, tmpoff);
+							PRIdOFF, tmpoff);
                         return -1;
                     }
 
@@ -1361,7 +1420,7 @@ int8_t
                         tsk_error_set_errno(TSK_ERR_HDB_READIDX);
                         tsk_error_set_errstr(
                             "hdb_lookup: Error seeking for next entries: %"
-                            PRIuOFF, tmpoff);
+							PRIdOFF, tmpoff);
                         return -1;
                     }
 
@@ -1524,10 +1583,8 @@ void
         hdb_info->hDb = NULL;
     }
 
-    if (hdb_info->idx_fname) {
-        free(hdb_info->idx_fname);
-        hdb_info->idx_fname = NULL;
-    }
+    free(hdb_info->idx_fname);
+    hdb_info->idx_fname = NULL;
 
     if (hdb_info->hIdx) {
         fclose(hdb_info->hIdx);
@@ -1539,20 +1596,14 @@ void
         hdb_info->hIdxTmp = NULL;
     }
 
-    if (hdb_info->uns_fname) {
-        free(hdb_info->uns_fname);
-        hdb_info->uns_fname = NULL;
-    }
+    free(hdb_info->uns_fname);
+    hdb_info->uns_fname = NULL;
 
-    if (hdb_info->idx_lbuf) {
-        free(hdb_info->idx_lbuf);
-        hdb_info->idx_lbuf = NULL;
-    }
+    free(hdb_info->idx_lbuf);
+    hdb_info->idx_lbuf = NULL;
 
-    if (hdb_info->idx_offsets) {
-        free(hdb_info->idx_offsets);
-        hdb_info->idx_offsets = NULL;
-    }
+    free(hdb_info->idx_offsets);
+    hdb_info->idx_offsets = NULL;
 
     hdb_info_base_close(hdb_info_base);
 
