@@ -144,15 +144,23 @@ mac_load_table(TSK_VS_INFO * vs)
         }
 
         strncpy(str, (char *) part->type, sizeof(part->name));
+        str[sizeof(part->name) - 1] = 0;
 
         if (NULL == tsk_vs_part_add(vs, (TSK_DADDR_T) part_start,
                 (TSK_DADDR_T) part_size, (TSK_VS_PART_FLAG_ENUM)flag, str, -1,
-                idx))
+                idx)) {
+            free(part_buf);
             return 1;
+        }
     }
     free(part_buf);
     part_buf = NULL;
 
+    // Bail if we didn't find any valid entries
+    if (vs->part_count == 0) {
+        return 1;
+    }
+
     /* Add an entry for the table length */
     if ((table_str = tsk_malloc(16)) == NULL) {
         return 1;
@@ -188,6 +196,13 @@ tsk_vs_mac_open(TSK_IMG_INFO * img_info, TSK_DADDR_T offset)
     // clean up any errors that are lying around
     tsk_error_reset();
 
+    if (img_info->sector_size == 0) {
+        tsk_error_reset();
+        tsk_error_set_errno(TSK_ERR_VS_ARG);
+        tsk_error_set_errstr("tsk_vs_mac_open: sector size is 0");
+        return NULL;
+    }
+
     vs = (TSK_VS_INFO *) tsk_malloc(sizeof(*vs));
     if (vs == NULL)
         return NULL;
@@ -201,7 +216,7 @@ tsk_vs_mac_open(TSK_IMG_INFO * img_info, TSK_DADDR_T offset)
 
     //vs->sect_offset = offset + MAC_PART_OFFSET;
 
-    /* inititialize settings */
+    /* initialize settings */
     vs->part_list = NULL;
     vs->part_count = 0;
     vs->endian = 0;
